/**
 * This is the main AJAX function.
 * Calls a URL on the server and returns the answer. The answer should
 * be either "OK" or "ERROR", followed by a newline and then an arbitrary
 * number of lines with result values or messages. Make sure the scripts
 * you call on the server answer this way if you want to use the
 * filterResult(), writeResult() etc. functions.
 *
 * @param urlToCall the URL which to call on the server
 * @return the server return value to the call (should be a string with
 *   either "OK" or "ERROR", followed by a newline and an arbitrary message.
 */
function getUrl ( urlToCall ) {
	var oHttp;
	if ( window.XMLHttpRequest ) {  // Gecko
		oHttp = new XMLHttpRequest();
		oHttp.open( "GET", urlToCall, false ) ;
		oHttp.send( null );
		return oHttp.responseText ;
	}
	else {
		oHttp = new ActiveXObject("MsXML2.XmlHttp");
		oHttp.open( "GET", urlToCall, false ) ;
		oHttp.send();
		return oHttp.responseText ;
	}
}

/**
 * Breaks down a server answer to an AJAX call. Expects the answer to be
 * "OK" or "ERROR", followed by a newline and then arbitrary messages.
 *
 * @param result the server answer
 * @result an array with two elements: { "OK"/"ERROR", the messages }
 */
function filterResult( reply ) {
	if ( !reply.length || reply.length < 1 ) return new Array( "", "" );
	var index = reply.indexOf("\n");
	var res = "";
	var msg = "";
	if ( index < 0 ) res = reply;
	else {
		res = reply.substring( 0, index );
		if ( index >= reply.length ) msg = "";
		else msg = reply.substring( index+1 );
	}
	return new Array( res, msg );
}

/**
 * Breaks down the server answer to an AJAX call and writes the message
 * to the user in a dialog window (using alert()).
 *
 * @param reply the server answer
 */
function writeReply ( reply ) {
	var arr = filterResult( reply );
	var res = arr[0];
	var msg = arr[1];
	if ( res == "ERROR" && msg.length < 1 )
		msg = "An error has occured.\nYour action was probably not successful.";
	if ( msg.length > 0 ) alert( msg );
}

/**
 * Breaks down the server answer to an AJAX call. If the result is an ERROR
 * message it is written to the user (using alert()).
 *
 * @param reply the server answer
 */
function writeReplyOnError ( reply ) {
	var arr = filterResult( reply );
	var res = arr[0];
	var msg = arr[1];
	if ( res == "ERROR" ) {
		if ( msg.length < 1 )
			msg = "An error has occured.\nYour action was probably not successful.";
		alert( msg );
	}
}

