inherit "/classes/Script";
inherit "/base/xml_data";

#include <macros.h>
#include <database.h>
#include <access.h>
#include <attributes.h>
#include <types.h>
#include <classes.h>
#include <events.h>
#include <config.h>

#define _XML _Server->get_module("Converter:XML")

#define RES(e,g) ([ "german":g, "english":e, ])
#define RES_MSG(e,g,l) RES(e,g)[l]

string show_error(mixed err)
{
    string str = master()->describe_backtrace(err[1]);
    str = replace(str, ({ "\r\n", "\n", "<", ">" }),
                       ({ "<BR/>", "<BR/>", "&lt;", "&gt;" }) );
    return str;
}

int gettime() { return time(); }

string execute(mapping vars)
{
    object                            obj, user;
    string                                 html;
    string                                  xml;
    mixed                            err, m_err;
    string action = vars["_action"];

    mapping client_map = identify_browser(
    vars["__internal"]["client"],
    vars["__internal"]["request_headers"]);
    string client_prefix = "";
    if ( client_map->language != "english" &&
         client_map->language != "german" )
             client_map["language"] = "english";

    user = this_user();

    string l = user->query_attribute(USER_LANGUAGE);
    if (!stringp(l)) l = client_map->language;
    client_map->language = l;
//werror("admin.pike : language l="+l);

    user = this_user();

    try_event(EVENT_EXECUTE, CALLER, this_object());

    object aktport=0;

    switch ( action ) { 
    case "clear_security_cache":
      object hkmodule = _Server->get_module("housekeeping");
      hkmodule->clear_security_cache();
      return result_page( "Security Cache cleared.", "/scripts/admin.pike?mode=showtools&nonavi=true");
      break;
    case "recover_users":
      hkmodule = _Server->get_module("housekeeping");
      hkmodule->recover_users();
      break;
    case "get_broken_users":
      string ret = "";
      hkmodule = _Server->get_module("housekeeping");
      array busers = hkmodule->get_null_users();
      foreach(busers, object user) {
        if (objectp(user)) {
          ret += user->get_object_id()+"/"+user->get_identifier()+"/";
        }
      }
      return ret;
      break;
    case "change_quotas":
	object quota = _Server->get_module("quota");
	html = "<ul>";
	foreach(indices(vars), string idx ) {
	    int grp_id;
	    if ( sscanf(idx, "quota_%d", grp_id) > 0 ) {
		object group = find_object(grp_id);
		if ( objectp(group) ) {
		    quota->set_quota(group, (int)vars[idx]);
		    html += "<li> Quota of Group " + group->get_identifier()+
			" set to " + vars[idx] + "</li>\n";
		}
	    }
	}
	html += "</ul>";
	return result_page(html, "JavaScript:history.back();");
	break;
    case "errorlog":
      string logerr = _Server->get_stderr()->read();
      if ( !stringp(logerr) )
	return result_page("Logfile not found !", "JavaScript:history.back();");
      html = replace(logerr, "\n", "<br/>");
      return result_page(html, "JavaScript:history.back();");
      break;
    case "messagelog":
      return result_page(_Server->get_stdout()->read(), "");
      break;
    case "show_errors":
	mapping errors = master()->get_errors();
	html = "<ul>";
	foreach( indices(errors), string file) {
	    html += "<li>"+file+"<BR/> ";
	    foreach(errors[file], string error_desc) {
		html += error_desc+"<BR/>";
	    }
	    html += "</li>";
	}
	html += "</ul>";
	return result_page(html, "JavaScript:history.back();");
	break;
    case "restart_port":
      html = "";
      aktport = find_port(vars);
      if (objectp(aktport)) {
        if (_Server->restart_port(aktport))
          html += RES_MSG("Restarted "+vars["portname"]+"-daemon on port "+vars["oldportnumber"]+" succesfully",
                          vars["portname"]+"-Daemon auf Port "+vars["oldportnumber"]+" wurde erfolgreich neu gestartet",l);
        else
          html += RES_MSG("Error restarting "+vars["portname"]+" port",
                           "Fehler beim Neustart des "+vars["portname"]+"-Ports",l);
      } else {
        html += RES_MSG("Error while restarting port. Cant find corresponding port object: name="+vars["portname"]+" portnr="+vars["oldportnumber"],
                        "Fehler beim Neustart des Ports. Port Objekt konnte nicht gefunden werden. Name="+vars["portname"]+" Portnr="+vars["oldportnumber"],l);
      }
	return result_simplepage(html, RES_MSG("Port restarted", "Port neu gestartet", l), l);
	break;
    case "start_port":
      html = "";
      if (stringp(vars["portname"])) {
        if (vars["newportnumber"] =="") vars["newportnumber"]="0";
        if (_Server->open_port(vars["portname"]))
          html += RES_MSG(vars["portname"]+"-daemon started succesfully",
                          vars["portname"]+"-Daemon wurde erfolgreich gestartet",l);
        else
          html += RES_MSG("Error starting "+vars["portname"]+" port",
                           "Fehler beim Start des "+vars["portname"]+"-Ports",l);
      } else {
        html += RES_MSG("Error while starting port. No portname specified",
                        "Fehler beim Starten des Ports. Kein Portname angegeben",l);
      }
	return result_simplepage(html, RES_MSG("Port restarted", "Port neu gestartet", l), l);
	break;
    case "stop_port":
      html = "";
      aktport = find_port(vars);
      if (objectp(aktport)) {
        if (_Server->close_port(aktport))
          html += RES_MSG("Stopped "+vars["portname"]+"-daemon on port "+vars["oldportnumber"]+" succesfully",
                          vars["portname"]+"-Daemon auf Port "+vars["oldportnumber"]+" wurde angehalten",l);
        else
          html += RES_MSG("Error stopping "+vars["portname"]+" port",
                           "Fehler beim Stoppen des "+vars["portname"]+"-Ports",l);
      } else {
        html += RES_MSG("Error stopping port. Cant find corresponding port object: name="+vars["portname"]+" portnr="+vars["oldportnumber"],
                        "Fehler beim Stoppen des Ports. Port Objekt konnte nicht gefunden werden. Name="+vars["portname"]+" Portnr="+vars["oldportnumber"],l);
      }
	return result_simplepage(html, RES_MSG("Port stopped", "Port gestoppt", l), l);
	break;
    case "delete_language":
        html = "";
// set the values of the mapping and update new language information
          mapping mLangs = GROUP("Admin")->query_attribute("multilanguage_languages");

          if (!mappingp(mLangs)) {
            return error_page(RES_MSG("Cant find supported languages at admin group:" +
			       " attribute multilanguage_languages not set on group Admin. (found no mapping)",
			       "Unterstuetzte Sprachen konnten nicht gefunden werden. " +
			       "Attribut multilanguage_languages ist nicht gesetzt bzw enthaelt kein mapping",l),
                         "JavaScript:history.back();");
          }
        // delete translations
            foreach(indices(vars), string skey) {
              string sourcelang;
              string targetlang;
              if ( sscanf(skey, "translation_%s_in_%s", sourcelang, targetlang) == 2 ) {
                m_delete(mLangs[sourcelang]["translations"], targetlang);  // delete translation
              }
            }
        // delete language itself
            m_delete(mLangs, vars["oldlanguagename"]);

          GROUP("Admin")->set_attribute("multilanguage_languages", mLangs);                 // set new language information
          html += RES_MSG("Language deleted", "Sprache wurde entfernt", l);
 	    return result_simplepage(html, RES_MSG("Language deleted", "Sprache wurde entfernt", l), l);
	break;
    case "update_languagedetails":
          html = "";
// set the values of the mapping and update new language information
          mapping mLanguages = GROUP("Admin")->query_attribute("multilanguage_languages");

          if (!mappingp(mLanguages)) {
            if (stringp(vars["oldlanguagename"]) && vars["oldlanguagename"]=="newlanguage") mLanguages = ([]); // create new mapping
            else return error_page(RES_MSG("Cant find supported languages at admin group:" +
			       " attribute multilanguage_languages not set on group Admin. (found no mapping)",
			       "Unterstuetzte Sprachen konnten nicht gefunden werden. " +
			       "Attribut multilanguage_languages ist nicht gesetzt bzw enthaelt kein mapping",l),
                         "JavaScript:history.back();");
          }

          if (stringp(vars["oldlanguagename"]) && vars["oldlanguagename"]=="newlanguage") { // adding a new language
            if (vars["newlanguagename"]=="newlanguage")
              return result_simplepage(RES_MSG("Invalid name for the new language", "Sie muessen den Namen aendern um eine neue Sprache hinzuzufuegen",l), RES_MSG("Invalid name for the new language", "Sie muessen den Namen aendern um eine neue Sprache hinzuzufuegen", l), l);

            mLanguages[vars["newlanguagename"]] = ([]);  // initialize mapping
            mLanguages[vars["newlanguagename"]]["extension"] = vars["newlanguageextension"];
            mLanguages[vars["newlanguagename"]]["translations"] = ([]);  // initialize translations- mapping

            foreach(indices(vars), string skey) {
              string sourcelang;
              string targetlang;
              if ( sscanf(skey, "translation_%s_in_%s", sourcelang, targetlang) == 2 ) {
                 if (sourcelang=="newlanguage" && targetlang=="newlanguage") mLanguages[vars["newlanguagename"]]["translations"][vars["newlanguagename"]] = vars[skey];  // set new translations
                 else {
                   if (sourcelang=="newlanguage") mLanguages[vars["newlanguagename"]]["translations"][targetlang] = vars[skey];  // set new translations
                   else                           mLanguages[sourcelang]["translations"][vars["newlanguagename"]] = vars[skey];  // set new translations
                 }
              }
            }

            GROUP("Admin")->set_attribute("multilanguage_languages", mLanguages);                 // set new language information
            html += RES_MSG("Added language succesfully", "Neue Sprache wurde angelegt", l);
            return result_simplepage(html, RES_MSG("Added language succesfully", "Neue Sprache wurde angelegt", l), l);
          }
          else {           // editing old languages
            html += RES_MSG("Language details updates", "Spracheinstellungen wurden uebernommen", l);
            mLanguages[vars["oldlanguagename"]]["extension"] = vars["newlanguageextension"];
            foreach(indices(vars), string skey) {
              string sourcelang;
              string targetlang;
              if ( sscanf(skey, "translation_%s_in_%s", sourcelang, targetlang) == 2 ) {
                 mLanguages[sourcelang]["translations"][targetlang] = vars[skey];  // set new translations
              }
            }
          }
          GROUP("Admin")->set_attribute("multilanguage_languages", mLanguages);                 // set new language information
 	    return result_simplepage(html, RES_MSG("Language details updated", "Spracheinstellungen wurden uebernommen", l), l);
	break;
    case "update_portdetails":
      html = "";
      aktport = find_port(vars);
      if (objectp(aktport)) {
        _Server->set_config(vars["relatedconfig"], (int)vars["newportnumber"]);
        if (_Server->restart_port(aktport))
          html += RES_MSG("Successfully updated port details.",
                          "Die Einstellungen wurden uebernommen.",l);
        else
          html += RES_MSG("Error updating port details. Error restarting port on portnr "+vars["newportnumber"],
                          "Fehler beim Übernehmen der Porteinstellungen. Der Port konnte nicht auf Port Nr. "+vars["newportnumber"]+" neu gestartet werden",l);
      } else {
        html += RES_MSG("Error updating port details. Cant find corresponding port object: name="+vars["portname"]+" portnr="+vars["oldportnumber"],
                        "Fehler beim Uebernehmen der neuen Werte. Port Objekt konnte nicht gefunden werden. Name="+vars["portname"]+" Portnr="+vars["oldportnumber"],l);
      }
	return result_simplepage(html, RES_MSG("Configuration updated", "Einstellungen wurden uebernommen", l), l);
	break;
    default:
	object xsl = _FILEPATH->path_to_object("/stylesheets/admin.xsl");


	xml = _XML->get_xml(this(), xsl, vars, 0);
	if ( vars["source"] == "true" ) {
      return xml;
//	    xml = (((xml/"<")*"&lt;")/">")*"&gt;";
//	    return "<html><body><pre>\n"+xml+"\n</pre></body></html>\n";
	}
	vars["host"] = _Server->query_config(CFG_WEBSERVER);
	vars["port_ftp"] = (string)_Server->query_config(CFG_WEBPORT_FTP);
	vars["port_http"] = (string)_Server->query_config(CFG_WEBPORT_HTTP);
	vars["user_name"] = (string)(objectp(user)?user->get_identifier():
				     "none");
	vars["version"] = STEAM_VERSION;
	vars["user_id"] = (string)(objectp(user) ? user->get_object_id(): "0");
	vars["selection"] = (string)(objectp(user)? (
	    arrayp(user->query_attribute("USER_SELECTION"))?
	    sizeof(user->query_attribute("USER_SELECTION")):"0") : "0");
	vars->inmemory = (string)master()->get_in_memory();
	vars->swapped  = (string)master()->get_swapped();
	if(objectp(get_module("package:web")))
		vars->webversion = get_module("package:web")->get_version();
	else
		vars->webversion = "interface version is kept secret by Tom";
	vars->libxsltversion = get_module("libxslt")->get_version();

         vars |= client_map;

      	html = _Server->get_module("libxslt")->run(xml, xsl, vars);
	break;
    }
    run_event(EVENT_EXECUTE, CALLER, this_object());
    return html; // "<html>"+html+"</html>\n";
}

object find_port(mapping vars) {
  array ports = master()->get_ports();
  object aktport = 0;
  foreach (ports, object port) {
    if (port->get_port_name()==vars["portname"] && port->get_port()==(int)vars["oldportnumber"])
      aktport = port;
  }
  return aktport;
}

string get_debug_xml()
{
  string        xml = "";

  mapping debug = _Server->debug_memory();

  xml += "<memory>";
  foreach(indices(debug), string idx) {
    if ( search(idx, "bytes") >= 0 ) continue;
    string bid = idx[4..strlen(idx)-2]+"_bytes";
    xml += "<"+idx[4..]+"><num>"+debug[idx] + "</num><bytes>"+
      (debug[bid]/1024)+"</bytes></"+idx[4..]+">\n";
  }
  xml += "</memory>";
  int uptime = time()-_Server->get_last_reboot();
  xml += "<requests><overall>"+
    get_module("log")->get_requests()+"</overall><permin>"+
    (get_module("log")->get_requests()*60/uptime)+
    "</permin></requests>\n";
  xml += "<download><bytes>"+(get_module("log")->get_download()/1024)+
    "</bytes><permin>"+(get_module("log")->get_download()*60/(uptime*1024))+
    "</permin></download>";

  xml += "<call_outs>";
  foreach( call_out_info(), array co ) {
    function f;
    if ( sizeof(co) < 4 ) continue;
    f = co[3];
    if ( !functionp(f) )
      continue;
    xml += "<callout><function>"+function_name(f)+"</function>"+
      (functionp(function_object(f)->get_object_id) ?
       get_module("Converter:XML")->show_object_ext(function_object(f)):
       "<object><name>---</name></object>")+
      "<id>"+co[0]+"</id><t>"+co[0]+"</t></callout>\n";
  }
  xml += "</call_outs>";

  xml += "<threads>";
#if constant(Thread)
  foreach(Thread.all_threads(), object thread) {
      if ( !objectp(thread) )
	  xml += "<thread/>";
      else {
	  function f = thread->backtrace()[0]->fun;
	  if ( function_name(f) == "run_thread" )
	      f = thread->backtrace()[0]->args[0];

	  object fobj = function_object(f);
	  xml += "<thread><id>"+thread->id_number()+"</id><function>"+
	      master()->describe_function(f)+"</function>"+
	      "<status>"+thread->status()+"</status></thread>";
      }
  }
#endif
  xml += "</threads>";

  xml += "<users>";
  foreach( master()->get_users(), object u) {
    if ( !objectp(u) ) continue;
    object uo = u->get_user_object();
    xml += "<user><type>"+u->describe()+"</type>";
    if ( functionp(u->get_idle) )
	xml += "<idle>"+u->get_idle()+"</idle>";
    if ( objectp(uo) )
      xml += get_module("Converter:XML")->show_object_ext(uo);
    xml += "</user>\n";
  }
  xml += "</users>";

  xml += "<gc>" + debug_gc() + "</gc>\n";
    
  return xml;
}

string debug_memory_time()
{
  mapping objs, mems, reqs;
  string xml = "";

  objs = get_module("log")->get_objects();
  mems = get_module("log")->get_memory();
  reqs = get_module("log")->get_request_map();
  foreach(indices(objs), int tid) {
    xml += "<time><stamp>"+tid+"</stamp><steam_objects>" +
      objs[tid] + "</steam_objects><requests>"+
      reqs[tid] + "</requests>";
    xml += "<objects>"+mems[tid]->num_objects + "</objects>";
    xml += "<mappings>"+mems[tid]->num_mappings + "</mappings>";
    xml += "<arrays>"+mems[tid]->num_arrays + "</arrays>";
    xml += "<strings>"+mems[tid]->num_strings + "</strings>";
    xml += "</time>\n";
  }
  return xml;
}

int compareports(object p1, object p2)
{
  return lower_case(p1->get_port_name()) >
         lower_case(p2->get_port_name());
}

string show_port_details()
{
    array ports = master()->get_ports();
    // umsortieren
    ports = Array.sort_array(ports, compareports);

    string xml = "";
    xml += "<count>"+sizeof(ports)+"</count>";
    foreach(ports, object port) {
//        xml += "<port nr='"+port->get_port()+"'>"+port->describe()+"</port>";
      xml += "<port nr='"+port->get_port()+"' name='"+port->get_port_name()+"'>"+
             "<description>"+port->describe()+"</description>"+
             "<name>"+port->get_port_name()+"</name>"+
             "<configname>"+port->get_port_config()+"</configname>"+
             "</port>";
    }
    return xml;
}

string result_simplepage(string phtml, string title, string l) {
  string html = "";
  html += "<html><head><title>"+title+"</title>";
  html += "<SCRIPT language=\"javascript\" type=\"text/javascript\">function reload_parent() { opener.location.reload(); }</SCRIPT>";
  html += "</head><body bgcolor=\"#D7D7D7\">";
  html += "<SCRIPT language=\"javascript\" type=\"text/javascript\">window.resizeTo(480, 125); window.moveTo(screen.width/2 - document.body.clientWidth/2, screen.height /2 - document.body.clientHeight/2);</SCRIPT>";
  html += phtml;
  html += "<BR/><BR/><DIV ALIGN=\"CENTER\"><INPUT TYPE=\"BUTTON\" VALUE=\""+RES_MSG("Close this window","Fenster schliessen", l)+"\" onClick=\"JavaScript:reload_parent();JavaScript:window.close()\"/></DIV>";
  html += "</body></html>";
  return html;
}

string debug_gc()
{
    string xml = "";
#if constant(Debug)
    mapping res = Debug.gc_status();
    foreach(indices(res), string idx) {
	if ( idx == "last_gc" )
	    res[idx] = get_module("Converter:XML")->get_time(res[idx]);
	xml += sprintf("<%s>%O</%s>\n", idx, res[idx], idx);
    }
#endif
    return xml;
}
