//! An implementation of the ANNOTEA protocol

inherit "/classes/Script";

static string presentation = _Server->get_server_url_presentation();
static string administration = _Server->get_server_url_presentation();

#include <macros.h>
#include <database.h>
#include <classes.h>
#include <attributes.h>
#include <access.h>
#include <exception.h>
#include <roles.h>

#define RDF_HEADER "<r:RDF xmlns:r=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" "+\
"xmlns:a=\"http://www.w3.org/2000/10/annotation-ns#\" "+\
"xmlns:d=\"http://purl.org/dc/elements/1.1/\">\r\n"

string 
compose_xml(object site)
{
    string  xml = "";
    LOG("site is " + (objectp(site) ? site->get_object_id():"not available"));
    if ( !objectp(site) ) return "<annotations></annotations>\n";
    array(object) annotations = site->get_annotations();
    
    if ( !arrayp(annotations) || sizeof(annotations) == 0 ) 
	return "<annotations></annotations>\n";

    foreach(annotations, object annotation) {
	if ( !objectp(annotation) ) continue;
	mapping annotate = annotation->query_annotation();
	xml += "<annotation>\n";
	foreach(indices(annotate), string key) {
	    if ( objectp(annotate[key]) )
		xml += "\t<"+key+">"+annotate[key]->get_identifier()+
		    "</"+key+">\n";
	    else if ( stringp(annotate[key]) || intp(annotate[key]) )
		xml += "\t<"+key+">"+annotate[key]+"</"+key+">\n";
	}
	xml += "</annotation>\n";
    }
    return xml +"\n";
}

object annotate(string site, mapping meta, mapping content, void|int ann)
{
    object          annotation;
    LOG("Content="+sprintf("%O", mkmapping(indices(content), values(content))));
    if ( ann == 0 ) {
	object factory = _Server->get_factory(CLASS_DOCEXTERN);
	object docExtern = factory->execute((["url": site, ]));
	factory = _Server->get_factory(CLASS_DOCUMENT);
	annotation = factory->execute( ([
	    "name":"annotation", 
	    "mimetype": 
	    (mappingp(content["http:ContentType"]) ?
	     content["http:ContentType"]["content"]:"text/html"), ]) );
	docExtern->add_annotation(annotation->this());
    }
    else
	annotation = find_object(ann);

    annotation->set_content(content["http:Body"]["content"]);
//    annotation->set_attribute(OBJ_CREATION_TIME, meta["a:created"]);
    annotation->set_attribute("a:created", meta["a:created"]);
    annotation->set_attribute("a:context", meta["a:context"]);
    annotation->set_attribute("a:annotates", meta["a:annotates"]);
    annotation->set_attribute("r:type", meta["r:type"]);
    
    annotation->sanction_object(_WORLDUSER, SANCTION_READ);
    return annotation;
}

array(object) query_annotations(string site)
{
    object   document;
    string       page;
    object         fp;

    if ( search(site, presentation) == 0 ) {
	sscanf(site, presentation+"%s", page);
	fp = get_module("filepath:url");
    }
    else if ( search(site, administration) == 0 ) {
	sscanf(site, administration+"%s", page);
	fp = get_module("filepath:tree");
    }
    else
	return ({ });

    if ( !stringp(page) || strlen(page) == 0 )
	page = "/";

    // fixme: use filepath
    document = fp->path_to_object(page);
    if ( !objectp(document) )
	return ({ });
    
    _SECURITY->check_access(document, this_user(), 
			    SANCTION_READ,ROLE_READ_ALL, false);

    return document->get_annotations_for();
}

string rdf_date(int t)
{
    string ti = Calendar.Second(t)->format_mtime();
    return replace(ti, " ", "T") + "+01:00";
}

string describe_annotation_rdf(string site, object annotation) 
{
    string           xml;
    mapping meta = ([ ]);

    meta["d:creator"] = annotation->get_creator()->get_identifier();
    meta["d:title"] = annotation->get_identifier();
    meta["a:context"] = site+"#xpointer(/html[1])";
    meta["a:created"] = rdf_date(
	annotation->query_attribute(OBJ_CREATION_TIME));
    meta["d:date"] = rdf_date(time());


    string resource = 
	presentation+"Annotation?object="+annotation->get_object_id();

    xml = "<r:Description r:about=\""+resource+"\">\r\n"+
	"<r:type r:resource=\"http://www.w3.org/2000/10/annotation-ns#Annotation\"/>\r\n"+
	"<r:type r:resource=\"http://www.w3.org/2000/10/annotationType#Comment\"/>\r\n"+
	"<a:body r:resource=\""+resource+"\"/>\r\n"+
	"<a:annotates r:resource=\""+site+"\"/>\r\n";
    
    foreach ( indices(meta), string a_smth ) {
	xml += "<"+a_smth+">"+meta[a_smth]+"</"+a_smth+">\r\n";
    }
    xml += "</r:Description>";
    return xml;
}


array|string execute(mapping vars)
{
    string url, host, page, xml;
    if ( vars->w3c_annotates ) {
	array annotations = query_annotations(vars->w3c_annotates);
	string xml = "<?xml version='1.0' encoding='iso-8859-1'?>\n";
	
	xml += RDF_HEADER;
	foreach ( annotations, object ann ) {
	    xml += describe_annotation_rdf(vars->w3c_annotates, ann);
	}
	
	xml += "</r:RDF>\n";
	return ({ xml, "application/xml" });
     }
    if ( vars->object ) {
	return find_object((int)vars->object)->get_content();
    }
}

