/* Copyright (C) 2000-2005  Thomas Bopp, Thorsten Hampel, Ludger Merkens
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later Fsearchversion.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */ 
   
inherit "/classes/Script";

import httplib;

#include <macros.h>
#include <database.h>
#include <access.h>
#include <attributes.h>
#include <classes.h>
#include <config.h>
#include <events.h>
#include <types.h>
#include <roles.h>

#define _XMLCONVERTER _Server->get_module("Converter:XML")
#define _SEARCH_KEYWORD _Server->get_module("index:keywords")
#define _SEARCH_NAME _Server->get_module("index:names")

static array(object) aoSelected;
static array(object)    aSearch;
object _SEARCH = _Server->get_module("searching");


array(string) __sanction = _SECURITY->get_sanction_strings();
static string sKeywords = "";

static mapping downloads = ([ ]);
static mapping mCopy     = ([ ]);
static mapping mSearch   = ([ ]);
static mapping mTrash    = ([ ]);

static void
restart()
{
    LOG("Restarting server (I am browser.pike)");
    _Server->shutdown(1);
}

string
redirection_url(string resMessage, string errMessage, mapping vars, object ctx)
{
    string script = "navigate.pike";
    if ( stringp(vars["_script"]) )
        script = vars["_script"];
    else {
      if (objectp(ctx)) {
        string buf_script = _FILEPATH->object_to_filename(ctx);
        // check if path is valid...
        if (stringp(buf_script) && (search(buf_script, "/void") != 0))
          script = httplib.replace_uml(buf_script);
      }
    }
    LOG("Redirecting to " + script);
    if ( stringp(errMessage) )
        errMessage = replace(errMessage, " ", "%20");
    if ( !stringp(resMessage) )
      resMessage = "Error";
    resMessage = replace(resMessage, " ", "%20");
    string url;
    if ( stringp(vars->id) && strlen(vars->id) > 0 )
        url = script + "?object="+vars["id"]+"&";
    else
        url = script + "?";
    url += (stringp(vars["type"]) ? "type="+vars["type"]+"&" : "")+
        (stringp(vars["active"]) ? "active="+vars["active"]+"&" : "")+
        (stringp(vars["mode"]) ? "mode="+vars["mode"]+"&" : "")+
        (stringp(vars["room"]) && vars->room != "0" && strlen(vars->room) > 0 ?
         "room="+vars["room"]+"&" : "")+
        (stringp(vars["jump"]) && vars["jump"] != "" ? "jump="+vars["jump"]+"&" : "")+
        "message="+resMessage+(stringp(errMessage) ? "&error="+errMessage:"")+
        (stringp(vars["jump"]) && vars["jump"] != "" ? "#"+vars["jump"] : "");
    return url;
}


string userdetails(object user)
{
    return "<a href=\"#\" onClick=\"JavaScript:open_userdetails('"+
        user->get_object_id()+"');\">"+user->get_identifier()+"</a>";
}

/**
 *
 *
 * @param
 * @return
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>)
 * @see
 */
array get_values(string|array v)
{
    array res;

    if ( !arrayp(v) )
    res = v / "\0";
    else
    res = v;

    array result = ({ });

    foreach ( res, string str ) {
    if ( stringp(str) && strlen(str) > 0 )
        result += ({ get_value(str) });
    }
    return result;
}

/**
 *
 *
 * @param
 * @return
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>)
 * @see
 */
array(object) get_selected(mapping vars, array(string)|void sels)
{
    array(object) selected = ({ });

    array(string) index = ({ });
    if ( !arrayp(sels) ) sels = ({ "objsel", "roomsel", "gateobjsel" });

    if ( stringp(vars["popup_id"]) && strlen(vars["popup_id"]) > 0 &&
    (int)vars["popup_id"] != 0 )
    return ({ find_object((int)vars["popup_id"]) });

    foreach ( sels, string sel ) {
    if ( stringp(vars[sel]) )
        index += (vars[sel]/"\0");
    else if ( arrayp(vars[sel]) ) {
      index += vars[sel];
    }
    }

    foreach(index, string key) {
	int     oid;
	object  obj;
	if ( sscanf(key, "%d", oid) == 1 && objectp(obj=find_object(oid)) )
	{
	    selected += ({ obj });
	}
    }
    return selected;
}

/**
 *
 *
 * @param
 * @return
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>)
 * @see
 */
array(object) get_selection(object user, object env)
{
    array(object) sel = user->query_attribute(USER_SELECTION);
    return sel;
}


array(object) get_search_result()
{
    return aSearch;
}

string get_search_keywords()
{
    return sKeywords;
}

/**
 *
 *
 * @param
 * @return
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>)
 * @see
 */
static string
html_from_list(array(object) aoSelection, array(object) aoSearch, mixed vars,
           object xsl)
{
    string sxml, html;
    object parser, obj;
    object user = this_user();
    mixed err;


    aSearch = aoSearch;
    sKeywords = copy_value(vars["keywords"]);
    m_delete(vars, "keywords");
    if ( !stringp(vars["_context"]) ) {
      if ( vars->room ) {
	object room = _FILEPATH->path_to_object(vars->room);
	if ( objectp(room) )
	  vars["_context"] = (string)(room->get_object_id());
        }
      else
	vars["_context"]=(string)(objectp(user)?user->get_object_id():"0");
    }
    sxml = _XMLCONVERTER->get_xml(this(), xsl, vars, 0);
    err = catch{
      string shtml = run_xml(sxml, xsl, vars);
      return shtml;
    };
    if (err) {
      string emergency =
	"<html><body><p>Error during html generation</p><p>"+
	err[0]+
	"</p><p>an emergency listing is given below<br/></p><table>";
      foreach(aSearch, object o)
	if (objectp(o))
	  emergency+=sprintf("<tr><td>%s</td><td><a href=\"/scripts/navigate.pike?object=%d\">%s</a></td><td><a href=\"/scripts/navigate.pike?object=%d\">environment</a></tr>", o->get_identifier(), o->get_object_id(), o->get_identifier(), (o->get_environment() ? o->get_environment()->get_object_id() : 0));
/* debugging only
      emergency += sprintf("<tr><td colspan='2'>XML<br/>%O</td></tr>",
			   replace(sxml, ({ "<", ">", "&" }), ({ "&lt;", "&gt;", "&amp;"})));
      emergency += sprintf("<tr><td colspan='2'>VARS<br/><hr/>%O</td></tr>",
			   vars);
*/         
      emergency+="</table></body></html>";
      return emergency;
    }
}

static string
html_from_object(object obj, mixed vars, object xsl)
{
    string   xml, html;

    html = run_xml(obj, xsl, vars);
    return "<html>\n"+html + "</html>\n<!---\n"+xml+"\n-->\n";
}


bool check_filter(object obj, string filter, mixed val)
{
    if ( !objectp(obj) )
    return false;

    if ( !stringp(val) || strlen(val) == 0 )
    return true;

    switch( filter ) {
    case "filter_mimetype":
    string mt = obj->query_attribute(DOC_MIME_TYPE);
    if ( stringp(mt) ) {
        string fmt, smt, cfmt, csmt;

        sscanf(mt, "%s/%s", fmt, smt);
        sscanf(val, "%s/%s", cfmt, csmt);
        if ( (cfmt == "*" || cfmt == fmt) &&
         (csmt == "*" || csmt == smt) )
        return true;
        return false;
    }

    if ( obj->query_attribute(DOC_MIME_TYPE) != val )
        return false;
    return true;
    case "filter_minfilesize":
    if ( !(obj->get_object_class() & CLASS_DOCUMENT) )
        return false;
    if ( obj->get_content_size() > (int)val )
        return true;
    return false;
    break;
    case "filter_maxfilesize":
    if ( !(obj->get_object_class() & CLASS_DOCUMENT) )
        return false;
    if ( obj->get_content_size() < (int)val )
        return true;
    return false;
    break;
    case "filter_class":
    if ( val == "all" )
        return true;
    if ( val ==
         _Server->get_factory(obj->get_object_class())->get_class_name() )
        return true;
    return false;
    break;
    }
    return true;
}

bool check_filters(object obj, mapping vars)
{
    foreach(indices(vars), string f) {
    if ( !check_filter(obj, f, vars[f]) )
        return false;
    }
    return true;
}

array filter_search(array results, mapping vars)
{
    array res = ({ });
    foreach(results, object o) {
      if ( !catch(_SECURITY->check_access(o, this_object(), SANCTION_READ, ROLE_READ_ALL, false)) && check_filters(o, vars) )
            res += ({ o });
    }
    return res;
}

string generate_password()
{
    string pw = "        ";
    for ( int i = 0; i < 8; i++ ) {
    pw[i] = random(26) + 97;
    }
    return pw;
}

string get_html_status(object created)
{
    string created_html = "<b>Object " + created->get_identifier()+
    " </b><BR/><BR/>";
    created_html += "Events:<BR/><ul>";
    mapping new_events;
    if ( functionp(created->get_my_events) )
      new_events = created->get_my_events();
    if (mappingp(new_events)) {
      foreach(indices(new_events), int event) {
        foreach(new_events[event], object event_obj) {
            created_html += "<li> Registered event ["+event+"] on "+
            event_obj->get_identifier() + " ("+
            event_obj->get_object_id() +") </li>";
        }
      }
    }
    created_html += "</ul>\n";
    created_html += "<BR/><BR/><b>"+
    "name="+created->query_attribute(OBJ_NAME)+",creator="+
    created->get_creator()->get_identifier()+",desc="+
    created->query_attribute(OBJ_DESC)+"</b><BR/><BR/>";
    created_html += "ACL:<BR/><ul>";
    mapping new_sanction = created->get_sanction();
    foreach(indices(new_sanction), object sanctioned_obj) {
    created_html += "<li> " +
        sanctioned_obj->get_identifier() + " is " +
        _Server->get_module("Converter:XML")->
        get_role(new_sanction[sanctioned_obj]) + "</li>";
    }
    created_html += "</ul>";

    return result_page(created_html, "JavaScript:history.back();");
}


#define RES(e,g) ([ "german":g, "english":e, "chinese":e, ])
#define RES_MSG(e,g,l) RES(e,g)[l]



mixed execute(mapping vars)
{
    object                  ann, xsl, obj, user;
    array(object)                       uSelect;
    string                            xml, html;
    bool                                 result;
    mixed                                   err;
    string                               action;
    mapping                          resMessage;
    mapping                          errMessage;
    mixed                                   key;
    object                              factory;
    object                                 regs;
    int                                       i;
    object the_room, room;
    
    try_event(EVENT_EXECUTE, CALLER, this_object());

    mapping client_map =
      identify_browser(
                       vars["__internal"]["client"],
                       vars["__internal"]["request_headers"]);

    string client_prefix = "";
    
    user = this_user();
    string l = user->query_attribute(USER_LANGUAGE);
    if ( stringp(l) )
      client_map->language = l;
    else
      l = client_map->language;


    aoSelected = get_selected(vars);


    int id = (int)vars["id"];
    object context = find_object(id);

    action = vars["_action"];
    if ( !stringp(action) )
        action = "select";

    resMessage = RES("action successful", "Aktion erfolgreich");
    errMessage = 0; 

    object tmod = _Server->get_module("tasks");

    object FORWARD = _Server->get_module("forward");  // define for all email- configuration cases

    switch(action) {
  case "set_loglevel":
    object logmodule = _Server->get_module("log");
    logmodule->set_log_level("http", (int)vars["http"]);
    logmodule->set_log_level("smtp", (int)vars["smtp"]);
    logmodule->set_log_level("events", (int)vars["events"]);
    logmodule->set_log_level("security", (int)vars["security"]);
    break;
	case "mailbox_action":
    int move_to_id;
    if(sscanf(vars["do"], "move_to_folder_%d", int id)){
      vars["do"] = "move_to_folder";
      move_to_id = id;
    }
		switch((string)vars["do"]) {
		   case "read":
				object read = get_module("table:read-documents");
				foreach(aoSelected, object osel){
					if(objectp(osel)) set_read(osel, read);
				}
				int mc = sizeof(aoSelected);
				resMessage = RES(mc+" mails marked as read", mc+" Nachrichten als gelesen markiert");
				break;
			case "move_to_folder":
				foreach(aoSelected, object selected) {
					context->remove_annotation(selected);
					object folder = find_object(move_to_id);
					if(objectp(folder))
						folder->add_annotation(selected);
				}
			    break;
			case "get_objects":
				resMessage = RES(sizeof(aoSelected) + " Objects taken !",
                                 sizeof(aoSelected) + " Objekte aufgenommen !");
				foreach(aoSelected, obj) {
					LOG("Moving: " + obj->get_identifier());
					obj->move(user);
				}
				// Falls Mailbox angezeigt wurde Objekte als Annotation am Nutzer entfernen
				if ( (context->get_object_class() & CLASS_USER) > 0 && vars["type"] == "mailbox") {
					object parent;
					foreach(aoSelected, obj) {
						parent = obj->get_annotating();
						if ( objectp(parent) ) {
							parent->remove_annotation( obj );
						}
					}
				}
				uSelect = get_selection(user, context);
				// db on 3.9.2003             nur die bereits in der Hand gehaltenen Objekte zu den selektierten hinzufuegen, wenn
				//                            das get_objects nicht aus dem Rucksack aufgerufen wurde
        		if ( arrayp(uSelect) && (context != user || vars["type"] == "mailbox") ) {
					aoSelected += uSelect;
				}
				aoSelected &= user->get_inventory();
				user->set_attribute(USER_SELECTION, aoSelected);
				break;
		   case "copy_objects":
				if ( vars->copy_id ) {
					if ( mCopy[vars->copy_id] )
						resMessage = mCopy[vars->copy_id];
					else {
						return result_page(RES_MSG("Copying in progress ...",
												   "Kopiervorgang läuft ...", l),
										   _FILEPATH->object_to_filename(context),
										   RES_MSG("Copying in progress ...",
												   "Kopiervorgang läuft ...", l),
										   redirect_meta("/scripts/browser.pike?_action=copy_objects&amp;id="+context->get_object_id()+"&amp;copy_id="+vars->copy_id+(stringp(vars->type)?"&amp;type="+vars->type:""), 3));
					}
				}
				else {
					vars->copy_id = user->get_identifier()+time();
					if (!objectp(context) )
					  context = this_user();
		
					start_thread(handle_copy, context, vars);
					string copyStr = "<ul>";
					foreach(aoSelected, object osel)
						copyStr += "<li>"+osel->query_attribute(OBJ_NAME)+"</li>";
					copyStr += "</ul>";
					return result_page(RES_MSG("You are copying ", "Kopiere ", l) +
									   copyStr,
									   _FILEPATH->object_to_filename(context),
									   RES_MSG("Copy Documents","Kopieren", l),
									   redirect_meta("/scripts/browser.pike?_action=copy_objects&amp;id="+context->get_object_id()+"&amp;copy_id="+vars->copy_id+(stringp(vars->type)?"&amp;type="+vars->type:""), 2));
				}
				break;
			case "link_objects":
				[resMessage, errMessage] = handle_link(context, vars);
				break;
			case "download":
				if ( vars->did ) {
				  if ( objectp(downloads[(int)vars->did]) ) {
					  object down = downloads[(int)vars->did];
					  m_delete(downloads, (int)vars->did);
					  return result_page(RES_MSG("Starting download...","Download wird gestartet...", l),
										 httplib.replace_uml(_FILEPATH->object_to_filename(context))+(stringp(vars->type)?"?type="+vars->type:""),
										 RES_MSG("Starting download","Starte Download", l),
										 redirect_meta(httplib.replace_uml(_FILEPATH->object_to_filename(down)),1));
				  }
				  return result_page(RES_MSG("Download in progress...","Download wird vorbereitet", l),
									 httplib.replace_uml(_FILEPATH->object_to_filename(context)),
									 RES_MSG("Download in progress","Download in Vorbereitung", l),
									 redirect_meta("/scripts/browser.pike?_action=download&amp;id="+context->get_object_id()+"&amp;did="+vars->did+(stringp(vars->type)?"&amp;type="+vars->type:""), 10));
				}
				object download=get_factory(CLASS_DOCUMENT)->execute(
					([ "name": "download"+time()+".tar", ]) );
				add_event(download,
						  EVENT_DOWNLOAD_FINISHED,
						  PHASE_NOTIFY,
					download_finished);
				start_thread(downloader, aoSelected, download, this_user());
				string downloadStr = "<ul>";
				foreach(aoSelected, object osel)
					downloadStr += "<li>"+osel->get_identifier()+"</li>";
				downloadStr += "</ul>";
				return result_page(RES_MSG("You are downloading ", "Sie haben folgende Objekte zum Download ausgewählt", l) + downloadStr,
								   _FILEPATH->object_to_filename(context),
								   RES_MSG("Download Documents","Download", l),
								   redirect_meta("/scripts/browser.pike?_action=download&amp;id="+context->get_object_id()+"&amp;did="+download->get_object_id()+(stringp(vars->type)?"&amp;type="+vars->type:""), 10));
				break;
			case "delete_objects":
				int envi_id = -1;
				if ( vars["type"] == "details" ) {
				  object envi = context->get_environment();
				  if (objectp(envi)) envi_id = envi->get_object_id();
				  else envi_id = this_user()->query_attribute(USER_WORKROOM)->get_object_id();
				}
				[resMessage, errMessage] = handle_delete(user, vars, context);
				if ( vars["type"] == "details" ) {
				  vars["id"] = envi_id;
				  vars["_script"] = "/scripts/navigate.pike";
				  vars["object"] = envi_id;
				  vars["type"] = "content";
				}
				break;
			default: return "do=" + vars["do"];
				break;
		}
		break;
    case "save_wi_layout":
      object myuser = this_user();
      mixed  wisettings, specialsettings;
      wisettings = myuser->query_attribute("USER_WI_LAYOUT");
      if (!mappingp(wisettings)) wisettings = ([ ]);
      
      foreach(indices(vars), string skey) {
        string sckey, kkey;
        if ( sscanf(skey, "wilayout_%s_%s", kkey , sckey) == 2 ) {
          if (!mappingp(wisettings[ kkey ])) {
            wisettings[kkey] = ([ ]);
          }
          wisettings[kkey][sckey] = vars[skey];
        }
      }
      myuser->set_attribute( "USER_WI_LAYOUT", wisettings );
      return RES_MSG("Settings saved", "Einstellungen wurden gespeichert", l);
      break;
    case "restore_version": object restorefrom = find_object((int)vars->restore);
                            if (objectp(restorefrom))
                              context->set_content(restorefrom->get_content());
                            break;
    case "logout":
        object xsl = OBJ("/stylesheets/logout.xsl");
        string myxml = _XMLCONVERTER->get_xml(this(), xsl, vars, 0);
        string myhtml = run_xml(myxml, xsl, vars);
        return myhtml;
    case "addbuddy":
        if (!(vars["buddy1"])) {
          werror("browser.pike - addbuddy : buddy to add not specified");
          break;
        }
        object buddy = find_object((int)vars["buddy1"]);
        if (!objectp(buddy)) {
          werror("browser.pike - addbuddy : cant find user to add with id"+vars["buddy1"]);
          break;
        }
        array(object) bl;
        bl = user->query_attribute(USER_FAVOURITES);
        if (!arrayp(bl)) bl = ({});

        if (search(bl, buddy) < 0) {
          array(object) newbl = allocate(sizeof(bl)+1);
          for (int i = 0; i < sizeof(bl); i++) newbl[i] = bl[i];
          newbl[sizeof(newbl)-1] = buddy;
          user->set_attribute(USER_FAVOURITES, newbl);
        }
        break;
    case "removebuddy":
        if (!(vars["buddy1"])) {
          werror("browser.pike - removebuddy : buddy to remove not specified");
          break;
        }
        object removebuddy = find_object((int)vars["buddy1"]);
        if (!objectp(removebuddy)) {
          werror("browser.pike - removebuddy : cant find user to remove with id"+vars["buddy1"]);
          break;
        }
        array(object) blist;
        blist = user->query_attribute(USER_FAVOURITES);
        if (!arrayp(blist)) blist = ({});

        blist -= ({removebuddy});
        user->set_attribute(USER_FAVOURITES, blist);

        break;
    case "remove_forward":
        if (!objectp(FORWARD))
            return error_page(RES_MSG("Module forward not found.\n",
                                     "Das Modul forward konnte nicht gefunden werden.\n",l), redirection_url("", "", vars, context));
        if (vars->removeforward != "") {
          string nf = vars->removeforward;
          if (stringp(nf)) {
             array forwards = FORWARD->get_forward(context);
             if (arrayp(forwards)) {
               forwards -= ({nf});
               FORWARD->delete_forward(this_user());
               foreach(forwards, string fw)
                   FORWARD->add_forward(this_user(), fw);
             } else werror("Browser.pike- remove_forward: forwards are no array");
          }
        }
      break;
    case "add_forward":
        if (!objectp(FORWARD))
            return error_page(RES_MSG("Module forward not found.\n",
                                     "Das Modul forward konnte nicht gefunden werden.\n",l), redirection_url("", "", vars, context));
        if (vars->newforward != "") {
          string nf = vars->newforward;
          if (stringp(nf))
             FORWARD->add_forward(context, nf);
        }
      break;
    case "remove_host":
        if (vars->removehost != "") {
	  mapping hosts = GROUP("Admin")->query_attribute("virtual_hosts");
	  hosts -= ([vars->removehost : 0]);
	  GROUP("Admin")->set_attribute("virtual_hosts", hosts);
        }
      break;
    case "add_host": 
        if (vars->newhost != "" && vars->newhostdestination != "") {
	  mapping hosts = GROUP("Admin")->query_attribute("virtual_hosts");
    if (!mappingp(hosts)) hosts = ([]);
	  hosts += ([vars->newhost : vars->newhostdestination]);
	  GROUP("Admin")->set_attribute("virtual_hosts", hosts);
        }
      break;      
    case "remove_alias":
        if (!objectp(FORWARD))
            return error_page(RES_MSG("Module forward not found.\n",
                                     "Das Modul forward konnte nicht gefunden werden.\n",l), redirection_url("", "", vars, context));
        if (vars->removealias != "") {
          string da = vars->removealias;
          FORWARD->delete_alias(da);
        }
      break;
    case "add_alias":
        if (!objectp(FORWARD))
            return error_page(RES_MSG("Module forward not found.\n",
                                     "Das Modul forward konnte nicht gefunden werden.\n",l), redirection_url("", "", vars, context));
        if (vars->newalias != "" && vars->newaliasdestination != "") {
          FORWARD->add_alias(vars->newalias, vars->newaliasdestination);
        }
      break;
    case "show_all_urls":
      object filepath = get_module("filepath:url");
      if (objectp(filepath)) {
        return "<html><head><meta http-equiv=\"refresh\" content=\"0;URL="+
            "/scripts/navigate.pike?object="+filepath->get_object_id()+"\"></head><body>\n";
      } else
          return error_page(RES_MSG("Module filepath:url not found.\n",
                              "Das Modul filepath:url konnte nicht gefunden werden.\n",l), redirection_url("", "", vars, context));
      break;   
    case "reload_xml":
        err = catch {
            context->load_xml_structure();
        };
        if ( err == 0 ) {
            return result_page(RES_MSG("XML Meta Information File ("+
                                       _FILEPATH->object_to_filename(context)+
                                       ") has been reloaded",
                                       "Die XML Daten von "+
                                       _FILEPATH->object_to_filename(context)+
                                       " wurden in den Speicher geladen.", l),
                               redirection_url("", "", vars, context));
        }
        else {
            return error_page(RES_MSG("Failed to load xml meta file...\n"+
                                      (stringp(err) ? err : err[0]+"\n"+
                                       html_backtrace(err[1])),
                                      "XML Daten von "+
                                      _FILEPATH->object_to_filename(context)+" konnten nicht geladen werden.\n"+
                                      (stringp(err) ? err : err[0]+"\n"+
                                       html_backtrace(err[1]))
                                      ,l),
                              redirection_url("", "", vars, context));
        }
        break;
    case "select":
        LOG("Selecting !");
        if ( vars["_clipboard"] != "true" ) {
            uSelect = get_selection(user, context);
            resMessage = RES(" clipboard changed", "Clipboard modifiziert");
            if ( arrayp(uSelect) ) {
                aoSelected += uSelect;
            }
        }
        user->set_attribute(USER_SELECTION, aoSelected);
        break;
    case "get_objects":
        resMessage = RES(sizeof(aoSelected) + " Objects taken !",
                         sizeof(aoSelected) + " Objekte aufgenommen !");
        object actenv1 = user->get_environment();
        array(object) myenv = ({ actenv1 });
        while(objectp(actenv1 = actenv1->get_environment())) {
          myenv += ({ actenv1 });
        }
        werror("myenv=%O", myenv);
        user->get_environment();
        
        foreach(aoSelected, obj) {
            LOG("Moving: " + obj->get_identifier());
            if (search(myenv, obj) >= 0) {
              object mywr = user->query_attribute(USER_WORKROOM);
              if (!objectp(mywr)) mywr = OBJ("/");
              user->move(mywr);
            }
            obj->move(user);
        }
        // Falls Mailbox angezeigt wurde Objekte als Annotation am Nutzer entfernen
        if ( (context->get_object_class() & CLASS_USER) > 0 && vars["type"] == "mailbox") {
          object parent;
          foreach(aoSelected, obj) {
            parent = obj->get_annotating();
            if ( objectp(parent) ) {
              parent->remove_annotation( obj );
            }
          }
        }
        uSelect = get_selection(user, context);
// db on 3.9.2003             nur die bereits in der Hand gehaltenen Objekte zu den selektierten hinzufuegen, wenn
//                            das get_objects nicht aus dem Rucksack aufgerufen wurde
        if ( arrayp(uSelect) && (context != user || vars["type"] == "mailbox") ) {
            aoSelected += uSelect;
        }
        aoSelected &= user->get_inventory();
        user->set_attribute(USER_SELECTION, aoSelected);
        break;
    case "drop_objects":
        aoSelected = user->query_attribute(USER_SELECTION);
        array inventory = user->get_inventory();
        uSelect = get_selection(user, context);
        i = 0;

        foreach(inventory, obj) {
            if ( search(aoSelected, obj) >= 0 ) {
                err = catch {
                    obj->move(context);
                    uSelect -= ({ obj });
                };
              if ( err != 0 ) {
                  if ( !stringp(errMessage) ) errMessage = RES("","");

                  errMessage->english +=
                      "Failed to move "+obj->get_identifier()+
                      " - no permissions ?\n";
                  errMessage->german +=
                      "Bewegen von "+obj->get_identifier()+
                      " fehlgeschlagen - eventuell haben sie nicht die erforderlichen Rechte.\n";
              }

              i++;
          }
        }
        user->set_attribute(USER_SELECTION, uSelect);
        resMessage = RES(i + " Objects dropped !",
                         i + " Objekte fallen gelassen !");
        break;
    case "rucksack_drop_objects":
        aoSelected = get_selected(vars, ({ "invsel" }));
//    aoSelected = user->query_attribute(USER_SELECTION);
        array dinventory = user->get_inventory();
        uSelect = get_selection(user, context);
        i = 0;

        foreach(dinventory, obj) {
            if ( search(aoSelected, obj) >= 0 ) {
                err = catch {
                    obj->move(context);
                    uSelect -= ({ obj });
                };
                if ( err != 0 ) {
                    if ( !stringp(errMessage) ) errMessage = RES("","");

                    errMessage->english +=
                        "Failed to move "+obj->get_identifier()+
                        " - no permissions ?\n";
                    errMessage->german +=
                        "Bewegen von "+obj->get_identifier()+
                        " fehlgeschlagen - keine Berechtigung ?\n";
                }
                i++;
            }
        }
        user->set_attribute(USER_SELECTION, uSelect);
        resMessage = RES(i + " Objects dropped !",
                         i + " Objekte fallen gelassen !");
        break;
    case "link_exit_bi":
        if ( sizeof(aoSelected) != 1 )
            return error_page(RES_MSG("You can only set one room as destination !", "Sie dürfen nur einen Raum als Ziel auswählen",l),
                              "JavaScript:history.back();");
        the_room = aoSelected[0];
        if ( !(the_room->get_object_class() & CLASS_ROOM) )
            return error_page(RES_MSG("You have to select a room, no other type of object is allowed !", "Sie müssen einen Raum auswählen. Andere Objekte sind als Ziel einer Verbindung nicht zulässig.", l),
                              "JavaScript:history.back();");
        // check if open dialog for name input is requested
        if (stringp(vars["name_input"]) && vars["name_input"] == "true") {
          err = catch{
            object xsl = _FILEPATH->path_to_object("/stylesheets/create.xsl");
            //xml = _FILEPATH->path_to_object("/documents/create.xml")->get_content();
            xml = _XMLCONVERTER->get_xml(this(), xsl, vars, 0);
            if ( vars["source"] == "true" ) {
                xml = (((xml/"<")*"&lt;")/">")*"&gt;";
                return "<html><body><pre>\n"+xml+"\n</pre></body></html>\n";
            }   
            vars["mode"]   = "Gate_Name";
            vars["objsel"] = the_room->get_object_id();
            vars["object"] = context->get_object_id();
            html = _Server->get_module("libxslt")->run(xml, xsl, vars);
            return html;
          };
          if (err) {
            return error_page("error running xml", "");
          }
        }
        factory = _Server->get_factory(CLASS_EXIT);
        obj = factory->execute( ([ "name": context->get_identifier(),
                                 "exit_to": 0, ]) );
        err = catch {
            obj->move(the_room);
        };
        if ( err != 0 ) {
            obj->delete();
            return error_page(RES_MSG("You have no permissions to insert objects in the destination room !", "Sie haben keine Berechtigung im Zielraum Objekte abzulegen.", l),
                              "JavaScript:history.back();");
      }
        room = context;
        context = obj;
    case "link_exit":
        if ( sizeof(aoSelected) != 1 )
            return error_page(RES_MSG("You can only set one room as destination !", "Sie dürfen nur einen Raum als Ziel auswählen", l),
                              "JavaScript:history.back();");
        the_room = aoSelected[0];
        if ( !(the_room->get_object_class() & CLASS_ROOM) )
            return error_page(RES_MSG("You have to select a room, no other type of object is allowed !", "Sie müssen einen Raum auswählen. Andere Objekte sind als Ziel einer Verbindung nicht zulässig.", l),
                              "JavaScript:history.back();");
        // check if open dialog for name input is requested
        if (stringp(vars["name_input"]) && vars["name_input"] == "true") {
          err = catch{
            object xsl = _FILEPATH->path_to_object("/stylesheets/create.xsl");
            //xml = _FILEPATH->path_to_object("/documents/create.xml")->get_content();
            xml = _XMLCONVERTER->get_xml(this(), xsl, vars, 0);
            if ( vars["source"] == "true" ) {
                xml = (((xml/"<")*"&lt;")/">")*"&gt;";
                return "<html><body><pre>\n"+xml+"\n</pre></body></html>\n";
            }   
            vars["mode"]   = "Gate_Name";
            vars["objsel"] = the_room->get_object_id();
            vars["object"] = context->get_object_id();
            html = _Server->get_module("libxslt")->run(xml, xsl, vars);
            return html;
          };
          if (err) {
            return error_page("error running xml", "");
          }
        }
        factory = _Server->get_factory(CLASS_EXIT);
        obj = factory->execute( (["name": the_room->get_identifier(),
                                 "exit_to": 0 ]) );
        if ( context->get_object_class() & CLASS_EXIT ) {
            obj->set_link_object(context);
            obj->move(room);
            context->set_link_object(obj);
        }
        else {
            obj->set_link_object(the_room);
            obj->move(context);
        }
        
        if (stringp(vars["name"]) && vars["name"] != "") {
          obj->set_attribute(OBJ_NAME, vars["name"]);
        }
        if (stringp(vars["objdesc"]) && vars["objdesc"] != "") {
          obj->set_attribute(OBJ_DESC, vars["objdesc"]);
        }
        
        return "<html><script language=\"javascript\">function doClose() { window.opener.location.reload(); window.close(); }</script><body onLoad=\"doClose()\"></body></html>";
      break;

      case "upgrade_script":
          werror("Upgrading: " + context->get_object_id()+" name="+context->get_identifier());
          mixed res = catch(context->upgrade());
          if ( res != 0 ) {
              string backtr = sprintf("%O",context->get_errors());
              werror("backtr="+backtr);
              backtr = replace(backtr, "\n", "<br />" );
              return error_page(RES_MSG("Failed to upgrade "+context->get_identifier()+":<BR/><br />"+backtr, "Das Aktualisieren von "+context->get_identifier()+" ist fehlgeschlagen:<BR/><br />"+backtr, l),
                                "JavaScript:history.back();");
          }
          return result_page(RES_MSG("Successfully upgraded program.", context->get_identifier()+" wurde erfolgreich aktualisiert.", l),
                             redirection_url(RES_MSG("Action successfull.", "Upgrade erfolgreich", l), "", vars, context));
 
        break;
      case "upgrade":
          werror("upgrading:"+vars["upgrade"]+"\r\n");
          program __prog;
          if ( stringp(vars["upgrade"]) && (int)vars["upgrade"] == 0 )
              __prog = (program)vars["upgrade"];
          if ( !programp(__prog) ) {
              if ( stringp(vars["upgrade"]) )
                  context = find_object((int)vars["upgrade"]);
              LOG("Upgrading: " + context->get_object_id()+"\n");
              //   program __prog = object_program(context->get_object());
              __prog =
                  master()->lookup_program("/DB:#"+context->get_object_id()+
                                           ".pike");
     
              if ( !programp(__prog) )
              __prog = object_program(context->get_object());
          }
          werror("Name="+__prog->get_identifier()+"Class: " +master()->describe_program(__prog));
          res = master()->upgrade(__prog);
    //      mixed res = __prog->upgrade();
          if ( !intp(res) ) {
              return error_page(RES_MSG("Failed to upgrade "+context->get_identifier()+"<BR/>"+res, "Das Aktualisieren von "+context->get_identifier()+" ist fehlgeschlagen.<BR/>"+res, l),
                                "JavaScript:history.back();");
          }
          return result_page(RES_MSG("Successfully upgraded program.", context->get_identifier()+" wurde erfolgreich aktualisiert.", l),
                             redirection_url(RES_MSG("Action successfull.", "Upgrade erfolgreich", l), "", vars, context));

      break;
    case "download":
        if ( vars->did ) {
          if ( objectp(downloads[(int)vars->did]) ) {
              object down = downloads[(int)vars->did];
              m_delete(downloads, (int)vars->did);
              return result_page(RES_MSG("Starting download...","Download wird gestartet...", l),
                                 httplib.replace_uml(_FILEPATH->object_to_filename(context))+(stringp(vars->type)?"?type="+vars->type:""),
                                 RES_MSG("Starting download","Starte Download", l),
                                 redirect_meta(httplib.replace_uml(_FILEPATH->object_to_filename(down)),1));
          }
          return result_page(RES_MSG("Download in progress...","Download wird vorbereitet", l),
                             httplib.replace_uml(_FILEPATH->object_to_filename(context)),
                             RES_MSG("Download in progress","Download in Vorbereitung", l),
                             redirect_meta("/scripts/browser.pike?_action=download&amp;id="+context->get_object_id()+"&amp;did="+vars->did+(stringp(vars->type)?"&amp;type="+vars->type:""), 10));
        }
        object download=get_factory(CLASS_DOCUMENT)->execute(
            ([ "name": "download"+time()+".tar", ]) );
        add_event(download,
                  EVENT_DOWNLOAD_FINISHED,
                  PHASE_NOTIFY,
            download_finished);
        start_thread(downloader, aoSelected, download, this_user());
        string downloadStr = "<ul>";
        foreach(aoSelected, object osel)
            downloadStr += "<li>"+osel->get_identifier()+"</li>";
        downloadStr += "</ul>";
        return result_page(RES_MSG("You are downloading ", "Sie haben folgende Objekte zum Download ausgewählt", l) + downloadStr,
                           _FILEPATH->object_to_filename(context),
                           RES_MSG("Download Documents","Download", l),
                           redirect_meta("/scripts/browser.pike?_action=download&amp;id="+context->get_object_id()+"&amp;did="+download->get_object_id()+(stringp(vars->type)?"&amp;type="+vars->type:""), 10));
        break;
    case "create":
        object e = master()->ErrorContainer();
        master()->set_inhibit_compile_errors(e);
        object created;
        err = catch {
            created = context->execute(vars);
        };
        master()->set_inhibit_compile_errors(0);
        if ( err != 0 ) {
            return error_page("Error while creating object<BR/>"+
                              err[0]+"<br/>"+
                              e->get()+"<br/>"+
                              html_backtrace(err[1]),
                              "JavaScript:history.back();");
        }

        if ( objectp(created) ) {
            created->move(user);
            user->set_attribute(USER_SELECTION,
                                user->query_attribute(USER_SELECTION) +
                                ({ created->this() }));
            // return status page of object !!
            string created_html = get_html_status(created);
            return created_html;
        }
        master()->set_inhibit_compile_errors(0);
      break;
    case "status":
        return get_html_status(context);
      case "all":
          uSelect = get_selection(user, context);
          aoSelected = context->get_inventory();
          user->set_attribute(USER_SELECTION, aoSelected + uSelect);
          break;
    case "order_inventory":
          [ resMessage, errMessage ] = order_inventory(user, context, vars);
          return "<html><body onLoad=\"opener.location.reload();window.close()\"></body></html>\n";
          break;
    case "order_inventory_slides":
        [ resMessage, errMessage ] =
            order_inventory_slides(user, context, vars);

        break;
      case "set_read":
          [resMessage, errMessage ] = handle_set_read(context);
          break;
    case "annotation":
        err = catch {
            ann = create_local_annotation(vars);
      };
        if ( err != 0 ) {
            return error_page(
                "Failed to add annotation <BR/> " + err[0] + "<BR/>"+
                html_backtrace(err[1]),
                "navigate.pike?object="+id+"&type=annotations"
                );
        }
        else {
            vars["active"] = (string)ann->get_object_id();
        }
        while ( objectp(context->get_annotating())) {
          context = context->get_annotating();
        }
        vars["id"] = context->get_object_id();
        vars["object"] = context->get_object_id();
        string redirect_url = redirection_url("", "" ,vars, context);
        return "<html><body onLoad=\"opener.location.href='"+redirect_url+"';window.close();\"></body></html>\n";
        break;
      case "delete_annotation":
          [resMessage, errMessage] = handle_delete_annotation(user, vars);
          array(object) anns_left = context->get_annotations();
          if ( sizeof(anns_left) > 0 )
              vars["active"] = anns_left[-1]->get_object_id();
          break;
    case "change":
        user->set_attribute(USER_SELECTION, aoSelected);
        break;
    case "clear":
        user->set_attribute(USER_SELECTION, ({ }));
      case "show_inventory_order":
          return html_from_object(context, vars,
                                  _FILEPATH->path_to_object(
                                      "/stylesheets/order.xsl"));
          break;
    case "active_group":
      if ( stringp(vars["active_group"]) ) {
        object g = find_object((int)vars["active_group"]);
        if (objectp(g))
          this_user()->set_active_group(g);
        else {
          g = GROUP(vars["active_group"]);
          if (objectp(g))
            this_user()->set_active_group(g);
        }
        if ( !stringp(vars["_script"]) && objectp(g) )
          return result_page(RES_MSG("Set Active group to "+g->get_identifier(),
                             "Aktive Gruppe ist nun "+g->get_identifier(), l),
                             "Javascript:history.back();");
      }
      break;
    case "search":
        if ( vars->search_id ) {
          clean_searches();
            if ( mSearch[vars->search_id] )
            {
                string t = mSearch[vars->search_id];
                return t;
            }
            else if ( !objectp(context) ) {
              return result_page(RES_MSG("This search is no longer valid !",
					   "Diese Suche ist leider nicht mehr gültig !", l), "/home/" + this_user()->get_user_name());
            }
            else {
                return result_page(RES_MSG("Search in progress ...",
                                           "Suche dauert an ...", l),
                                   ((stringp(vars["room"])&&vars["room"]!="")?vars["room"]:"JavaScript:history.back();"));
            }
        }
        else {
            string _perform=vars["_perform"];
            int newsearch = 0;
             if (_perform=="stylesheet_add") {               
                 vars["advsearch_objtype"]="group";
                 vars["keywords"] = vars["keywords_stylesheetadd"];
                 if (!stringp(vars["quicksearch"])) return perform_quicksearch(vars, 0, 1);
             }            
            
             if (_perform=="acl_add") {
                 vars["advsearch_objtype"]="user+group";
                 vars["keywords"] = vars["keywords_acladd"];
                 if (!stringp(vars["quicksearch"])) return perform_quicksearch(vars, 1, 1);
             }

             if (_perform=="addbuddies") {
                 vars["advsearch_objtype"]="user";
                 vars["keywords"] = vars["keywords_addbuddy"];
                 if (!stringp(vars["quicksearch"])) return perform_quicksearch(vars, 1, 0);
             }


             if (_perform!=0 && _perform=="new_search")  newsearch = 1;
             if (!stringp(vars["keywords"]) || vars["keywords"] == "") {
                if (vars["keywords"] =="") m_delete(vars,"keywords");
                vars["_perform"] = "new_search";
                newsearch = 1;
             }

             if (_perform=="grp_invite") {
                 vars["advsearch_objtype"]="user";
                 if (!stringp(vars["quicksearch"])) return perform_quicksearch(vars, 1, 0);
             }
             if (_perform=="link_exit") {
                 vars["advsearch_objtype"]="room";
                 vars["object"] = vars->environment;
                 vars["room"] = vars->environment;
                 vars["id"] = vars->environment;
             }
             if (_perform=="link_exit_bi") {
                 vars["advsearch_objtype"]="room";
                 vars["object"] = vars->environment;
                 vars["room"] = vars->environment;
                 vars["id"] = vars->environment;
             }
             if (vars["advsearch_fulltext"]=="yes")
                 vars["advsearch_objtype"]="document";

	     if (!stringp(vars["advsearch_objtype"])) {
	       //keyword search
	       object searchspm = get_module("package:search");
	       if (objectp(searchspm)) {
		 vars->_action = "search";
		 return searchspm->execute( vars );
	       }
	     }

            OBJ("/stylesheets/selection.xsl")->get_stylesheet("german");
            OBJ("/stylesheets/selection.xsl")->get_stylesheet("english");
            OBJ("/stylesheets/selection.xsl")->get_stylesheet("chinese");

            if (newsearch != 0)
              return html_from_list(0, 0, vars, _FILEPATH->path_to_object("/stylesheets/selection.xsl"));

            vars->search_id = user->get_identifier()+"_"+time();
            if (!objectp(context) )
                context = this_user();

	    string head, foot;
	    vars->user = this_user()->get_object_id();

	    object origResult = handle_search(user, copy_value(vars));
	    object result =  Async.HtmlHandler(handleSearchResult, origResult);
	    vars->language = l;
	    origResult->vars = vars;
	    result->set_html(
		result_page("<h2>"+RES_MSG("You are searching for:",
				    "Suchbegriff:" , l)+
			    vars["keywords"] + "</h2><br />" +
			    RES_MSG("Searching might take a while ...",
				    "Die Suche kann eine Weile dauern", l),
			    ((stringp(vars["room"])&&vars["room"]!="")?vars["room"]:"JavaScript:history.back();")));
	    return result;
        }
        break;
    case "acquire":
        foreach(aoSelected, obj) {
            obj->set_acquire(context);
        }
        break;
    case "delete_objects":
    case "delete":
        int envi_id = -1;
        if ( vars["type"] == "details" ) {
          object envi = context->get_environment();
          if (objectp(envi)) envi_id = envi->get_object_id();
          else envi_id = this_user()->query_attribute(USER_WORKROOM)->get_object_id();
        }
        [resMessage, errMessage] = handle_delete(user, vars, context);
        if ( vars["type"] == "details" ) {
          vars["id"] = envi_id;
          vars["_script"] = "/scripts/navigate.pike";
          vars["object"] = envi_id;
          vars["type"] = "content";
        }
        break;
    case "move":
      [resMessage, errMessage] = handle_move(user, vars);
      break;
    case "empty_trashbin":
        if ( vars->trash_id ) {
            if ( mTrash[vars->trash_id] )
                resMessage = mTrash[vars->trash_id];
            else {
                return result_page(RES_MSG("Empty Trashbin in progress ...",
                                           "Papierkorb wird geleert ...", l),
                                   _FILEPATH->object_to_filename(context),
                                   RES_MSG("Empty Trashbin in progress ...",
                                           "Papierkorb wird geleert ...", l),
                                   redirect_meta("/scripts/browser.pike?_action=empty_trashbin&amp;id="+context->get_object_id()+"&amp;trash_id="+vars->trash_id, 3));
            }
        }
        else {
            vars->trash_id = user->get_identifier()+time();
            if (!objectp(context) )
              context = user->query_attribute(USER_TRASHBIN);
            start_thread(handle_emptytrashbin, context, vars);
            return result_page(RES_MSG("Empty Trashbin", "Papierkorb wird geleert", l),
                               _FILEPATH->object_to_filename(context),
                               RES_MSG("Empty Trashbin","Papierkorb wird geleert", l),
                               redirect_meta("/scripts/browser.pike?_action=empty_trashbin&amp;id="+context->get_object_id()+"&amp;trash_id="+vars->trash_id, 2));
        }
        break;       
    case "copy_objects":
    case "copy":
        if ( vars->copy_id ) {
            if ( mCopy[vars->copy_id] )
                resMessage = mCopy[vars->copy_id];
            else {
                return result_page(RES_MSG("Copying in progress ...",
                                           "Kopiervorgang läuft ...", l),
                                   _FILEPATH->object_to_filename(context),
                                   RES_MSG("Copying in progress ...",
                                           "Kopiervorgang läuft ...", l),
                                   redirect_meta("/scripts/browser.pike?_action=copy_objects&amp;id="+context->get_object_id()+"&amp;copy_id="+vars->copy_id+(stringp(vars->type)?"&amp;type="+vars->type:""), 3));
            }
        }
        else {
            vars->copy_id = user->get_identifier()+time();
            if (!objectp(context) )
              context = this_user();

            start_thread(handle_copy, context, vars);
            string copyStr = "<ul>";
            foreach(aoSelected, object osel)
                copyStr += "<li>"+osel->query_attribute(OBJ_NAME)+"</li>";
            copyStr += "</ul>";
            return result_page(RES_MSG("You are copying ", "Kopiere ", l) +
                               copyStr,
                               _FILEPATH->object_to_filename(context),
                               RES_MSG("Copy Documents","Kopieren", l),
                               redirect_meta("/scripts/browser.pike?_action=copy_objects&amp;id="+context->get_object_id()+"&amp;copy_id="+vars->copy_id+(stringp(vars->type)?"&amp;type="+vars->type:""), 2));
        }
        break;
    case "link_objects":
    case "link":
      [resMessage, errMessage] = handle_link(context, vars);
      break;
    case "restart":
        call(restart, 2);
      return "<html><head><meta http-equiv=\"refresh\" content=\"20;URL="+
          "admin.pike\"></head><body>\n"+
          "<h2>The server is restarting now....</h2></body></html>\n";
      break;
    case "server_config":
        context = OBJ("/register/register.html");
      if ( objectp(context) ) {
          context->set_attribute("denied", 0);
          context->set_attribute("no_email", 0);
          context->set_attribute("activate", 0);
      }

      foreach(indices(vars), string config) {
          string configuration;
          if ( sscanf(config,"config_%s", configuration) == 1 ) {
//            LOG("Set config:"+configuration+"="+get_value(vars[config]));
              _Server->set_config(
                  configuration, get_value(vars[config]));
          }
          if ( config =="variable_config_key" && stringp(vars[config])  && strlen(vars[config])>0) {
//            LOG("Set variable config:"+get_value(vars[config])+"="+get_value(vars["variable_config_value"]));
              _Server->set_config(get_value(vars[config]), get_value(vars["variable_config_value"]));
          }
          if ( sscanf(config, "register_%s", configuration) == 1 ) {
              if ( objectp(context) )
                  context->set_attribute(configuration, 1);
          }
      }
      break;
    case "server_config_delete":
      if ( stringp(vars["todelete"]) && strlen(vars["todelete"])>0) {
        _Server->delete_config(get_value(vars["todelete"]));
      }
      break;
    case "request":
       object reqGroup = find_object((int)vars["request"]);
       vars["_script"] = "/groups/"+reqGroup->get_identifier();
       if ( !catch(_SECURITY->check_access(reqGroup, this_user(), SANCTION_INSERT, ROLE_READ_ALL, false)) || reqGroup->is_invited(user) || (
           stringp(vars["grp_pw"]) && reqGroup->has_password() ) )
       {
           if (stringp(vars["grp_pw"]) && reqGroup->has_password() ) {
             if (!reqGroup->check_group_pw(vars["grp_pw"])) {
               return error_page(
                 RES_MSG("Failed to add user to Group.<BR/>"+
                 "Wrong Group password.",
                 "Sie konnten nicht zur Gruppe hinzugefügt werden.<BR/>Das Gruppenpasswort ist falsch.", l),
                 redirection_url("", "", vars, context));
             }
           }
           err = catch {
               int myar = reqGroup->add_member(user, vars["grp_pw"]);
               if (myar == 1) {
                 create_group_exit(reqGroup, user);
                 resMessage = RES("You were added to this group.",
                  "Sie wurden zur Gruppe hinzugefügt.");
               } else if (myar == -1) {
                 reqGroup->remove_member(user);
                 resMessage = RES("Failed to add you to this group. The group is full.",
                  "Sie konnten dieser Gruppe nicht beitreten. Die Gruppe ist voll.");
               } else {
                 resMessage = RES("Failed to add user !",
                 "Benutzer konnte nicht hinzugefügt werden!");
               }
           }; 
           resMessage = RES("You were added to this group.",
                "Sie wurden zur Gruppe hinzugefügt.");

           if ( err != 0 ) {
               resMessage = RES("Failed to add user !",
                                "Benutzer konnte nicht hinzugefügt werden!");
               errMessage = RES("Failed to add user: " + err[0],
                                "Benutzer konnte nicht hinzugefügt werden: " + err[0]);
           }
       }
       else {
           if ( reqGroup->requested_membership(user) ) {
           resMessage = RES("Already requested membership ...",
                    "Mitgliedschaft wurde bereits beantragt.");
           errMessage = RES("You already requested membership.",
                    "Mitgliedschaft wurde bereits beantragt.");
           }
           else if ( reqGroup->is_member(user) ) {
           resMessage = RES("You are already a member of that group.",
                    "Sie sind bereits Mitglied der Gruppe.");
           errMessage = RES("You are already a member of that group.",
                    "Sie sind bereits Mitglied der Gruppe.");
           }
           else {
             resMessage = RES("You requested membership in the group.",
                    "Sie haben Mitgliedschaft in der Gruppe "+
                    "beantragt.");
             reqGroup->add_membership_request(user);
             mapping reqMsg =
               RES(
               "<html><body>"+
               "The user "+user->query_attribute(OBJ_NAME)+" requested membership in the group "+
               reqGroup->get_identifier() + "\n<br/>"+
               (stringp(vars["grp_reason"])?"Reason:\n<br/>"+ vars["grp_reason"]:"")+
               "<br/>View group "+reqGroup->query_attribute(OBJ_NAME)+" :\n<br/>"+
               get_visit_link_group(reqGroup, "?mode=admin")+
               "</body></html>\n",
               "<html><body>"+
               "Der Benutzer "+user->query_attribute(OBJ_NAME)+" moechte Mitglied in der Gruppe "+
               reqGroup->get_identifier() + " werden\n<br/>"+
               (stringp(vars["grp_reason"])?"Grund:\n<br/>"+vars["grp_reason"]:"")+
               "<br/>Zum Ansehen der Gruppe "+reqGroup->query_attribute(OBJ_NAME)+" folgen Sie bitte dem untenstehenden Link :\n<br/>"+
               get_visit_link_group(reqGroup, "?mode=admin")+
               "</body></html>\n");
           foreach(reqGroup->get_admins(), object grpAdmin) {
               grpAdmin->mail(reqMsg[(grpAdmin->query_attribute(USER_LANGUAGE)!=0?grpAdmin->query_attribute(USER_LANGUAGE):"english")], reqGroup->get_identifier()+
                      ": Membership request");
               tmod->add_task(grpAdmin, this(), "add_grp_member",
                      ({ reqGroup, user }),
                      ([ "english": "Add " + userdetails(user)+
                     " to "+
                     reqGroup->get_identifier() + " (requested)",
                      "german": userdetails(user)+
                     " zur Gruppe "+
                     replace_uml(reqGroup->get_identifier()) +
                     " hinzufuegen (requested)",
                     ]));
           }
           }
       }
       break; 
    case "reject":
       object rejGroup = find_object((int)vars["reject"]);
       vars["_script"] = "/groups/"+rejGroup->get_identifier();
       err = catch {
           rejGroup->remove_member(user);
           delete_group_exit(rejGroup, user);
       };
       resMessage = RES("User has been removed from this group !",
            "Der Benutzer wurde aus dieser Gruppe entfernt!");

       if ( err != 0 ) {
           resMessage = RES("Failed to remove user !",
                            "Benutzer konnte nicht aus der Gruppe entfernt werden!");
           errMessage = RES("Failed to remove user: " + err[0],
                            "Benutzer konnte nicht aus der gruppe entfernt werden werden:" + err[0]);
       }
       break;
       case "tasks":
       if ( vars->tasks ) {
         mixed taskv = get_values(vars->tasks);
         if ( arrayp(taskv) ) {
             foreach(taskv, mixed task) {
               err = catch(tmod->run_task((int)task));
               if ( err != 0 ) {
                 werror("Error on running task :\n" + sprintf("%O\n",err));
               }
             }
         }
         else {
           tmod->run_task((int)vars["tasks"]);
         }
       }
       tmod->tasks_done(this_user());
       // if browsing a group set script to "/groups/.."
       if (objectp(context) && (context->get_object_class() & CLASS_GROUP)) {
           vars["_script"] = "/groups/"+context->get_identifier();
           if (vars["type"] == "content") m_delete(vars, "type");
       }

       break;
       case "user_details":
       if ( vars["mode"] == "user" ) {
           context->set_attribute(USER_FULLNAME, vars["user_name"]);
           context->set_attribute(USER_FIRSTNAME, vars["user_firstname"]);
           context->set_attribute(USER_LANGUAGE, vars["user_language"]);
           l = context->query_attribute(USER_LANGUAGE);
           context->set_attribute(OBJ_DESC, vars["user_description"]);
           change_email(context, context->query_attribute(USER_EMAIL), vars["user_email"]);
           context->set_attribute(USER_EMAIL, vars["user_email"]);
           context->set_attribute(USER_ADRESS, vars["user_adress"]);
           int oldforward = context->query_attribute(USER_FORWARD_MSG);
           int newforward = (vars["forward"]?1:0);
           context->set_attribute(USER_FORWARD_MSG,
                      (vars["forward"] ? 1 : 0));
           context->set_attribute("launcher",
                      (vars["launcher"] ? 1 : 0));

           if (oldforward==1 && newforward==0) {
             // delete forwards
             FORWARD->delete_forward(context);
           }
           if (oldforward==0 && newforward==1) {
             // add USER_EMAIL as forward and USER NAME to save a local copy
             string nf = context->query_attribute(USER_EMAIL);
             string nf2 = FORWARD->get_mask_char()+context->get_user_name();  // enable local copy by default
             array forwards = FORWARD->get_forward(context);
             if (!arrayp(forwards)) forwards = ({});
             if (arrayp(forwards)) {
               forwards += ({nf});
               forwards += ({nf2});
               FORWARD->delete_forward(context);
               foreach(forwards, string fw)
                   FORWARD->add_forward(context, fw);
             } else werror("Browser.pike- user_details - mailconfig: forwards are no array\n");
           }

           if ( stringp(vars["active_group"]) ) {
           object g = find_object((int)vars["active_group"]);
           context->set_active_group(g);
           }
           if ( stringp(vars["new_password"]) && vars["new_password"]!="" )
           {
           string backb = "navigate.pike?object="+id+
               "&mode=user&type=details";

           if ( vars["new_password"] == vars["confirm_password"] )
           {
               if ( context->check_user_password(vars["old_password"]))
               {
               err = catch {
                   context->set_user_password(
                   vars["new_password"]);
               };
                       }
               else if ( this_user()->get_object_id() !=
                 context->get_object_id() &&
                 _ADMIN->is_member(this_user()) )
               {
               err = catch {
                   context->activate_user();
                   context->set_user_password(
                   vars["new_password"]);
               };
               }
               else {
               return error_page(
                   RES_MSG("Failed to change password !<BR/>"+
                   "Old password does not match - try again !",
                   "Das Passwort konnte nicht geändert werden<BR/>Das von ihnen eingegebene aktuelle Passwort stimmt nicht.", l),
                   backb);

               }
                       if ( err != 0 ) {
               return error_page(RES_MSG("Failed to change password - no access ?", "Das Passwort konnte nicht geändert werden<BR/>Eventuell haben Sie keine Berechtigung das Passwort zu ändern.", l),
                   backb);
               }
               else if ( this_user()->get_object_id() ==
                 context->get_object_id() )
               {
               return  result_page(RES_MSG("Successfully changed your password !<BR/>You have to re-authorize now in order to continue.",
                                           "Ihr Passwort wurde geändert", l),
                   backb);
               }
           }
           else {
               return error_page(RES_MSG("Failed to change password !<BR/>Cannot confirm new password - try again !", "Das Passwort konnte nicht geändert werden<BR/>Die zweite Passworteingabe stimmt nicht mit der ersten Eingabe überein.", l), backb);
           }
           }
       }
       else if ( vars["mode"] == "requests" ) {
           array(object) groups = extract_objects(vars["request"]);
           foreach(groups, object grp) {
           object owner = grp->get_creator();
           grp->add_membership_request(user);
           string messagelanguage = (owner->query_attribute(USER_LANGUAGE)!=0?owner->query_attribute(USER_LANGUAGE):"english");
           string messagebody =
               RES_MSG(
               "<html><body>"+
               "The user requested membership in the group "+
               replace_uml(grp->get_identifier()) + "\n<br/>"+
               "To View the group follow the link below:\r\n<br/>"+
               get_visit_link_group(grp, "?mode=admin")+
               "</body></html>\n",
               "<html><body>"+
               "Der Benutzer moechte in die Gruppe "+
               replace_uml(grp->get_identifier()) + " aufgenommen werden\n<br/>"+
               "Um die Gruppe anzusehen folgen Sie dem untenstehenden Link:\r\n<br/>"+
               get_visit_link_group(grp, "?mode=admin")+
               "</body></html>\n",
               messagelanguage);
           string messagesubject=
               RES_MSG(grp->get_identifier()+": Request Membership",
               "Mitgliedschaftsanfrage fuer die Gruppe "+grp->get_identifier(),
               messagelanguage);
           grp->get_creator()->mail(messagebody, messagesubject);
           }
       }
       else if ( vars["mode"] == "image" ) {
           string|object    oldPic;
           int           oldPicNum;

           oldPic = context->query_attribute(OBJ_ICON);

           if ( stringp(oldPic) &&
            sscanf(oldPic, "%*sobject=%d", oldPicNum) )
           {
           obj = find_object(oldPicNum);
           if ( objectp(obj) )
               obj->delete();
           }
           else if ( objectp(oldPic) ) {
           if ( !objectp(oldPic->get_environment()) )
               oldPic->delete();
           }
           LOG("Uploading: " + vars["URL"]);
           if ( strlen(vars["user_image"]) > 50000 )
           return result_page(
               "The maximum size of a picture is 50k.",
               "JavaScript:history.back();");

           factory = _Server->get_factory(CLASS_DOCUMENT);
           string converted_name3 = replace(vars["user_image.filename"], "\\", "/" );
           converted_name3 = basename(converted_name3);
           obj = factory->execute((["url":converted_name3]));
           obj->set_content(vars["user_image"]);
           obj->set_attribute( "OBJ_TYPE", "document_icon_usericon");
           context->set_acquire_attribute(OBJ_ICON, 0);
           context->set_attribute(OBJ_ICON, obj);
           obj->sanction_object(_WORLDUSER, SANCTION_READ);
           if (context != this_user()) {
             obj->sanction_object(context, SANCTION_ALL);
             obj->sanction_object_meta(context, SANCTION_ALL);
           }
       }
       else if ( vars["mode"] == "mailconfig" ) {
           int store_sent_mail = (vars["store_sent_mail"] ? 1:0);
           context->set_is_storing_sent_mail(store_sent_mail);
           
           if (store_sent_mail) {
             object sent_mail_folder = context->get_sent_mail_folder();
             if (!objectp(sent_mail_folder)) {
               context->create_sent_mail_folder();
             }
           }
           if (!objectp(FORWARD))
             return error_page(RES_MSG("Module forward not found.\n",
                             "Das Modul forward konnte nicht gefunden werden.\n",l), redirection_url("", "", vars, context));
           change_email(context, context->query_attribute(USER_EMAIL), vars["user_email"]);
           context->set_attribute(USER_EMAIL, vars["user_email"]);

           int newlocalcopy = (vars["localcopy"] ? 1:0);
           int oldlocalcopy = 0;

           array fwds = FORWARD->get_forward(context);
           if (arrayp(fwds)) {
             if (search(fwds, FORWARD->get_mask_char()+context->get_user_name()) < 0) oldlocalcopy = 0 ;
             else oldlocalcopy = 1;
           }

           array forwards = FORWARD->get_forward(context);
           if (!arrayp(forwards)) forwards = ({});
           if (arrayp(forwards)) {
             foreach(forwards, string fw)
               if (fw==context->get_user_name()) oldlocalcopy = 1;
           }
           if (oldlocalcopy == 0 && newlocalcopy ==1) {
             // add indentifier to forwards
             string nf = FORWARD->get_mask_char()+context->get_user_name();
             forwards = FORWARD->get_forward(context);
             if (!arrayp(forwards)) forwards = ({});
             if (arrayp(forwards)) {
               FORWARD->delete_forward(context);
               foreach(forwards, string fw)
                   FORWARD->add_forward(context, fw);
             } else werror("Browser.pike- user_details - mailconfig: forwards are no array\n");
             FORWARD->add_forward(context, nf);
           }
           if (oldlocalcopy == 1 && newlocalcopy == 0) {
             // remove indentifier from forwards
             string df = context->get_user_name();
             forwards = FORWARD->get_forward(context);
             if (!arrayp(forwards)) forwards = ({});
             if (arrayp(forwards)) {
               forwards -= ({ FORWARD->get_mask_char()+df });
               FORWARD->delete_forward(context);
               foreach(forwards, string fw)
                   FORWARD->add_forward(context, fw);
             } else werror("Browser.pike- user_details - mailconfig: forwards are no array\n");
           }
           int oldforward = context->query_attribute(USER_FORWARD_MSG);
           int newforward = (vars["forward"]?1:0);
           if (oldforward==1 && newforward==0) {
             // delete forwards
             FORWARD->delete_forward(context);
           }
           if (oldforward==0 && newforward==1) {
             // add USER_EMAIL as forward
             string nf = context->query_attribute(USER_EMAIL);
             string nf2 = FORWARD->get_mask_char()+context->get_user_name();  // enable local copy by default
             forwards = FORWARD->get_forward(context);
             if (!arrayp(forwards)) forwards = ({});
             if (arrayp(forwards)) {
               forwards += ({nf});
               forwards += ({nf2});
               FORWARD->delete_forward(context);
               foreach(forwards, string fw)
                   FORWARD->add_forward(context, fw);
             } else werror("Browser.pike- user_details - mailconfig: forwards are no array");
           }
           context->set_attribute(USER_FORWARD_MSG,
                      (vars["forward"] ? 1 : 0));
       }
       // To access the user with his name
       if ((context->get_object_class() & CLASS_USER) != 0) {
         vars["_script"] = "/~"+context->get_user_name();
       }
       break;
      case "publish":
        string urlval = vars["attr_url"];
        if ( stringp(urlval) && strlen(urlval) > 0 &&
             urlval != context->query_attribute(OBJ_URL) )
          {
            err = catch {
              context->sanction_object(_WORLDUSER, SANCTION_READ|SANCTION_EXECUTE|SANCTION_ANNOTATE);
            };
            if ( urlval == "none" )
              urlval = 0;
            object prev=get_module("filepath:url")->resolve_path(0,urlval);
            if ( objectp(prev) ) {
              errMessage =
                RES("The Document <a href=\"/scripts/navigate.pike?object="+
                    prev->get_object_id() +
                    "&type=details&mode=advanced\">"+
                    prev->get_identifier() + "</a> already "+
                    "registered the URL '"+urlval+"'",
                    "Das Objekt <a href=\"/scripts/navigate.pike?object="+
                    prev->get_object_id() +
                    "&type=details&mode=advanced\">"+
                    prev->get_identifier() +
                    "</a> benutzt bereits die URL '"+urlval+"'");
            }
            else {
              if ( stringp(urlval) &&
                   strlen(urlval) > 0 &&
                   urlval[0] != '/' )
                {
                  // add leading path of environment
                  string urlpath = _FILEPATH->object_to_path(context);
                  urlval = urlpath + "/" + urlval;
                }
              if ( !objectp(context->get_acquire_attribute(OBJ_URL)) )
                context->set_acquire_attribute(OBJ_URL,
                                               get_module("filepath:url"));
              context->set_attribute(OBJ_URL, urlval);
            }
          }
        if ( stringp(vars["attr_xsl__public"]) ) {
          object pxsl =
            _FILEPATH->resolve_path(context, vars["attr_xsl__public"]);
          if ( !objectp(pxsl) )
            pxsl = _FILEPATH->path_to_object(vars["attr_xsl__public"]);
          context->set_attribute("xsl:public", pxsl);
        }
        break;
    case "update_icon":
          factory = _Server->get_factory(CLASS_DOCUMENT);
          string converted_name2 = replace(vars["icon_image.filename"], "\\", "/" );
          converted_name2 = basename(converted_name2);
          obj = factory->execute((["name":converted_name2,]));
          obj->set_content(vars["icon_image"]);
          obj->set_attribute( "OBJ_TYPE", "document_icon_documenticon");
          obj->sanction_object(_WORLDUSER, SANCTION_READ);
          context->set_acquire_attribute(OBJ_ICON, 0);
          context->set_attribute(OBJ_ICON, obj);
          break;
      case "update_stylesheet_details":
          html ="";
//          return result_simplepage("Stylesheet updated succesfully", "Stylesheet updated", "english");
//          html += "Group="+vars["stylesheet_group"]+"<BR/>";
//          html += "Stylesheet="+vars["stylesheet_group_selected"]+"<BR/>";

          object stylesheet = _FILEPATH->path_to_object(vars["stylesheet_group_selected"]);

          if (vars["stylesheet_group_selected"] == "userdefined") {
            stylesheet = _FILEPATH->path_to_object(vars["userdefined_stylesheet"]);
          }

          if (!objectp(stylesheet)) {
            if (vars["stylesheet_group_selected"]=="none") stylesheet=0;
            else  return error_page(RES_MSG("Failed to set stylesheet, stylesheet not found: "+vars["stylesheet_group_selected"], "Das Stylesheet konnte nicht gesetzt werden. Stylesheet nicht gefunden: "+vars["stylesheet_group_selected"], l),
                              "JavaScript:history.back();");
          }

          if (vars["stylesheet_group"]!="public") {
            object group = find_object((int)vars["stylesheet_group"]);

            if (!objectp(group)) {
              return error_page(RES_MSG("Failed to set stylesheet, group not found: "+vars["stylesheet_group"], "Das Stylesheet konnte nicht gesetzt werden. Gruppe nicht gefunden: "+vars["stylesheet_group"], l),
                              "JavaScript:history.back();");
            }

            mapping mStyles = context->query_attribute("xsl:content");

            if ( !mappingp(mStyles) )
              mStyles = ([ ]);

            mStyles[group] = stylesheet;
            context->set_acquire_attribute("xsl:content", 0);
            context->set_attribute("xsl:content", mStyles);
            if ((context->get_object_class() & CLASS_MESSAGEBOARD) > 0) {
              context->set_attribute("xsl:annotations", mStyles);
            }
          }

          if (vars["stylesheet_group"]=="public") {
            context->set_acquire_attribute("xsl:public", 0);
            context->set_attribute("xsl:public", stylesheet);
          }

          return result_simplepage(RES_MSG("Succesfully updated stylesheet:<BR/>"+html,
                               "Stylesheet wurde gesetzt:<BR/>"+html,l),
                         RES_MSG("Updated stylesheet","Stylesheet gesetzt",l),l);
          break;
      case "update_css_style":
          html ="";
          object css_style = _FILEPATH->path_to_object(vars["stylesheet_user_selected"]);

          if (vars["stylesheet_user_selected"] == "userdefined") {
            css_style = _FILEPATH->path_to_object(vars["userdefined_stylesheet"]);
          }

          if (!objectp(css_style)) {
            if (vars["stylesheet_user_selected"]=="none") css_style=0;
            else  {
              string stylename = vars["stylesheet_user_selected"];
              if ( stylename == "userdefined" ) 
                 stylename = vars["userdefined_stylesheet"];
              return error_page(RES_MSG("Failed to set css style, style not found: "+vars["stylesheet_user_selected"], "Der angegebene CSS Style konnte nicht gesetzt werden. CSS Styles nicht gefunden: "+stylename, l), "JavaScript:history.back();");
            }
          }

          context->set_attribute("USER_CSS_STYLE", css_style);
          break;
          
      case "update_multilanguage_details":

          mapping mSupportedlanguages = GROUP("Admin")->query_attribute("multilanguage_languages");

          if (!mappingp(mSupportedlanguages))
            return error_page(RES_MSG("Cant find supported languages at admin group:" +
                               " attribute multilanguage_languages not set on group Admin. (found no mapping)",
                               "Unterstuetzte Sprachen konnten nicht gefunden werden. " +
                               "Attribut multilanguage_languages ist nicht gesetzt bzw enthaelt kein mapping",l),
                         "JavaScript:history.back();");

          mapping mMultilanguage = context->query_attribute("language_index");
          if ( !mappingp(mMultilanguage) )
              mMultilanguage = ([ ]);

          foreach(indices(mSupportedlanguages), string slang) {
            mMultilanguage[slang]                  = vars["multilanguage_"+slang+"_index"];
            mMultilanguage[slang+"_button"]        = vars["multilanguage_"+slang+"_button"];
            mMultilanguage[slang+"_active_button"] = vars["multilanguage_"+slang+"_active_button"];
          }
          context->set_attribute("language_index", mMultilanguage);
        break;
      case "update_cont_type":

          string oldstype = context->query_attribute("cont_type");
          if ( !stringp(oldstype) )
              oldstype = "default";

          string stype = "default";
          if (stringp(vars["cont_type"])) stype = vars["cont_type"];

          //werror("cont_type: vars->cont_type="+vars["cont_type"]+" oldstype="+oldstype);

          if (stype=="multi_language" && oldstype!="multi_language") {
            //werror("setting new icon for multilanguage- container...");
            // change icon only if no user- defined icon is set
            if (context->get_acquire_attribute(OBJ_ICON) == _Server->get_module("icons") ) {
              context->set_acquire_attribute(OBJ_ICON, 0);
              if ((context->get_object_class() & CLASS_ROOM) != 0)
                   context->set_attribute(OBJ_ICON, _FILEPATH->path_to_object("/images/doctypes/type_area_multilanguage.gif"));
              else context->set_attribute(OBJ_ICON, _FILEPATH->path_to_object("/images/doctypes/type_folder_multilanguage.gif"));
            }
          }
          if (stype!="multi_language" && oldstype=="multi_language") {
            //werror("setting standard icon for multilanguage- container...");
            context->set_attribute(OBJ_ICON, 0);
            context->set_acquire_attribute(OBJ_ICON, _Server->get_module("icons"));
          }
          context->set_attribute("cont_type", vars["cont_type"]);
        break;
      case "update_main_attr":
      string|object val;

      if ( vars["editlinkicon"] == "true" ) {
          factory = _Server->get_factory(CLASS_DOCUMENT);
          string converted_name = replace(vars["icon_image.filename"], "\\", "/" );
          converted_name = basename(converted_name);
          obj = factory->execute((["name":converted_name,]));
          obj->set_content(vars["icon_image"]);
          obj->set_attribute( "OBJ_TYPE", "document_icon_documenticon");
          obj->sanction_object(_WORLDUSER, SANCTION_READ);
          context->set_acquire_attribute(OBJ_LINK_ICON, 0);
          context->set_attribute(OBJ_LINK_ICON, obj);
          break;
      }

      foreach(indices(vars), string skey) {
          LOG("Key: " + skey);
          string sStyle;
          int     group;
          string sskey;

          // unmask the ":" char which is not allowed in HTML-Output (parser restriction)
          if (stringp(skey)) sskey = replace(skey, "__", ":");

          if ( sscanf(sskey, "stylesheet_%s_%d", sStyle, group) == 2 )
          {
              val = get_value(vars[skey]);
              if ( strlen(vars[skey]) == 0 )
                continue;
              if ( arrayp(val) )
                val = val[0];

              val = find_object((int)val);

              if ( !objectp(val) && stringp(vars[skey]) ) {
                  val = _FILEPATH->resolve_path(context, vars[skey]);
                  if ( !objectp(val) )
                      val = _FILEPATH->path_to_object(vars[skey]);
              }

              mapping mStyles = context->query_attribute(sStyle);
              if ( !mappingp(mStyles) )
                  mStyles = ([ ]);

              mStyles[find_object(group)] = val;
              context->set_acquire_attribute(sStyle, 0);
              context->set_attribute(sStyle, mStyles);
              if ((context->get_object_class() & CLASS_MESSAGEBOARD) > 0) {
                if (sStyle = "xsl:content")
                  context->set_attribute("xsl:annotations", mStyles);
              }
          }

          if ( sscanf(skey, "attr_%d", key) != 1 &&
               sscanf(skey, "attr_%s", key) != 1 )
              continue;
          err = catch {
            
            // unmask the ":" char which is not allowed in HTML-Output (parser restriction)
            if (stringp(key)) key = replace(key, "__", ":");   
            
            regs = _Server->get_factory(context->get_object_class())->
              describe_attribute(key);

            val = get_value(vars[skey]);
            if (objectp(regs) && regs->get_type()==CMD_TYPE_OBJECT &&
                !objectp(val))
              val = _FILEPATH->path_to_object(val);
            if ( arrayp(regs) &&
                 regs[REGISTERED_TYPE] == CMD_TYPE_STRING && intp(val) )
              val = (string) val;

            LOG("Key="+key+",Value="+sprintf("%O",val));
            // setting an url should make the document public

            if ( val != context->query_attribute(key) ) {
              context->set_attribute(key, val);
            }
          };
          if ( err )
            FATAL("Error updating attribute: %s,\n %O", key, err);
      }

      if ( context->get_object_class() & CLASS_OBJECT ) // immer halt...
      {
          LOG("XSL:Use_Public = " + vars["xsl_usepublic"]);
          if ( vars["xsl_usepublic"] != 0 )
          vars["xsl_usepublic"] = 1;
          else
          vars["xsl_usepublic"] = 0; // not zero type !!!

          if ( (int)vars["xsl_usepublic"] !=
           context->query_attribute("xsl:use_public") )
          {
          context->set_acquire_attribute("xsl:use_public", 0);
          context->set_attribute("xsl:use_public",
                     vars["xsl_usepublic"]);
          }
      }
      
// Disable Link Exchange option because link exchange cant be turned off anymore
/*
      if ( context->get_object_class() & CLASS_CONTAINER )
      {
          LOG("Link Exchange = " + vars["link_exchange"]);
          if ( vars["link_exchange"] != 0 )
          vars["link_exchange"] = 1;
          else
          vars["link_exchange"] = 0; // not zero type !!!

          if ( (int)vars["link_exchange"] !=
           context->query_attribute(CONT_EXCHANGE_LINKS) )
          {
          context->set_acquire_attribute(CONT_EXCHANGE_LINKS, 0);
          context->set_attribute(CONT_EXCHANGE_LINKS,
                     vars["link_exchange"]);
          }
      }
*/

      if ( context->get_object_class() & CLASS_GROUP )
      {
          if ( vars["exclusive_subgroups"] != 0 )
          vars["exclusive_subgroups"] = 1;
          else
          vars["exclusive_subgroup"] = 0;
          context->set_attribute(GROUP_EXCLUSIVE_SUBGROUPS,
                     vars["exclusive_subgroups"]);
      }
      break;
      case "update_stylesheets":
      foreach(indices(vars), string stylesheet) {
          LOG("Var:"+stylesheet);
      }
      break;
      case "update_keywords":
      array arr = context->query_attribute(OBJ_KEYWORDS);
      array new_arr = ({ });

      foreach(indices(vars), string kw) {
          if ( sscanf(kw, "keyword_%d", key) == 1 ) {
          arr[key-1] = get_value(vars[kw]);
          }
      }
      foreach(arr, mixed v) {
          if ( stringp(v) )
          new_arr += ({ v });
      }
      array keys = get_values(vars["keyword_new"]);
      if ( arrayp(keys) && sizeof(keys) > 0 )
          new_arr += keys;
      LOG("New keywords="+sprintf("%O", new_arr));
      object tagmod = get_module("tagging");
      if ( objectp(tagmod) ) {
           tagmod->tag_object(context, new_arr);
      }
      else {
          context->set_attribute(OBJ_KEYWORDS, new_arr);
      }
      break;
      case "addacl":
        xsl = _FILEPATH->path_to_object("/stylesheets/invite.xsl");
        xml =  _XMLCONVERTER->get_xml(context, xsl, vars);
        vars["language"]=l;
        return
          _Server->get_module("libxslt")->run(
          xml,
          xsl,
          vars) + "<!---\n"+xml+"-->";
        break;
      case "invite":
      xsl = _FILEPATH->path_to_object("/stylesheets/invite.xsl");
      xml =  _XMLCONVERTER->get_xml(context, xsl, vars);
      vars["language"]=l;
      return
          _Server->get_module("libxslt")->run(
          xml,
          xsl,
          vars) + "<!---\n"+xml+"-->";
      break;
      case "grp_password":
      err = catch { context->set_group_password(vars["grp_password"]); };
      resMessage = RES("Group Password set !",
               "Gruppen Passwort wurde gesetzt.");
      if ( err != 0 ) {
          resMessage = RES("Failed to set group password",
                  "Gruppen Passwort konnte nicht gesetzt werden.");
          errMessage = RES("No privileges to set group password !",
                   "Sie haben keine Rechte um das Gruppenpasswort zu setzen.");
      }
      break;
      case "grp_delete_password":
      err = catch { context->set_group_password(0); };
      resMessage = RES("Group Password deleted !",
               "Gruppen Passwort wurde gelöscht.");
      if ( err != 0 ) {
          resMessage = RES("Failed to delete group password",
                  "Gruppen Passwort konnte nicht gelöscht werden.");
          errMessage = RES("No privileges to delete group password !",
                   "Sie haben keine Rechte um das Gruppenpasswort zu löschen.");
      }
      break;
      
      
    case "update_group_freeentry":
        if ( vars->grp_freeentry )
            context->sanction_object(_WORLDUSER,SANCTION_INSERT|SANCTION_READ);
        else
            context->sanction_object(_WORLDUSER, 0);
        break;
    case "move_group":
      string oldgroupname = context->get_identifier();
      object dest_grp = find_object((int)vars->togrp);
      if ( !objectp(dest_grp) )
        return error_page("The destination group was not found !",
                           "JavaScript:history.back();");
      if ( !get_factory(CLASS_GROUP)->move_group(context, dest_grp) )
        return error_page(RES_MSG("Failed to move to destination group " +
                               dest_grp->get_identifier(),
                               "Die Gruppe konnte nicht in die Gruppe " +
                               dest_grp->get_identifier()+" bewegt werden.",l),
                           "JavaScript:history.back();");
        string newgroupname = context->get_identifier();
        return "<html><script language=\"javascript\">function my_string_replace(source, replace, search) {var searchlength = search.length;var replacelength = replace.length;var pos = source.indexOf(search, 0);while (pos >= 0){source = source.substring(0, pos) + replace + source.substring(pos + searchlength);pos = source.indexOf(search, pos + replacelength);}return source;} function doClose() {var location = window.opener.document.URL;location = my_string_replace(location, '"+newgroupname+"', '"+oldgroupname+"');window.opener.location = location;window.close();}</script><body onLoad=\"doClose()\"></body></html>";
      break;
      case "grp_name":
        factory = get_factory(CLASS_GROUP);
        err = catch {
           factory->rename_group(context, vars->name);
        };
        if ( err ) {
          errMessage = RES( "Failed to rename the group " +
                            context->get_group_name() + " to " +
                            vars->name + " !",
                            "Das Umbenennen der Gruppe " +
                            context->get_group_name() + " in " +
                            vars->name + " ist fehlgeschlagen !" );
          werror( "Failed to rename group %O to %O: %s\n%O\n",
                  context->get_identifier(), vars->name, err[0], err[1] );
        }
        context->set_attribute(OBJ_DESC, vars->attr_OBJ_DESC);
        vars->_script = "/groups/"+context->get_identifier();
      break;
      case "addbuddies":
          array(object) buddylist;
          int changed=0;
          buddylist = context->query_attribute(USER_FAVOURITES);
          if (!arrayp(buddylist)) buddylist = ({});
          foreach(aoSelected, object nobj) {
            if (objectp(nobj)) {
              if (search(buddylist, nobj) < 0) {
                buddylist += ({nobj});
                changed = 1;
              }
            }
          }
          if (changed) context->set_attribute(USER_FAVOURITES, buddylist);
        break;
      case "acl_add":
          array(object) acl;
          acl = context->query_attribute("OBJ_ACL_ADDS");
          if (!arrayp(acl)) acl = ({});
          foreach(aoSelected, object nobj) {
            if (objectp(nobj)) {
              if (search(acl, nobj) < 0) acl += ({nobj});
            }
          }
          context->set_attribute("OBJ_ACL_ADDS", acl);
        break;
      case "stylesheet_add":
          array(object) ssl;
          ssl = context->query_attribute("OBJ_STYLESHEET_ADDS");
          if (!arrayp(ssl)) ssl = ({});
          foreach(aoSelected, object nobj) {
            if (objectp(nobj)) {
              if (search(ssl, nobj) < 0) ssl += ({nobj});
            }
          }
          context->set_attribute("OBJ_STYLESHEET_ADDS", ssl);
        break;  
      case "add_stylesheet_add": 
//werror("access.pike - add_stylesheet_add : running...\n");
        if (!(vars["toadd"])) {
          werror("browser.pike - add_stylesheet_add : object to add not specified\n");
          break;
        }
        object obuddy = find_object((int)vars["toadd"]);
        if (!objectp(obuddy)) {
          werror("browser.pike - add_stylesheet_add : cant find object to add with id"+vars["toadd"]+"\n");
          break;
        }
        array(object) obl;
        obl = context->query_attribute("OBJ_STYLESHEET_ADDS");
        mapping ss_content = context->query_attribute("xsl:content");
        if (!mappingp(ss_content)) ss_content = ([]);
        if (!arrayp(obl)) obl = ({});
        if ((search(obl, obuddy) <0) && (search(indices(ss_content), obuddy) < 0) ) {
          obl += ({obuddy});
          context->set_attribute("OBJ_STYLESHEET_ADDS", obl);
        }
        break;  
    case "remove_stylesheet_add":
        if (!(vars["toremove"])) {
          werror("browser.pike - remove_stylesheet_add : add to remove not specified\n");
          break; 
        }
        object removestylesheetadd = find_object((int)vars["toremove"]);
        if (!objectp(removestylesheetadd)) {
          werror("browser.pike - remove_stylesheet_add : cant find object to remove with id="+vars["toremove"]+"\n");
          break;
        }        
        mapping mssl_content = context->query_attribute("xsl:content");
        if (!mappingp(mssl_content)) mssl_content = ([]);
        mssl_content = mssl_content - ([ removestylesheetadd:0 ]);
        context->set_attribute("xsl:content", mssl_content);         
        array(object) oblist;
        oblist = context->query_attribute("OBJ_STYLESHEET_ADDS");
        if (!arrayp(oblist)) oblist = ({});
        if (search(oblist, removestylesheetadd) >= 0) {
          oblist -= ({removestylesheetadd});
          context->set_attribute("OBJ_STYLESHEET_ADDS", oblist);
        }
      break;        
      case "grp_invite":
      if ( !(context->get_object_class() & CLASS_GROUP) )
          return error_page(RES_MSG("Context is not a group object !", "Der kontext ist keine Gruppe.", l),
                "JavaScript:history.back();");
      if ( vars->einladen_email ) {
        object script = OBJ("/sources/create_user.pike");
        // direkt per e-mail einladen
        script->invite_user(vars->einladen_email, context);
        // und jetzt eine E-Mail schicken!
        get_module("smtp")->send_mail(vars->einladen_email, "[sTeam] Sie sind eingeladen worden !", "Sie sind zur Mitarbeit in der Gruppe " + context->get_identifier() + " eingeladen worden.<br/>", "noreply@steam.upb.de");
      }
      foreach(aoSelected, obj) {
          if ( obj->get_object_class() & CLASS_USER ) {
          obj->mail(RES(
                "Hello!\r\n<br/>You have been invited to join the "+
                " group "+ context->get_identifier() + "\r\n<br/>"+
                "Please visit the Link below to join the group\r\n<br/>"+
                get_visit_link_group(context),
                "Hallo!\r\n<br/>Sie wurden in die Gruppe "+ context->get_identifier() + " eingeladen.\r\n<br/>"+
                "Bitte folgen Sie untenstehendem Link um der Gruppe beizutreten\r\n<br/>"+
                get_visit_link_group(context)),
                RES("Invitation to join " + context->get_identifier(),
                    "Einladung zur Gruppe " + context->get_identifier()));
          context->invite_user(obj);
          tmod->add_task(obj, this(), "join_invited_group",
                 ({ context, obj }),
                 ([ "english": "Join "+
                    context->get_identifier() + " (invited)",
                    "german": "Gruppe " +
                    context->get_identifier() +
                    " beitreten (eingeladen)", ]));
          }
      }
      vars["_script"] = "/groups/"+context->get_identifier();
      vars["mode"] = "admin";
      resMessage = RES("You invited the users to join the group " +
               context->get_identifier() + " !",
               "Sie haben die Benutzer in die Gruppe " +
               context->get_identifier() + " eingeladen!");
      break;
      case "grp_invite_directadd":
        if ( !(context->get_object_class() & CLASS_GROUP) )
            return error_page(RES_MSG("Context is not a group object !", "Der Kontext ist keine Gruppe.", l),
                  "JavaScript:history.back();");
        foreach(aoSelected, obj) {
            if (!context->is_member(obj)) {
              if ( obj->get_object_class() & CLASS_USER ) {
                int myres = context->add_member(obj);
                // check if user was added (1=added, 0=failed, -1=pending, -2=pending failed)
                if (myres == 1) {
                  obj->mail(RES(
                      "Hello!\r\n<br/>You have been added to the "+
                      "group "+ context->get_identifier() +
                      "\r\n\r\n<br/><br/>"+
                      "To View the group follow the link below:\r\n<br/>"+
                      get_visit_link_group(context)+"\r\n<br/>",
                      "Hallo!\r\n<br/>Sie wurden in die Gruppe "+ context->get_identifier() + " aufgenommen."+
                      "\r\n\r\n<br/><br/>"+
                      "Folgen Sie untenstehendem Link um sich die Gruppe anzusehen:\r\n<br/>"+
                      get_visit_link_group(context)+"\r\n<br/>"),
                      RES("You were added to group " + context->get_identifier(),
                          "Sie wurden zur Gruppe " + context->get_identifier()+" hinzugefuegt"));
                  // create exit in users workroom
                  object owrkr = obj->query_attribute(USER_WORKROOM);
                  if (objectp(owrkr)) {
                    if ( !catch(_SECURITY->check_access( owrkr, user, SANCTION_INSERT, ROLE_READ_ALL, false)) ) {
                      create_group_exit(context, obj);
                    }
                    else {
                      tmod->add_task(obj, this(), "create_group_exit", ({ context, obj }),
                             ([ "english": "Create exit to group " +
                                context->get_identifier(),
                                "german": "Ausgang zur Gruppe " +
                                context->get_identifier() + " erzeugen", ]));
    
                    }
                  }
                  resMessage = RES("You added the users to group " +
                  context->get_identifier() + " !",
                  "Sie haben die Benutzer der Gruppe " +
                  context->get_identifier() + " hinzugefuegt!");
                } else if (myres == -1) {
                  context->remove_member(obj);
                  resMessage = RES("Add users failed. Group is full !",
                  "Benutzer konnten nicht hinzugefügt werden, die Gruppe ist voll");
                }
              }
            }
        }
        vars["_script"] = "/groups/"+context->get_identifier();
        vars["mode"] = "admin";
      break;
      case "grp_removeinvitation":
        aoSelected = get_selected(vars, ({ "invited" }));
        foreach(aoSelected, obj)
          context->remove_invite(obj);
        resMessage = RES("You removed the invitations for the users!",
                         "Sie haben die Einladungen zurueck genommen!");
      break;
      case "grp_create":
          factory = _Server->get_factory(CLASS_GROUP);
          if ( vars["subgroup"] != "none" ) {
              object subgroup = find_object((int)vars["subgroup"]);
              vars["parentgroup"] = subgroup; // strange naming
          }
          err = catch {
              obj = factory->execute( ([ "name": vars["new_group_name"],
                                       "parentgroup": vars["parentgroup"],
                                       ]) );
              obj->set_attribute(OBJ_DESC, vars["new_group_description"]);
              obj->add_member(this_user());
              // for redirection
              vars->id      = (string)obj->get_object_id();
              vars->_script = "/scripts/navigate.pike";
              create_group_exit(obj, this_user());
          };
          if ( err != 0 ) {
              werror("Failed to create group !!!\n"+sprintf("%O", err));
              resMessage = RES(err[0], err[0]);
              errMessage = RES("Failed to create group !",
                               "Das Anlegen der Gruppe ist fehlgeschlagen !");
          }
          else {
              errMessage = RES("Group created.",
                               "Gruppe wurde erfolgreich angelegt.");
              errMessage = 0;
          }
#if 0
      if ( vars["subgroup"] != "none" ) {
          err = catch {
          object subgroup = find_object((int)vars["subgroup"]);
          LOG("Subgroup found: " + subgroup->get_identifier());
          subgroup->add_member(obj);
          };
              if ( err == 0 ) {
          errMessage = 0;
          resMessage = "Group created and subgroup set.";
          }
          else {
          errMessage = "Failed to set group as subgroup.";
          resMessage = "Group created, but no subgroup defined.";
          LOG("Failed to defined subgroup:\n"+err[0]+"\n"+sprintf("%O", err[1]));
          }
      }
#endif
      break;
      case "send_message":
        object touser = find_object((int)vars["touser"]);
        if ( !objectp(touser) ) {
          resMessage = RES("Failed to send mail to user !",
                   "Das Senden der Nachricht ist fehlgeschlagen.");
          errMessage = RES("Failed to find destination of mail !",
                  "Der Empfaenger konnte nicht ermittelt werden.");
        }
        else {
          string message_name = replace(vars->messagesubject,"/","_");
          obj = get_factory(CLASS_DOCUMENT)->execute( ([ "name":message_name, "mimetype":"text/plain"]));
          obj->set_attribute(OBJ_DESC, vars->messagesubject);
          obj->set_content(vars->messagebody);
          obj->set_attribute(DOC_ENCODING, "utf-8");
          string url = vars["annotfile.filename"];
          if ( stringp(url) && strlen(url) > 0 &&
           stringp(vars["annotfile"])  )
          {
          factory = _Server->get_factory(CLASS_DOCUMENT);
          ann = factory->execute( ([ "name":url, "acquire":obj, ]) );
          function f = ann->receive_content(strlen(vars["annotfile"]));
          f(vars["annotfile"]);
          f(0);
          obj->add_annotation(ann); 
          }
          touser->mail(obj);
          resMessage = RES("Mail successfully send !",
                   "Nachricht ist erfolgreich versandt worden !");
        }
        break;
		
	  
	  // Erweiterte Mailingfuntionalitaet fuer die neue Mailbox
	  // moritz: 23.05.07
	  //
	  case "send_message_extended":
//	  werror("\n-----------------\nSind in \"send_message_extended\".\n");
	  array tousers = ({ });
	  array(string) invalid = ({ });
	  object smtp = get_module("smtp");
	  object mail;
	  int noContent = 0;
	  int produceWarnings = 0;
	  string errTextG = "<h2>Das Senden Ihrer Nachricht ist leider fehlgeschlagen!</h2>Gründe:<br />";
	  string errTextE = "<h2>Failed to send Message!</h2>Causes:<br />";
	  
	  // Auswertung der Auswahlliste
	  if (arrayp(vars["to_own"])) {
		  array(string) to_own = vars["to_own"];
//		  werror("Ist Array! Es wurden "+ sizeof(to_own) +" Gruppen bzw. Freunde ausgewählt, und zwar:\n");
		  for(int i = 0; i < sizeof(to_own); i++) {
//			werror(to_own[i]+"\n");
			tousers += ({ find_object((int)to_own[i]) });
		  }
	  }else if (stringp(vars["to_own"])) {
		  string to_own = vars["to_own"];
//		  werror("Ist ein String! Es wurde nur eine Gruppe bzw. ein Freund ausgewählt, und zwar: "+to_own+"\n");
		  tousers += ({ find_object((int)to_own) });
	  }else {
    //werror("Es wurde nichts Ausgewählt!");
    }
	  
	  // Auswertung der freien Eingabe von Benutzern und Gruppen
	  array(string) to_free;
	  array(string) help;
	  if(stringp(vars["to_free"]) && sizeof(vars["to_free"]) != 0) {
//		  werror("Der Ausgangsstring von to_free ist: " + vars["to_free"] + "\n");
		  to_free = vars["to_free"] / ",";
		  
//		  werror("Die Adressaten sind:\n");
		  foreach(to_free, string elem) {
			  elem = String.trim_all_whites(elem);
			  if(elem != "" && !isEmptyString(elem)) {
//				  werror(elem + ": ");
				  if(objectp(get_module("users")->lookup(elem))) {
//					  werror("gültiger steamUser\n");
					  tousers += ({ get_module("users")->lookup(elem) });
				  }else if(objectp(get_module("groups")->lookup(elem))){
//					  werror("gültige steamGruppe\n");
					  tousers += ({ get_module("groups")->lookup(elem) });
				  }else if(mailAdressValid(elem)) {
//					  werror("syntaktisch gültige email-Adresse\n");
					  tousers += ({ elem });
			      }else {
//					  werror("kein gültiger steamUser, keine gültige steamGruppe und keine syntaktisch korrekte Mailadresse!\n");
					  invalid += ({ elem });
				  }
			  }
		  }
	  }else {
    //werror("Die Eingabe bei to_free ist leer!\n");
    }	
	  
	  //Das Erzeugen der eigendlichen Mail
	  if(sizeof(tousers) != 0 && stringp(vars["messagesubject"]) && vars["messagesubject"] != "") {
//		  werror("OK. Wir haben Adressaten und ein Subject...\n");
		  
		  string body = "";
		  
		  string message_name = replace(vars->messagesubject,"/","_");
		  mail = get_factory(CLASS_DOCUMENT)->execute( ([ "name":message_name, "mimetype":"text/plain"]));
		  mail->set_attribute(OBJ_DESC, vars->messagesubject);
		  if(stringp(vars["messagebody"]) && vars["messagebody"] != "") {
			  mail->set_content(vars->messagebody);
			  body = vars->messagebody;
		  }else {
//			  werror("Kein Inhalt eingegeben.\n");
			  noContent = 1;
		  }
		  mail->set_attribute(DOC_ENCODING, "utf-8");
		  string url = vars["annotfile.filename"];
		  if ( stringp(url) && strlen(url) > 0 &&
		  stringp(vars["annotfile"])  ) {
			  factory = _Server->get_factory(CLASS_DOCUMENT);
			  ann = factory->execute( ([ "name":url, "acquire":obj, ]) );
			  function f = ann->receive_content(strlen(vars["annotfile"]));
			  f(vars["annotfile"]);
			  f(0);
			  mail->add_annotation(ann); 
		  }
//      werror("SENDE MAILS !!");
      array(string) stringadresses = ({}); 
		  foreach(tousers, mixed user) {
			  if(objectp(user)) {
//          werror("Sending Mail to: " + user->get_identifier());
          if (objectp(mail->get_annotating())) {
//            werror("Object "+mail->get_id()+" is already annotated. duplicating...");
            mail = mail->duplicate();
          }
				  user->mail(mail);
//				  werror("Mail versended an " + user->get_user_name() + ".\n");
			  }
			  else if(stringp(user)) {
				  stringadresses += ({ user });
			  }
		  }
      
      if (sizeof(stringadresses) > 0) {
        smtp->send_mail(stringadresses, vars["messagesubject"], body);
      }
      
		  resMessage = RES("Mail successfully sent !",
			   "Nachricht ist erfolgreich versandt worden !");
			   
		  if(sizeof(invalid) != 0 || noContent == 1) {
			  produceWarnings = 1;
			  errTextG = "<h2>Nachricht ist erfolgreich versandt worden!</h2>Aber es gab Warnungen:<ol>";
		  	  errTextE = "<h2>Mail successfully send!</h2>But there were some warnings:<ol>";
			  
			  if(sizeof(invalid) != 0) {
				  errTextG += "<li>Es gab ungültige Eingaben. Zu den Eigaben<ul>";
				  errTextE += "<li>There were invalid entries. The entries<ul>";
				  foreach(invalid, string s) {
						errTextG += "<li>"+s+"</li>";
						errTextE += "<li>"+s+"</li>";
				  }
				  errTextG += "</ul> konnten keine sTeam-Benutzer oder -Gruppen gefunden werden. Es wurden <em>keine</em> Nachrichten an sie Versandt!</li></ul>";
				  errTextE += "</ul> do not seem to be valid users or groups in your sTeam system. No messags have been send there!</li></ul>";
			  }	  
			  if(noContent == 1) {
				  errTextG += "<li>Ihre Nachricht enthält keinen Text.</li></ol>";
				  errTextE += "<li>Your message contains no text.</li></ol>";
			  }else{
				  errTextG += "</ol>";
				  errTextE += "</ol>";
			  }
		  }
		  
		  if(produceWarnings)errMessage = RES(errTextE, errTextG);
		 
	  } else {
//		werror("Wir haben keine gültigen Adressaten oder kein Subject.\n");
		errTextG += "<ol>";
		errTextE += "<ol>";
		if(sizeof(tousers) == 0) {
			errTextG += "<li>Es gibt keine gültigen Empfänger für Ihre Nachricht, denn ";
			errTextE += "<li>There are no valid recipients for your message because ";
			if(sizeof(invalid) == 0) {
				errTextG += "es wurden keine Empfänger für die Nachricht ausgewählt oder eingegeben.</li>";
				errTextE += "you have not specified any for your message.</li>";
			} else {
				errTextG += "es wurden nur ungültige Empfänger eingegeben:<ul>";
				errTextE += "you only entered invalid recipients:<ul>";
				foreach(invalid, string s) {
					errTextG += "<li>"+s+"</li>";
					errTextE += "<li>"+s+"</li>";
				}
				errTextG += "</ul></li>";
				errTextE += "</ul></li>";
			}
		}
		if(stringp(!vars["messagesubject"]) || vars["messagesubject"] == "") {
			errTextG += "<li>Es fehlt der Betreff Ihrer Nachricht. Bitte geben Sie einen an.</li></ol>";
			errTextE += "<li>No subject is specified for your message. Please give an subject.</li></ol>";
		}
		
        resMessage = RES("Failed to send mail!",
                         "Das Senden der Nachricht ist fehlgeschlagen.");
        errMessage = RES(errTextE, errTextG);
	  }
        break;	
		
      case "grp_add":
      uSelect = extract_objects(vars["non_member"]) +
          extract_objects(vars["request"]);
      foreach(uSelect, obj) {
          if ( context->add_member(obj) ) {
          string messagelanguage = (obj->query_attribute(USER_LANGUAGE)!=0?obj->query_attribute(USER_LANGUAGE):"english");
          string messagebody =
                RES_MSG(
                "You have been added to the group "+
                context->get_identifier()+"\r\n\r\n<br/><br/>"+
                "To view the group visit the link below\r\n<br/>"+
                get_visit_link_group(context),
                "Sie wurden zur Gruppe "+
                context->get_identifier()+" hinzugefuegt\r\n\r\n<br/><br/>"+
                "Um die gruppe anzusehen folgen Sie untenstehendem Link\r\n<br/>"+
                get_visit_link_group(context),
                messagelanguage);
          string messagesubject=
                RES_MSG("Welcome to "+context->get_identifier(),"Willkommen in der Gruppe "+ context->get_identifier(),messagelanguage);
          obj->mail(messagebody, messagesubject);
          tmod->add_task(obj, this(), "create_group_exit", ({ context, obj }),
                 ([ "english": "Create exit to group " +
                    context->get_identifier(),
                    "german": "Ausgang zur Gruppe " +
                    context->get_identifier() + " erzeugen", ]));
          }
      }
      break;
      case "grp_admins":
      uSelect = extract_objects(vars["member"]);
      foreach(uSelect, obj) {
        context->sanction_object(obj, SANCTION_ALL);
        context->sanction_object_meta(obj, SANCTION_ALL);
      }
      break;
      case "grp_remove":
      uSelect = extract_objects(vars["member"]);

      if ( context->get_object_id() == _STEAMUSER->get_object_id() ) {
          resMessage = RES("User Deletion:<BR/>",
                   "Loeschen von Benutzern:<BR/>");
          foreach(uSelect, obj) {
          string uname = obj->get_identifier();
          err = catch {
              obj->delete();
          };
                  if ( err != 0 ) {
              return error_page("While deleting: " +uname+"<BR/>"+
                    err[0]+"<BR/>"+
                    sprintf("%O",
                        replace(err[1],"\n","<BR/>")),
                    "JavaScript:history.back();");
          }
          resMessage->english += "Deleted user: " + uname + "<BR/>";
          resMessage->german += "Benutzer geloescht: "+uname + "<BR/>";
          }
          return result_page(resMessage[client_map->language],
                 "JavaScript:history.back();");
      }
      foreach(uSelect, obj) {
          if ( context->remove_member(obj) ) {
          // removing subgroup
          if ( obj->get_object_class() & CLASS_GROUP )
          {
              obj->mail("The subgroup "+ obj->get_identifier()+
                " has been removed from the group "+
                context->get_identifier(),
                "Subgroup " + obj->get_identifier()+
                " removed from " + context->get_identifier());
              obj->delete();
          }
          else
          {
              string messagelanguage = (obj->query_attribute(USER_LANGUAGE)!=0?obj->query_attribute(USER_LANGUAGE):"english");
              obj->mail(
                RES_MSG("You have been removed from the group "+
                context->get_identifier()+"\r\n\r\n<br/><br/>"+
                "To view the group visit the link below\r\n<br/>"+
                get_visit_link_group(context),
                "Sie wurden aus der Gruppe "+context->get_identifier()+" entfernt\r\n\r\n<br/><br/>"+
                "Um die Gruppe anzusehen folgen Sie untenstehendem Link\r\n<br/>"+ 
                get_visit_link_group(context),
                messagelanguage)
                ,
                RES_MSG(
                "Bye from " + context->get_identifier(),
                "Auf Wiedersehen in der Gruppe "+context->get_identifier(),
                messagelanguage)
                );
              tmod->add_task(obj, this(), "remove_group_exit",
                     ({ context, obj }),
                     ([ "english": "Remove Gate to " +
                    context->get_identifier() +
                    " (membership canceled)",
                    "german": "Ausgang zur Gruppe "+
                    context->get_identifier()+" löschen "+
                    "(Mitgliedschaft aufgehoben)",
                    ]));
          }
          }
      }
      resMessage = RES(
          "Successfully removed selected members from the group.",
          "Die Benutzer sind erfolgreich aus der Gruppe entfernt worden.");
      break;
      case "grp_delete":
          err = catch {
              context->delete();
          };
          if ( err == 0 ) {
              errMessage = 0;
              resMessage = RES("Successfully deleted group !",
                               "Gruppe wurde erfolgreich gelöscht !");
              vars->_script = "/scripts/groups.pike";
              m_delete(vars, "id");

          }
          else {
              errMessage = RES("Failed to delete group: " +
                               context->get_identifier(),
                               "Die Gruppe " +
                               context->get_identifier() +
                               " konnte nicht geloescht werden !");
              resMessage = RES("Deletion of group failed !",
                               "Das Löschen der Gruppe ist fehlgeschlagen !");
              werror("Failed to delete group:"+err[0]+"\n"+sprintf("%O",err[1]));
          }
          break;
    case "unpack":

        break;
    case "grp_deny":
      uSelect = extract_objects(vars["request"]);

      array memReqs = context->query_attribute(GROUP_MEMBERSHIP_REQS);

      foreach(uSelect, obj) {
          if ( arrayp(memReqs) ) {
              if ( search(memReqs, obj) >= 0 )
                  memReqs -= ({ obj });
          }
          string messagelanguage = (obj->query_attribute(USER_LANGUAGE)!=0?obj->query_attribute(USER_LANGUAGE):"english");
          obj->mail(
            RES_MSG("Your request for membership in the group "+
                    context->get_identifier() + " has been denied !",
                    "Ihre Anfrage auf Mitgliedschaft fuer die Gruppe "+context->get_identifier()+
                    "wurde abgelehnt.",
                    messagelanguage)
            ,
            RES_MSG(
                    context->get_identifier() + ": membership request denied",
                    context->get_identifier() + ": Mitgliedschaft wurde abgelehnt",
                    messagelanguage)
            );
      }
      context->set_attribute(GROUP_MEMBERSHIP_REQS, memReqs);
      resMessage = RES("Denied membership in group to selected users !",
                       "Die Aufnahme der Benutzer wurde abgelehnt !");
      break;
    }

    run_event(EVENT_EXECUTE, CALLER, this_object());

    string err_msg = (mappingp(errMessage) ? errMessage[client_map->language]:
                      "");
      string rurl = redirection_url(resMessage[l],
                                  "", vars, context);
      if ( mappingp(errMessage) ) {
          return error_page(errMessage[l], rurl);
      }
      return "<html><head><meta http-equiv=\"refresh\" content=\"0;URL="+
          rurl+"\"/></head></html>\n";
}

#define SEARCH_TIME 600

void clean_searches()
{
    foreach(indices(mSearch), string s) {
	int t;
  array x = s / "_";
  if (sizeof(x) < 2) {
    m_delete(mSearch, s);
    continue;
  }
 	sscanf(x[sizeof(x)-1], "%d", t);
	if ( time() - t > SEARCH_TIME )
	    m_delete(mSearch, s);
    }
}

/**
 *
 *
 * @param
 * @return
 * @see
 */
void change_email(object user, string oldemail, string newemail)
{
  object FORWARD;
  FORWARD = _Server->get_module("forward");
  if (!objectp(FORWARD))
    return;
  string nf = newemail;
  array forwards = FORWARD->get_forward(user);
  if (!arrayp(forwards)) forwards = ({});
  if (arrayp(forwards)) {
     forwards -= ({oldemail});
     forwards += ({newemail});
     FORWARD->delete_forward(user);
     foreach(forwards, string fw)
        FORWARD->add_forward(user, fw);
  } else werror("Browser.pike- change_email(): forwards are no array");
}


/**
 *
 *
 * @param
 * @return
 * @see
 */
mixed handle_copy(object context, mapping vars)
{
    object obj;
    int    num;

    array(object) selects = this_user()->query_attribute(USER_SELECTION);

    num = 0;
    foreach( aoSelected, obj ) {
      mixed err = catch {
        LOG("Copy of " + obj->get_identifier() + "!!\n");
        object dup = obj->duplicate(true);
        dup->move(this_user());
        selects += ({ dup->this() });
      };
      if ( err == 0 )
        num++;
      else
        FATAL("Error on copy:" + sprintf("%O", err));
    }
    this_user()->set_attribute(USER_SELECTION, selects);
    mCopy[vars->copy_id] = RES((string)num+ " Objects copied and taken",
                (string)num+ " Objekte kopiert und aufgenommen");
    return ({ RES((string)num+ " Objects copied and taken",
          (string)num+ " Objekte kopiert und aufgenommen"), 0 });
}

/**
 * @param
 * @return
 * @see
 */
 mixed handle_emptytrashbin(object context, mapping vars) {
  object obj;
  
  foreach(context->get_inventory(), obj ) {
    mixed err = catch {
        obj->delete();
    };
    if ( err != 0 )
        FATAL("Error deleting...\n"+sprintf("%O", err));
  }
  
  if ( sizeof(context->get_inventory()) == 0 )
    mTrash[vars->trash_id] = RES("Trashbin emptied!", "Papierkorb entleert");
  else
    mTrash[vars->trash_id] = RES("A problem occured while emptying the trashbin",
                     "Beim Leeren des Papierkorbs gab es ein Problem");
}

/**
 *
 *
 * @param
 * @return
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>)
 * @see
 */
mixed handle_link(object context, mapping vars)
{
    object obj;
    mapping errMsg = 0;
    int num = 0;

    object factory = _Server->get_factory(CLASS_LINK);
    object exit_factory = get_factory(CLASS_EXIT);
    array(object) selects = this_user()->query_attribute(USER_SELECTION);

    foreach( aoSelected, obj ) {
    mixed err = catch {
      object link;
      if ( obj->get_object_class() & CLASS_ROOM )
        link = exit_factory->execute(
                    ([ "name": obj->query_attribute(OBJ_NAME),
                       "exit_to": obj, ]));
      else
        link = factory->execute(
                    ([ "name": obj->query_attribute(OBJ_NAME),
                       "link_to": obj, ]));
      link->move(this_user());
      selects += ({ link });
    };
    if ( err == 0 )
      num++;
    else {
        if ( !mappingp(errMsg) )
        errMsg = RES("", "");
        errMsg->english +=
        "Failed to create link to " +obj->query_attribute(OBJ_NAME)+"\n";
        errMsg->english +=
        "Das Anlegen des Links auf " +obj->query_attribute(OBJ_NAME)+
        "ist fehlgeschlagen.\n";
        FATAL("Error while creating link....\n"+
          sprintf("%O", err));
    }
    }
    this_user()->set_attribute(USER_SELECTION, selects);
    return ({ RES(
          num + " Links created and taken",
          num + " Links erzeugt und aufgenommen."),
          errMsg });
}

/**
 *
 *
 * @param
 * @return
 * @author Thomas Bopp (astra@upb.de)
 * @see
 */
mixed
handle_delete(object user, mapping vars, object context)
{
    object  obj;
    string html;
    mixed   err;
    int  result;
    int num = 0;
    mapping errMsg = 0;
    array objects = copy_value(aoSelected);

    object creator, trashbin, env, environment;

    trashbin = user->query_attribute(USER_TRASHBIN);
    env = context;
    while ( objectp(env) ) {
	environment = env;

	if ( environment->get_object_class() & CLASS_TRASHBIN )
	    break;

	object trashs = env->get_inventory_by_class(CLASS_TRASHBIN);
	if ( arrayp(trashs) && sizeof(trashs) >= 1 ) {
	    // check access
	    if (_SECURITY->get_user_permissions(trashs[0], this_user(), SANCTION_INSERT) & SANCTION_INSERT)
	       trashbin = trashs[0];
	}
	env = env->get_environment();
    }


    foreach(aoSelected, obj) {
      string selectn = copy_value(obj->get_identifier());
      LOG("Deleting: " + selectn);
    
      creator = obj->get_creator();
      if ( !objectp(environment) )
          environment = obj->get_environment();
      err = catch {
        if ( obj->get_object_class() & CLASS_TRASHBIN )
          throw(({"cannot delete trashbin", backtrace()}));
        else if ( objectp(environment) && environment->get_object_class() & CLASS_TRASHBIN ) {
          obj->delete();
        }
        else if ( obj->get_object_class() & CLASS_LINK )
        {
          if ( obj->get_object_class() & CLASS_EXIT )
          {
              object link = obj->get_link_object();
              if ( objectp(link) && link->get_object_class()&CLASS_EXIT )
                link->move(trashbin);
          }
          obj->move(trashbin);
        }
        else if ( objectp(obj->get_annotating()) ) {
          object father = obj->get_annotating();
          catch(father->remove_annotation(obj));
          obj->move(trashbin);
        }
        else {
          obj->move(trashbin);
        }
      };
      if ( err == 0 )
          num++;
      else {
          FATAL("Deletion of " + obj->get_identifier() + " failed !\n"+
          sprintf("%O\n", err));
          if ( !mappingp(errMsg) ) errMsg = RES("", "");
          errMsg->english += "deletion of " + obj->get_identifier()+"("+
        obj->get_object_id() + ") failed : "+
        err[0];
          errMsg->german += "loeschen von " + obj->get_identifier()+"("+
        obj->get_object_id() + ") ist fehlgeschlagen : "+
        err[0];
      }
      if ( err != 0 )
          html += "<p>deletion "+ selectn +
        " failed due to permissions !</p>\n";
      else if ( !result )
          html += "<p>deletion "+ selectn +
        " failed !</p>\n";
      else
          html += "<p>deletion "+ selectn +
        " successfull ...</p>\n";
    }
    array(object) selection = user->query_attribute(USER_SELECTION);
    if ( !arrayp(selection) ) selection = ({ });
    user->set_attribute(USER_SELECTION, selection - aoSelected);

    return ({ RES(
	num + " Objects deleted",
	num + " Objekte geloescht"),
		  errMsg });
}

/**
 *
 *
 * @param
 * @return
 * @author Thomas Bopp (astra@upb.de)
 * @see
 */
mixed handle_move(object user, mapping vars)
{
    int      target = (int)vars["id"];
    object tobj = find_object(target);
    object                        obj;
    int result;
    int num = 0;
    mapping errMsg = 0;

    array(object) moved = ({ });

    if ( objectp(tobj) ) {
    foreach(aoSelected, obj) {
        mixed err = catch {
        result = obj->move(tobj);
        };
        if ( err == 0 )
        {
        num++;
        moved += ({ obj });
        }
        else {
        if ( !mappingp(errMsg) )
            errMsg = RES("","");
        errMsg->english += replace(
            "Failed to move " + obj->get_identifier() +
            " (lack of permission ?)\n", "'", "\\'" );
        errMsg->german += replace(
            "Bewegen von " + obj->get_identifier() +
            " fehlgeschlagen (keine Berechtigungen ?)\n", "'", "\\'" );
        }
    }
    }
    else {
    return ({ "No destination for move", "" });
    }
    user->set_attribute(USER_SELECTION, aoSelected - moved);
    return ({ RES(
          num + " Objects moved to "+tobj->get_identifier(),
          num + " Objekte bewegt nach "+tobj->get_identifier()),
          errMsg });
}

array oeq(string|int value)
{
   return ({ "=", (string)value });
}
array olt(string|int value)
{
    return ({ "<", (string)value });
}
array ogt(string|int value)
{
    return ({ ">", (string)value });
}

mixed handle_search(object user, mapping vars, void|string l)
{
  if ( stringp(vars["keyword_new"]) && search(vars->keyword_new, "\0") >= 0 )
    vars->keyword_new = "";

  /* + "advsearch_creator":"",
     "advsearch_datefrom":"TT.MM.JJJJ",
     "advsearch_dateto":"TT.MM.JJJJ",
     "advsearch_fulltext":"no",
     ? "advsearch_maxsize":"",
     + "advsearch_mimetype":"all",
     ? "advsearch_minsize":"",
     + "advsearch_modifiedby":"",
     + "advsearch_objtype":"all", */

    array(string) asFilterClasses =
        ([
            "document" : ({"\"/classes/Document\"", "\"/classes/DocGraphics\"", "\"/classes/DocText\"",
			       "\"/classes/DocMP3\""}),
            "container" : ({"\"/classes/Container\"", "\"/classes/Room\"" }),
            "gate" : ({ "\"/classes/Exit\"" }),
            "room" : ({ "\"/classes/Room\"" }),
            "messageboard" : ({"\"/classes/Messageboard\""}),
            "galery" : ({"\"/classes/Container\"", "\"/classes/Room\"" }),
            "link" : ({ "\"/classes/Link\"", "\"/classes/DocExtern\"" }),
            "user" : ({ "\"/classes/User\"" }),
            "group" : ({ "\"/classes/Group\"" }),
            "user+group":  ({ "\"/classes/Group\"", "\"/classes/User\"" }), ])[vars["advsearch_objtype"]];

    if ( !objectp(_SEARCH) )
      _SEARCH = get_module("searching");
    if ( !objectp(_SEARCH) )
      error("FATAL Server error - failed to find searching module !");

    array(object) oSearchResult;
    string sQuery;
    sQuery=vars["keywords"];
    sQuery = "%"+sQuery+"%";

    if ( !arrayp(asFilterClasses) )
      asFilterClasses = ({ });
    vars->user = this_user()->get_object_id();
    object oQueryExecuter = _SEARCH->searchQuery(search_finished, vars, asFilterClasses);
    oQueryExecuter->first_query(STORE_ATTRIB, OBJ_NAME,
				oQueryExecuter->like(sQuery),
				asFilterClasses);
    if (vars["advsearch_objtype"]!="user") {
      oQueryExecuter->extend(STORE_ATTRIB, OBJ_KEYWORDS,
           oQueryExecuter->like(sQuery));
      oQueryExecuter->extend(STORE_ATTRIB, OBJ_DESC,
           oQueryExecuter->like(sQuery));
    }

 if (vars["advsearch_objtype"]=="user" || vars["advsearch_objtype"]=="user+group")
   {
     oQueryExecuter->extend(STORE_ATTRIB, USER_FIRSTNAME,
			    oQueryExecuter->like(sQuery));
     oQueryExecuter->extend(STORE_ATTRIB, USER_FULLNAME,
			    oQueryExecuter->like(sQuery));
     oQueryExecuter->extend(STORE_ATTRIB, USER_EMAIL,
			    oQueryExecuter->like(sQuery));
   }
 if (stringp(vars->advsearch_fulltext) && vars["advsearch_fulltext"]!="no")
   oQueryExecuter->extend_ft(vars["keywords"]);

#ifdef OBJ_VERSIONOF
 oQueryExecuter->limit(STORE_ATTRIB, OBJ_VERSIONOF, oQueryExecuter->eq(0));
#endif

 if ( vars->advsearch_objtype=="document" &&
      vars["advsearch_mimetype"]!="all")
   {
     oQueryExecuter->limit(STORE_ATTRIB, DOC_MIME_TYPE,
			   oQueryExecuter->eq(vars["advsearch_mimetype"]));
   }
 if (stringp(vars["advsearch_creator"]) && strlen(vars->advsearch_creator) > 0 ) {
   object oCreator = USER(vars["advsearch_creator"]);
   if (objectp(oCreator)) {
     string sCreator="%"+oCreator->get_object_id();
     oQueryExecuter->limit(STORE_ACCESS, "Creator", oQueryExecuter->eq(sCreator));
   }
 }
 if ( stringp(vars["advsearch_modifiedby"]) && strlen(vars->advsearch_modifiedby) > 0 ) {
   object oEditor = USER(vars["advsearch_modifiedby"]);
   if (objectp(oEditor)) {
     string sEditor="%"+oEditor->get_object_id();
     oQueryExecuter->limit(STORE_ATTRIB, DOC_USER_MODIFIED, oQueryExecuter->eq(sEditor));
   }
 }

 if ( stringp(vars["advsearch_maxsize"]) && strlen(vars->advsearch_maxsize) > 0 ) {
   int maxsize = (int) vars["advsearch_maxsize"];
   oQueryExecuter->limit(STORE_CONTENT, "CONTENT_SIZE", olt(maxsize));
 }
 if ( stringp(vars["advsearch_minsize"]) && strlen(vars->advsearch_minsize) > 0 ) {
   int minsize = (int) vars["advsearch_minsize"];
   oQueryExecuter->limit(STORE_CONTENT, "CONTENT_SIZE", ogt(minsize));
 }

 object calendar = Calendar.ISO->set_language("german");
 string sdate = vars["advsearch_datefrom"];
 if (sdate !="TT.MM.JJJ") {
   mixed err = catch{
     Calendar.Day date = calendar->dwim_day(sdate);
     oQueryExecuter->limit(STORE_ATTRIB, OBJ_LAST_CHANGED,
                                  ogt(date->unix_time()));
   };
 }
 sdate = vars["advsearch_dateto"];
 if (sdate !="TT.MM.JJJJ") {
   mixed err = catch{
     Calendar.Day date = calendar->dwim_day(sdate);
     oQueryExecuter->limit(STORE_ATTRIB, OBJ_LAST_CHANGED,
			   olt(date->unix_time()));
   };
 }
 return oQueryExecuter->run_async();
}

mixed isReadable(object obj, object user) {
  if ( !objectp(obj) ) return 0;
  if ( _SECURITY->check_user_access(obj, user, SANCTION_READ, ROLE_READ_ALL, false) ) return obj;
  return 0;
}

static object objectify(string res)
{
    return find_object((int)res);
}

string handleSearchResult(mixed id, array oSearchResult)
{
    mapping vars = id->vars;
    object user = find_object(vars->user);
    vars->this_user = vars->user;
    catch(seteuid(user));
    oSearchResult = map(oSearchResult, objectify);
    vars->client = ([ ]);
    array resultList = ({ });
    int num = 0;
    for (int i = 0; i < sizeof(oSearchResult); i++) {
        if ( isReadable(oSearchResult[i], user) ) {
            resultList += ({ oSearchResult[i] });
            num++;
        }
        if ( num >= 200 )
            break;
    }

    mSearch[id->vars->search_id] =
	html_from_list(
	    get_selection(user, 0),
        resultList,
	    vars,
	    _FILEPATH->path_to_object("/stylesheets/selection.xsl"
		)
	    );
    return "<script type='text/javascript'>window.location.href = '/scripts/browser.pike?_action=search&search_id=" + vars->search_id + "&room="+vars->room+"';\n</script>";
}

void search_finished(mapping vars, array oSearchResult)
{
  object user = find_object(vars->user);
  vars->this_user = vars->user;
  mSearch[vars->search_id] =
    html_from_list(get_selection(user, 0), 
		   map(oSearchResult, isReadable, user), 
		   vars,
		   _FILEPATH->path_to_object("/stylesheets/selection.xsl"));
}

object find_father_annotation(object current, object ann)
{
    array(object) annotations;

    annotations = current->get_annotations();
    if ( search(annotations, ann) >= 0 )
    return current;

    foreach(annotations, object a) {
    object res = find_father_annotation(a, ann);
    if ( objectp(res) )
        return res;
    }
    return 0;
}

/**
 *
 *
 * @param
 * @return
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>)
 * @see
 */
mixed handle_delete_annotation(object user, mapping vars)
{
    object annotation, ann, father;
    mapping resMessage, errMessage;
    int                          i;


    errMessage = 0;
    resMessage = RES("","");

    annotation = find_object((int)vars["todelete"]);
    array(object) annotations;

    father = find_object((int)vars["id"]);
    father = find_father_annotation(father, annotation);

    annotations = ({ annotation });

    foreach(annotations, ann) {
    if ( !objectp(ann) )
        continue;
    mixed err = catch {
        _SECURITY->access_delete(0, ann, user);
    };
    if ( err != 0 ) {
        if ( !mappingp(errMessage) )
        errMessage = RES("","");
        errMessage->english += "You have no access to delete " +
        ann->get_identifier() + "\n";
        errMessage->german += "Sie haben keine Rechte um " +
        ann->get_identifier() + " zu loeschen.\n";
    }
    }
    if ( mappingp(errMessage) )
    return ({ RES(
              "Failed to remove Annotation thread !",
              "Loeschen der Annotationen ist fehlgeschlagen !"),
              errMessage });
              
              
   //LOG("Removing annotation from father-object="+father->get_object_id() +":"+father->get_identifier());
   
    catch(father->remove_annotation(annotation));
    foreach(annotations, ann) {
    if ( !objectp(ann) )
        continue;
    ann->delete();
    }

    return ({ RES(
          "Removed annotation (thread) !",
          "Annotationen wurden geloescht (thread) !"),
          errMessage });
}

/**
 *
 *
 * @param
 * @return
 * @author Thomas Bopp (astra@upb.de)
 * @see
 */
mixed handle_zip(object user, mapping vars)
{
    object         factory,  zip, obj;
    int                        result;
 
    factory = _Server->get_factory(CLASS_DOCUMENT);
    zip = factory->execute(([ "name":time()+".zip"]));
    zip->move(user);
    user->set_attribute(USER_SELECTION, ({ }));
}

object create_local_annotation(mapping vars)
{
    string result, mimetype;
    // first check the annotation for well-formdness
    LOG("content:\n"+vars["annotbody"]);

    object factory = _Server->get_factory(CLASS_DOCUMENT);
    object obj, attached;

    attached = 0;
    if ( stringp(vars["annotfile.filename"]) &&
         strlen(vars["annotfile.filename"]) > 0 )
    {
        string fname = basename(vars["annotfile.filename"]);
        array tokens = fname /".";
        string ext;
        if ( sizeof(tokens) >= 2 )
            ext = tokens[-1];
        else
            ext = "";
        mimetype = _TYPES->query_mime_type(lower_case(ext));

        attached = factory->execute((["name":fname,"mimetype":mimetype,]));
        attached->set_attribute("attachement", true);
        attached->set_content(vars["annotfile"]);
    }

    if ( vars["annottype"] == "1" )
        obj = find_object((int)vars["active"]);
    else
        obj = find_object((int)vars["id"]);

    object annotation;

    if ( !objectp(obj) )
    THROW("No Annotation context !", E_ERROR);

    mimetype = "text/plain";
    if ( vars->doc_mimetype )
        mimetype = vars->doc_mimetype;

#ifdef RESTRICTED_NAMES
    annotation = factory->execute( ([
    "name": replace(vars["annottitel"],"/","_"),
    "mimetype":mimetype, ]));
#else
    annotation = factory->execute( ([
        "name": vars->annottitel,
        "mimetype":mimetype, ]));
#endif
    annotation->set_attribute(OBJ_DESC, vars->annottitel);
    if ( !objectp(annotation) )
    THROW("Failed to create annotation !", E_ERROR);

    annotation->set_content(vars["annotbody"]);
    annotation->sanction_object(
    obj->get_creator(),
    annotation->query_sanction(obj->get_creator()) | SANCTION_MOVE);

    obj->add_annotation(annotation->this());
    annotation->set_acquire(obj);
    if ( objectp(attached) ) {
        annotation->add_annotation(attached->this());
        attached->set_acquire(annotation);
    }
    annotation->set_attribute(DOC_ENCODING, "utf-8");
    return annotation;
}

array order_inventory_slides(object user,object context, mapping vars)
{
    array inv = context->get_inventory();
    array(int) order = allocate(sizeof(inv));

    int num, pos;
    pos = 0;
    foreach(inv, object obj) {
    if ( sscanf(obj->get_identifier(), "Folie%d.%*s", num) == 0 )
        if ( sscanf(obj->get_identifier(), "Slide%d.%*s", num) == 0 )
        num = 1000000;

    LOG("Position of " + obj->get_identifier() + " to "+ num);
    //vars["pos_"+num] = (string)obj->get_object_id();
    order[pos++] = num;
    }
    LOG("Order:"+sprintf("%O", order));
    context->order_inventory(order);
    return ({ (["german":"Ok!","english":"Ok!",]), 0 });
}

array order_inventory(object user, object context, mapping vars)
{
    int               num;
    object            obj;
    array(object)     inv;
    mapping order = ([ ]);
    mapping oorder= ([ ]);

    inv = context->get_inventory();
    num = 0;
    // current position of objects...
    foreach( inv, obj ) {
    order[obj->get_object_id()] = num;
    oorder[num++] = obj->get_object_id();
    }


    array ind = indices(vars);
    foreach ( ind, string idx ) {
    if ( sscanf(idx, "pos_%d", num) == 0 ) continue;
    if ( num == order[(int)vars[idx]] ) continue;
    LOG("switch index:"+vars[idx]+" to " + num + "(prev="+order[(int)vars[idx]]+")");
    context->swap_inventory(num, order[(int)vars[idx]]);
    mixed helporder = order[(int)vars[idx]];
    order[(int)vars[idx]] = num;
    order[oorder[num]] = helporder;
    mixed helpoorder = oorder[num];
    oorder[num] = oorder[helporder];
    oorder[helporder] = helpoorder;
    }
    return ({ "Ok!", 0 });
}

void add_grp_member(object grp, object user)
{
    grp->add_member(user);
    _Server->get_module("tasks")->add_task(user, this(), "create_group_exit",
                       ({ grp, user }),
                       ([ "english": "Create exit to group " +
                          grp->get_identifier(),
                          "german": "Ausgang zur Gruppe " +
                          grp->get_identifier() + " erzeugen", ]));
}

void delete_group_exit(object grp, object user)
{
    object dest = grp->query_attribute(GROUP_WORKROOM);
    object wr = user->query_attribute(USER_WORKROOM);
    array exits = wr->get_inventory_by_class(CLASS_EXIT);

    // search exit to groups workarea in users workarea
    foreach ( exits, object ex )
    if ( ex->get_exit() == dest ) {
      ex->delete();
      return;
    }
}

void create_group_exit(object grp, object user)
{
    object dest = grp->query_attribute(GROUP_WORKROOM);
    object wr = user->query_attribute(USER_WORKROOM);
    array exits = wr->get_inventory_by_class(CLASS_EXIT);

    // check if exit already exists in workarea
    foreach ( exits, object ex )
    if ( ex->get_exit() == dest )
        return;


    object factory = _Server->get_factory(CLASS_EXIT);
    object exit = factory->execute(
        ([ "name": grp->parent_and_group_name() + " workarea", "exit_to": dest, ]) );
    exit->sanction_object(this(), SANCTION_ALL);
    exit->move(wr);
}

void join_invited_group(object grp, object user)
{
    grp->add_member(user);
    create_group_exit(grp, user);
}

void remove_group_exit(object grp, object user)
{
    object wr = user->query_attribute(USER_WORKROOM);
    if ( objectp(wr) ) {
    foreach(wr->get_inventory_by_class(CLASS_EXIT), object exit)
        if ( exit->get_exit() == grp->query_attribute(GROUP_WORKROOM) )
        {
        exit->delete();
        return;
        }
    }
}

static void download_cont(object cont, function ul, string prefix, object tar)
{
    prefix = prefix + cont->get_identifier() + "/";
    ul(tar->tar_content(prefix));
    foreach( cont->get_inventory(), object file ) {
      if ( file->get_object_class() & CLASS_DOCUMENT )
        ul(tar->tar_content(prefix + file->get_identifier(),
			    file->get_content()));
      else if ( file->get_object_class() & CLASS_CONTAINER )
        download_cont(file, ul, prefix, tar);
    }
}

static void downloader(array(object) files, object obj, object user)
{
    function ul;

    object tar = get_module("tar");
    ul = obj->receive_content(0);

    foreach(files, object file) {
    if ( file->get_object_class() & CLASS_DOCUMENT )
        ul(tar->tar_content(file->get_identifier(), file->get_content()));
    else if ( file->get_object_class() & CLASS_CONTAINER )
        download_cont(file, ul, "", tar);
    }
    ul(0);
    obj->move(user);

    downloads[obj->get_object_id()] = obj;
}

void set_read(object obj, object read)
{
    read->download_document(0, obj, this_user());
    foreach(obj->get_annotations(), object ann)
      set_read(ann, read);
}

array handle_set_read(object obj)
{
    object read = get_module("table:read-documents");
    set_read(obj, read);
    return ({ RES("Documents marked as read.",
          "Dokumente wurden als gelesen markiert."), ""});
}

void download_finished(int event, object obj, object caller)
{
    obj->delete();
}

string result_simplepage(string phtml, string title, string l) {
  string html = "";
  html += "<html><head><title>"+title+"</title>";
  html += "<SCRIPT language=\"javascript\" type=\"text/javascript\">function reload_parent() { opener.location.reload(); }</SCRIPT>";
  html += "</head><body bgcolor=\"#D7D7D7\">";
  html += "<SCRIPT language=\"javascript\" type=\"text/javascript\">window.resizeTo(480, 125); window.moveTo(screen.width/2 - document.body.clientWidth/2, screen.height /2 - document.body.clientHeight/2);</SCRIPT>";
  html += phtml;
  html += "<BR/><DIV ALIGN=\"CENTER\"><INPUT TYPE=\"BUTTON\" VALUE=\""+RES_MSG("Close this window","Fenster schliessen", l)+"\" onClick=\"JavaScript:reload_parent();JavaScript:window.close()\"/></DIV>";
  html += "</body></html>";
  return html;
}

// performs a quick lookup
string perform_quicksearch(mapping vars, int usersearch, int groupsearch) {
  object g,u;

  if (usersearch)  u = USER(vars["keywords"]);
  if (groupsearch) g = GROUP(vars["keywords"]);

  array sresult = ({ });

  if (objectp(u)) sresult += ({u});
  if (objectp(g)) sresult += ({g});
  
  vars->language = this_user()->query_attribute(USER_LANGUAGE);
  
  return html_from_list(get_selection(this_user(), 0), sresult, vars,
		   _FILEPATH->path_to_object("/stylesheets/selection.xsl"));
}

string get_visit_link_group(object group, string|void params) {
  string ret = "<a href=\""+get_visit_link_href_group(group, params)+"\">"+get_visit_link_href_group(group, params)+"</a>";
  return ret;
}

string get_visit_link_href_group(object group, string|void params) {
  string ret =
                "https://"+_Server->query_config(CFG_WEBSERVER)+
               (_Server->query_config(CFG_WEBPORT_HTTP)!=443?":"+_Server->query_config(CFG_WEBPORT_HTTP):"")+
               "/groups/"+replace_uml(group->get_identifier())+
               (stringp(params)?params:"");
 return ret;
}

string get_visit_link(int id) {
  string ret = "<a href=\""+get_visit_link_href(id)+"\">"+get_visit_link_href(id)+"</a>";
  return ret;
}

string get_visit_link_href(int id) {
  string ret =
               "https://"+
               _Server->query_config(CFG_WEBSERVER)+
               (_Server->query_config(CFG_WEBPORT_HTTP)!= 443?":"+_Server->query_config(CFG_WEBPORT_HTTP):"")+
               "/scripts/navigate.pike?"+id;
  return ret;
}

int isEmptyString(string s) {
	for(int i = 0; i < sizeof(s); i++)
		if(s[i] != ' ') return 0;
	return 1;
}

int mailAdressValid(string adress){
	string localPart;
	string domainPart;
	
	if(sscanf(adress, "%s@%s", localPart, domainPart) != 2){
//		werror("Struktur local@domain scheint nicht korrekt.\n");
		return 0;
	}else if(sscanf(localPart, "%*[A-Za-z0-9.!#$%&'*+-/=?^_`{|}~]") != 1 || sizeof(localPart) == 0 || sizeof(localPart) > 64){
//		werror("localpart scheint nicht korrekt.\n");
		return 0;
	}else if(sscanf(domainPart, "%*[A-Za-z0-9.-]") != 1 || sizeof(domainPart) == 0 || sizeof(domainPart) > 255){
//		werror("domainpart scheint nicht korrekt: "+localPart+", "+domainPart+"\n");
		return 0;
	}else 
		return 1;
}
