inherit "/classes/Script";

#include <macros.h>
#include <database.h>
#include <classes.h>
#include <events.h>
#include <attributes.h>
#include <access.h>

static array(string) protocols = ({ "http:", "https:", "webdav:", "ftp:", "ftps:", "irc:", "nntp:" });

#define RES(e,g) ([ "german":g, "english":e, "chinese":e, ])
#define RES_MSG(e,g,l) RES(e,g)[l]



string
execute(mapping vars)
{
    object created;
    object factory;
    int oid = (int)vars["environment"];
    object env = find_object(oid);

    try_event(EVENT_EXECUTE, CALLER, this_object());
    object browser =_FILEPATH->path_to_object("/scripts/browser.pike");

    if ( vars["objkind"] == "single-exit" || vars["objkind"] == "exit" ) {
	    vars["keywords"] = vars["destination"];
      if ( vars["objkind"] == "exit" )
          vars["_perform"] = "link_exit_bi";
      else
          vars["_perform"] = "link_exit";
      vars["_performdesc"] = "Set destination for exit";
      vars["_action"] = "search";
      vars["_context"] = (string)env->get_object_id();
      vars["filter_class"] = "Room";
      return browser->execute(vars);
    }

    LOG("Execute="+sprintf("%O\n", mkmapping(indices(vars), values(vars))));
    LOG("Selected class == " +vars["objkind"]);
    if ( !stringp(vars["objkind"]) )
	return result_page("No object type submitted by browser!",
			   "JavaScript:history.back();");

    if ( vars["objkind"] == "room" || vars["objkind"] == "gallery" )
      factory = _Server->get_factory(CLASS_ROOM);
    else if ( vars["objkind"] == "container" || vars["objkind"] == "mail_container" )
      factory = _Server->get_factory(CLASS_CONTAINER);
    else if ( vars["objkind"] == "exit" || vars["objkind"] == "single-exit" ) 
      factory = _Server->get_factory(CLASS_EXIT);
    else if ( vars["objkind"] == "messageboard" ) 
      factory = _Server->get_factory(CLASS_MESSAGEBOARD);
    else if ( vars["objkind"] == "link" ) {
      factory = _Server->get_factory(CLASS_DOCEXTERN);
      vars["url"] = vars["adress"];
      string proto;
      if ( sscanf(vars["url"], "%s//%*s", proto) > 0 ) {
        if ( search ( protocols, proto ) == -1 ) {
          return error_page("Unknown Protocol: "+ proto, "JavaScript:history.back();");
	      }
	    }
	    else
	      vars["url"] = "http://"+vars["url"];
    }
    else if ( vars["objkind"] == "monitor" ) {
	    factory = _FILEPATH->path_to_object("/sources/monitor.pike");
    }

    // prevent factory from creating a bidirectional exit
    if ( vars["objkind"] == "single-exit" )
      vars["exit_to"] = 0; 

    LOG("Factory is =" + factory->get_identifier());
    
    created = factory->execute(vars);

	
    if ( stringp(vars["objdesc"]) ) {
      string desc = vars->objdesc;
      sscanf(desc, "%s\0%*s", desc);
      created->set_attribute(OBJ_DESC, desc);
    }			       

    if ( stringp(vars["gallery"]) ) {
      created->set_acquire_attribute("xsl:content", 0);
      created->set_attribute("xsl:content",
          ([ _STEAMUSER:
           _FILEPATH->path_to_object("/stylesheets/gallery.xsl"), ]) );
    }

    if(vars["objkind"] == "mail_container") {
      created->sanction_object( this(), SANCTION_ALL );
      created->set_attribute(OBJ_TYPE, "container_mailbox");
      env->add_annotation( created );
    } else {
      array selection;
      if ( objectp(env) ) {
        if ( vars["objkind"] == "monitor" ) {
            env->set_attribute("cont:monitor", created);
            created->set_monitoring(env);
        }
        else
          created->move(env);
      }
      else {
      selection = this_user()->query_attribute("USER_SELECTION");
        if ( arrayp(selection) ) 
            selection += ({ created->this() });
        else 
            selection = ({ created->this() });
        this_user()->set_attribute("USER_SELECTION", selection);
        created->move(this_user());
      }
    }
    
    run_event(EVENT_EXECUTE, CALLER, this_object());
    // load javascript in create.js doesnt work on safari !
    //return "<html><script language=\"javascript\" src=\"/gui_js/create.js\"><body onLoad=\"doClose()\"></body></html>";
    return "<html><script language=\"javascript\">function doReload() {window.opener.location.reload();window.close()} function my_string_replace(source, replace, search) {var searchlength = search.length;var replacelength = replace.length;var pos = source.indexOf(search, 0);while (pos >= 0){source = source.substring(0, pos) + replace + source.substring(pos + searchlength);pos = source.indexOf(search, pos + replacelength);}return source;} function doClose() {try {var location = window.opener.document.URL;var ito = location.indexOf('to_obj', 0);var ifrom = location.indexOf('from_obj', 0);if (ito > 0 && ifrom > 0) {var to = location.split('to_obj=');to = to[1];to = to.split('&');to = to[0];var from = location.split('from_obj=');from = from[1];from = from.split('&');from = from[0];if (to-from < 19) {var newto = to;newto++;location = my_string_replace(location, \"to_obj=\"+newto, 'to_obj='+to);window.opener.location = location;window.close();}else doReload();}else doReload();} catch(e) {doReload();} }</script><body onLoad=\"doClose()\"></body></html>";
}

/**
 *
 *  
 * @param 
 * @return 
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>) 
 * @see 
 */
void event_exit_moved(int event,object exit,object caller,object env,object to)
{
    LOG("Event exit moved called for " + exit->get_object_id()+"!!!");
    object other_exit = exit->get_link_object();
    LOG("Setting name of " + other_exit->get_identifier()+"/"+other_exit->get_object_id());
    other_exit->set_attribute(OBJ_NAME, to->get_identifier());
    LOG("Setting name of " + exit->get_identifier()+"/"+exit->get_object_id());
    exit->set_attribute(OBJ_NAME, other_exit->get_environment()->get_identifier());
    remove_event(exit, EVENT_MOVE, event_exit_moved);
}

string redirection_url(string resMessage, string errMessage, mapping vars, object ctx) {
    string script = "navigate.pike";
    if ( stringp(vars["_script"]) )
        script = vars["_script"];
    else {
      if (objectp(ctx)) {
        string buf_script = _FILEPATH->object_to_filename(ctx);
        // check if path is valid...
        if (stringp(buf_script) && (search(buf_script, "/void") != 0))
          script = httplib.replace_uml(buf_script);
      }
    }
    LOG("Redirecting to " + script);
    if ( stringp(errMessage) )
        errMessage = replace(errMessage, " ", "%20");
    if ( !stringp(resMessage) )
      resMessage = "Error";
    resMessage = replace(resMessage, " ", "%20");
    string url;
    if ( stringp(vars->id) && strlen(vars->id) > 0 )
        url = script + "?object="+vars["id"]+"&";
    else
        url = script + "?";
    url += (stringp(vars["type"]) ? "type="+vars["type"]+"&" : "")+
        (stringp(vars["active"]) ? "active="+vars["active"]+"&" : "")+
        (stringp(vars["mode"]) ? "mode="+vars["mode"]+"&" : "")+
        (stringp(vars["room"]) && vars->room != "0" && strlen(vars->room) > 0 ?
         "room="+vars["room"]+"&" : "")+
        (stringp(vars["jump"]) && vars["jump"] != "" ? "jump="+vars["jump"]+"&" : "")+
        "message="+resMessage+(stringp(errMessage) ? "&error="+errMessage:"")+
        (stringp(vars["jump"]) && vars["jump"] != "" ? "#"+vars["jump"] : "");
    return url;
}

