inherit "/classes/Script";

#include <macros.h>
#include <database.h>
#include <classes.h>
#include <attributes.h>
#include <config.h>
#include <roles.h>
#include <access.h>

#define EMAIL_RESPONSE(name) \
"Welcome to sTeam !\r\n"+\
"Your login name is '"+name+"' and you can login into the sTeam Server\r\n"+\
"with the password you selected."
#define EMAIL_RESPONSE_ACTLINK(name) \
"\r\nTo do so you have to click on the link given below in order to activate your user.\r\n"

static mapping invites = ([ ]);

void init_script() 
{
  add_data_storage("store_invites",retrieve_invite, restore_invite, 1);
}

mapping retrieve_invite()
{
  return ([ "invites": copy_value(invites), ]);
}

void restore_invite(mapping restore)
{
  if ( CALLER != _Database )
    steam_error("Unauthorized call to restore-invites !");
  invites = restore->invites;
  if ( !mappingp(invites) )
    invites = ([ ]);
}

//! invite a user by e-mail
void invite_user(string email, object grp)
{
  if (!_SECURITY->check_access(grp,CALLER,SANCTION_INSERT,ROLE_INSERT_ALL,false))
    steam_error("Cannot invite user to grp - no permissions !");
  if ( arrayp(invites[email]) )
    invites[email] += ({ grp });
  else
    invites[email] = ({ grp });
  require_save("store_invitations");
}

array get_invites(string email)
{
  if ( !arrayp(invites[email]) )
    return 0;
  return copy_value(invites[email]);
}

string generate_password()
{
  string pw_chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!$%&=?+-";
  int pw_length = 8;
  string pw = "";
  for ( int i=0; i<pw_length; i++ ) {
    int rnd = random((int)sizeof(pw_chars));
    pw += pw_chars[rnd..rnd];
  }
  return pw;
}

string
execute(mapping vars)
{
    object      factory, created;
    string name, password, email;
    mixed                    err;
    object                   msg;
    object                  user;
    object                   cfg;
    object              workroom;

//    LOG("Creating User:\n"+sprintf("%O", vars));

    name     = vars["nickname"];
    email    = vars["email"];
    user = MODULE_USERS->lookup(name);
    cfg  = _FILEPATH->path_to_object("/register/register.html");

    if ( stringp(vars["activate"] ) ) {
	if ( !objectp(user) )
	    return error_page("Failed to activate user - not found !","");
	if ( !user->get_activation() )
	    return result_page("Your login was already activated. Just "+
                         "<a href=\"https://"+
                         _Server->query_config(CFG_WEBSERVER)+
                         (_Server->query_config(CFG_WEBPORT_HTTP)!=443?":"+_Server->query_config(CFG_WEBPORT_HTTP):"") +
                         "/scripts/navigate.pike\">login</a> with "+
			       "your username and password.\n", "");

	if ( user->activate_user((int)vars["activate"]) )
	    return result_page("Your login was activated and you can now "+
                         "<a href=\"https://"+
			       _Server->get_server_name()+
			       (_Server->query_config(CFG_WEBPORT_HTTP)!=443?":"+_Server->query_config(CFG_WEBPORT_HTTP):"") +
			       "/\">login</a> with your password.","");

	return error_page("The activation code you submitted is invalid!<BR/>"+
			  "If you clicked on the link in the systems "+
			  "response contact e-mail <a href=\"mailto:"+
			  _Server->get_config(CFG_EMAIL) +
			  "\">steam support</a>.", "");
    }
    else if ( objectp(user) ) {
	return error_page("The login '"+name+"' does already exist !",
			  "JavaScript:history.back();");
    }
    if ( !stringp(email) || sscanf(email, "%*s@%*s.%*s") != 3 ) {
	return error_page("The E-Mail does not look correctly. user@host",
			  "JavaScript:history.back();");
    }

    if ( objectp(cfg) && cfg->query_attribute("denied") ) {
	err = catch {
	    _SECURITY->check_access(this(), this_user(),
				    SANCTION_READ,ROLE_READ_ALL, false);
	};
        if ( err ) {
	    return error_page("Even though usually sTeam allows everyone "+
			      "to freely create logins,<BR/>this server "+
			      "is restricted and you need to contact the "+
			      "admins for a login.",
			      "JavaScript:history.back()");
	}
    }

    bool act = false;

    if ( stringp(vars->pw) ) {
	if ( sscanf(vars["pw"], "%s\0", password) != 1 )
	    password = vars["pw"];
	
	if ( password != vars["pw_repeat"] )
	    return result_page(
		"Password doesnt match. Make sure the password and repeated "+
		"password is the same.",
		"JavaScript:history.back();");
    }
    else {
	vars->pw = generate_password();
	vars->password = vars->pw;
	act = true;
    }
    seteuid(USER("root"));
    err = catch {
	factory  = _Server->get_factory(CLASS_USER);
  
	created = factory->execute(vars);
	if ( act || arrayp(get_invites(email)) || 
	     (objectp(cfg) && cfg->query_attribute("activate")) ) 
	{
	    created->activate_user(factory->get_activation());
	    act = true;
	}
    };
    if ( err ) {
      return error_page("<h2>Error creating user !</h2><BR/>"+
			sprintf("%O\n", err[1]),
			"JavaScript:history.back();");
    }
    if ( arrayp(get_invites(email)) ) {
      foreach(get_invites(email), object grp) {
	grp->add_member(created);
      }
    }

    object group;
    if ( vars->group ) {
	group = get_module("groups")->lookup(vars->group);
	if ( objectp(group) ) {
	    err = catch(group->add_member(created));
            if ( err ) { 
		FATAL("Failed to add new user to group: %s", vars->group);
	    }
	    vars["groupname"] = group->get_identifier();
	}
    }

    if ( vars->emailobj ) {
	object eobj = find_object(vars->emailobj);
	vars->emailbody = eobj->get_content();
    }
    int activation = factory->get_activation();
    vars->activation = (string)activation;

    mapping templ_vars = ([ ]);
    vars["host"] = _Server->query_config(CFG_WEBSERVER);
    vars["port"] = (string)_Server->query_config("https_port");
    vars["username"] = (string)(this_user()->get_identifier());
    foreach(indices(vars), string key) {
      mixed val = vars[key];
      if ( stringp(val) )
	templ_vars["{"+key+"}"] = val;
    }

    string adminreply = "admin@" + _Server->query_config("machine") + (stringp(_Server->query_config("domain"))?"." +_Server->query_config("domain"):"");
    
    if ( vars->emailbody ) {
	if ( strlen(vars->emailbody) > 0 && vars->emailbody != "0" ) {
	    string emailtext = replace(vars->emailbody, 
				       indices(templ_vars),
				       values(templ_vars));
	    if ( !vars->emailsubject )
		vars->emailsubject = "sTeam Registration";
	    
	    MODULE_SMTP->send_mail(email, 
				   vars->emailsubject,
				   emailtext,
				   adminreply,
				   adminreply,
				   vars->emailtype);
	}
    }
    else if ( (!objectp(cfg) || !cfg->query_attribute("no_email")) )
    {
	MODULE_SMTP->send_mail(email, "sTeam Registration!",
			       EMAIL_RESPONSE(name)+
			       (activation > 0  ?
        EMAIL_RESPONSE_ACTLINK(name)+
				"Your activation link is\r\n  "+
				"http://" +_Server->get_server_name()+
                                (_Server->query_config(CFG_WEBPORT_PRESENTATION)!=80?":"+_Server->query_config(CFG_WEBPORT_PRESENTATION):"") +
				_Server->query_config(CFG_WEBMOUNT)+
			       "scripts/create_user.pike?activate="+
				activation + 
				"&nickname="+httplib.replace_uml(name):""), adminreply, adminreply);
    }
    if ( vars->redirect ) {
      return redirect(vars->redirect);
    }
    else if ( vars->responseobj ) {
      object responseObj = find_object(vars->responseobj);
      return result_page(responseObj->get_content(), "/");
    }
    return result_page("Your login has been successfully created.<BR/>"+
		       (act?"":"Now just wait for the activation e-mail !"),
		       "http://"+_Server->query_config(CFG_WEBSERVER)+
                         (_Server->query_config(CFG_WEBPORT_PRESENTATION)!=80?":"+_Server->query_config(CFG_WEBPORT_PRESENTATION):"") +
		       _Server->query_config(CFG_WEBMOUNT));
}



