inherit "/classes/Object";

#include <macros.h>
#include <database.h>
#include <classes.h>
#include <events.h>
#include <attributes.h>
#include <types.h>

string 
execute(mapping vars)
{
    int   oid = (int)vars["object"];
    object                      obj;
    string                     html;
    mapping              attributes;

    if ( (int)vars["acquiring"] == 1 )
	set_acquiring();
    else if ( (int)vars["events"] == 1 )
	check_events();
    else if ( (int)vars["bench"] == 1 )
	return run_bench();
    
    if ( oid == 0 ) {
	obj = _ROOTROOM;
    }
    else {
	obj = find_object(oid);
    }
    html = "<html><body>\n";
    html += "<h2>Object: <b>"+obj->get_identifier()+"</b></h2>";
    if ( obj->get_object_class() & CLASS_USER ) {
	array(object) sockets = obj->get_sockets();
	html += "<table><tr><td>Class</td><td>Connection?</td></tr>\n";
	foreach(sockets, object socket) {
	    if ( !objectp(socket) )
		html += "<tr><td>---</td><td>NULL</td></tr>\n";
	    else 
		html += "<tr><td>"+socket->get_client_class()+ "</td><td>"+
		    (objectp(socket->get_user_object()) ? "yes" : "no")+
		    "</td></tr>\n";
	}
	html += "</table>\n";
    }
    html += "<table><tr><td>Attribute</td><td>Value</td></tr>\n";
    attributes = obj->get_attributes();
    foreach(indices(attributes), mixed attr) {
	mixed val = obj->query_attribute(attr);
	html += "<tr><td><b>"+attr+"</b></td><td>";
	if ( sizeof(attributes[attr]) == 5 && 
	     (objectp(attributes[attr][4])|| functionp(attributes[attr][4])) )
	    html += "Acquired!";
        else if ( objectp(val) )
	    html += "(object)";
        else if ( arrayp(val) )
	    html += "(array)";
	else if ( mappingp(val) )
	    html += sprintf("%O", mkmapping(indices(val), values(val)));
        else 
	    html += val;
        html += "</td></tr>";
    }
    html += "</table>\n";
    html += "<br/><h3><i>Events</i></h3><br/>";
    html += "<table><tr><td>Event</td><td>Objects....</td></tr>\n";
    mapping events = obj->get_my_events();
    int event;
    object  o;

    foreach(indices(events), event) {
	mapping event_objs = events[event];
	html += "<tr><td>"+event+"</td><td>";
	foreach(indices(event_objs), o) {
	    if ( objectp(o) )
		html += o->get_identifier() + "("+sizeof(event_objs[o])+"),";
	}
	html += "</td></tr>\n";
   }
   html +="</table>\n";
   html += "<br/><h3>Other Objects events...</h3><br/>";
   events = obj->get_events();
   foreach(indices(events), event) {
       array(mixed) listeners;
       listeners = events[event];
       html += "<tr><td>"+event+"</td><td>";
       for ( int i = 0; i < sizeof(listeners); i++ ) {
	   array(mixed) listens;
	   listens = listeners[i];
	   if ( sizeof(listens) == 0 || !functionp(listens[0]) )
	   { 
	       html += "invalid!,"; 
	       continue; 
	   }
           o = function_object(listens[0]);
	   html += o->get_identifier() + ",";
       }
       html += "</td></tr>";
   }
   html += "</table>";
   
   html += "</body></html>\n";
   return html;
}

void
set_acquiring()
{
    array(object) objs;
    object obj;
    
    object content, attributes, access;

    content = _FILEPATH->path_to_object("/design/content.xsl");
    attributes =_FILEPATH->path_to_object("/design/attributes.xsl");
    access =_FILEPATH->path_to_object("/design/access.xsl");
    
    _ROOT->set_attribute("xsl:content", content);
    _WORLDUSER->set_attribute("xsl:content", content);
    _ROOTROOM->set_attribute("xsl:content", content);
    
    _ROOT->set_attribute("xsl:attributes", attributes);
    _WORLDUSER->set_attribute("xsl:attributes", attributes);
    _ROOTROOM->set_attribute("xsl:attributes", attributes);
    
    _ROOT->set_attribute("xsl:access", access);
    _WORLDUSER->set_attribute("xsl:access", access);
    _ROOTROOM->set_attribute("xsl:access", access);
	
    objs = _ROOTROOM->get_inventory();
    foreach ( objs, obj ) {
	if ( obj->get_object_class() & CLASS_CONTAINER )
	    objs += obj->get_inventory();
    }
}

void
check_events()
{
    array(object) objs;    
    object      obj, o;
    objs = _ROOTROOM->get_inventory();
    foreach ( objs, obj ) {
	if ( obj->get_object_class() & CLASS_CONTAINER )
	    objs += obj->get_inventory();
    }
    foreach(objs, obj) {
	// check monitoring events !
	mapping events = obj->get_events();
	array(mixed) listeners = events[EVENT_ATTRIBUTES_CHANGE];
	if ( arrayp(listeners) ) {
	    LOG("("+obj->get_object_id()+") has " + sizeof(listeners) + 
		" events");
#if 0
	    for ( int i = 0; i < sizeof(listeners); i++ ) {
		o = function_object(listeners[i][0]);
		mapping m = o->get_my_events();
		if ( !mappingp(m[EVENT_ATTRIBUTES_CHANGE][obj]) ) {
		    obj->ignore_event(EVENT_ATTRIBUTES_CHANGE, listeners[i][2]);
		    LOG("Removing an event !");
		}
	    }
#endif
	}
	else {
	    LOG("("+obj->get_object_id()+") defines no events yet.");
	}
	
	//obj->_rm_all();
	object env = obj->get_environment();
	if ( objectp(env) ) {
	    env->_add_event(obj, EVENT_ATTRIBUTES_CHANGE, PHASE_NOTIFY, 
			    env->monitor_attributes);
	}
    }
}

string run_bench()
{
    int i, t;

    t = time();
    return "<html><body><h2>10000 functioncalls of set_attribute took"+
	" "+(time()-t)+" seconds !</h2></body></html>\n";
}


private static void
describe_function(string func, string desc, string ... args)
{
    LOG("Describe function: " + func + "\n"+desc + "\n" + (args*"\n"));
}
