inherit "/classes/Script";

#include <macros.h>
#include <database.h>
#include <access.h>
#include <attributes.h>
#include <types.h>
#include <classes.h>
#include <events.h>
#include <config.h>

class AsyncWrapper {
  inherit Async.Return;

  Async.Return asReturn;

  void create(Async.Return ret) {
    asReturn = ret;
    asReturn->resultFunc = asyncResult;
  }

  void asyncResult(mixed id, mixed result) {
    mixed res;
    string format = asReturn->vars["call_params"]["format"];
    if ( format == "xml" || format == "xml_noheader" ) {
      mimetype = "text/xml";
      res = result;
    }
    else {
      result = replace(result, "\n", "<br/>");
      mimetype = "text/html";
      res = result_page( result, "" );
    }
    
    ::asyncResult(id, res);
  }

}

mixed get_diff ( int|string old_id, int|string new_id, string format ) {
  mixed oldid = old_id, newid = new_id;
  if ( stringp(old_id) ) {
    if ( sscanf( old_id, "%d", oldid ) < 1 ) oldid = old_id;
  }
  if ( stringp(new_id) ) {
    if ( sscanf( new_id, "%d", newid ) < 1 ) newid = new_id;
  }
  object oldObject = find_object( oldid );
  object newObject = find_object( newid );
  
  if ( !objectp( oldObject ) )
    return "Fehler: Object "+old_id+" nicht gefunden.";
  if ( !objectp( newObject ) )
    return "Fehler: Object "+new_id+" nicht gefunden.";

  mixed result = get_module("ServiceManager")->call_service( "jdiff",
    ([ "old":oldObject->get_content(),
       "oldid":sprintf("%d",oldObject->get_object_id()),
       "new":newObject->get_content(),
       "newid":sprintf("%d",newObject->get_object_id()),
       "format":format ]) );
  return result;
}

mixed execute ( mapping vars ) {
  if ( !stringp( vars["oldversion"] ) || !stringp( vars["newversion"] ) )
    return result_page( "Fehler: Variable <tt>oldversion</tt> oder <tt>newversion</tt> ist nicht gesetzt.", "" );
  string output_format = vars["outputformat"];
  if ( !stringp(output_format) ) output_format = "normal";

  int oldId = (int)vars["oldversion"];
  int newId = (int)vars["newversion"];
  object oldObject = find_object( oldId );
  object newObject = find_object( newId );
  if ( !objectp( oldObject ) )
    return result_page( "Fehler: Object "+oldId+" nicht gefunden.", "" );
  if ( !objectp( newObject ) )
    return result_page( "Fehler: Object "+newId+" nicht gefunden.", "" );

  object result = get_module("ServiceManager")->call_service_async( "jdiff",
    ([ "old":oldObject->get_content(),
       "oldid":sprintf("%d",oldId),
       "new":newObject->get_content(),
       "newid":sprintf("%d",newId),
       "format":output_format ]) );
  if ( objectp(result) && mappingp(result->vars) )
    result->vars += ([ "call_params" : vars ]);

  return AsyncWrapper( result );
}
