inherit "/classes/Script";

#include <macros.h>
#include <database.h>
#include <attributes.h>
#include <classes.h>
#include <types.h>
#include <events.h>

mapping mExports = ([ ]);

static string 
export_object(object obj, mapping vars, object user,
	      function ul,string path, int|void depth)
{
    object xmlconv = _Server->get_module("Converter:XML");
    object xsl;
    object xslt = _Server->get_module("libxslt");
    object tar = _Server->get_module("tar");

    string         xml;
    string result, tmp;
    object o;  

    if ( obj->get_object_class() & CLASS_USER )
	return "";

    xsl = _FILEPATH->path_to_object("/stylesheets/export_public.xsl");
    xml = xmlconv->get_xml(obj, xsl);
    if ( depth > 0 )
	vars["rootdir"] = ("../"*depth);

    tmp = xslt->run(xml, xsl, copy_value(vars));
    tmp += "<!---\n"+xml+"\n--->\n";

    
    ul(tar->tar_content(path + "index.html", tmp));

    array(object) inv = obj->get_inventory();
    object retr_ann = 
	_FILEPATH->path_to_object("/scripts/retrieve_annotations.pike");
    foreach(inv, o) {
	if ( o->get_object_class() & CLASS_USER )
	    continue;

	if ( (o->get_object_class() & CLASS_DOCUMENT) &&
	     (!arrayp(vars["export_mimetypes"]) || 
	      search(vars["export_mimetypes"], 
		     o->query_attribute(DOC_MIME_TYPE)) >= 0 ) ) 
	{
	    ul(tar->tar_content(path + o->get_identifier(), o->get_content()));
	}
	else if ( (o->get_object_class() & CLASS_CONTAINER) && 
		  !(o->get_object_class() & CLASS_ROOM) && depth == 0 &&
		  stringp(vars["export_container"]) ) 
	{
	    export_object(o, vars, user, ul, 
			  path + o->get_identifier() + "/",
			  depth+1);
	}
	
	ul(tar->tar_content(path+"annotations/annotations_"+o->get_object_id()+".html",
			    retr_ann->html_show_annotations(
				o->get_annotations())));
    }
  
    object leiste = _FILEPATH->path_to_object("/images/kopfleiste.jpg");
    ul(tar->tar_content(path+"kopfleiste.jpg", leiste->get_content()));

    return result;
}

void exporter(object obj, object doc, mapping vars, string id, object user)
{
    LOG("Export Thread created...\n");
    
    mixed err = catch {
	object tar = _Server->get_module("tar");
	function ul = doc->receive_content(0);
	export_object(obj, vars, user, ul, obj->get_identifier()+"/");
	mapping icons = _Server->get_module("icons")->get_icons();
	string result = "";
	foreach(values(icons), object icon) {
	    if ( !objectp(icon) ) continue;
	    ul(tar->tar_content(obj->get_identifier()+"/icons/"+
				icon->get_object_id(), 
				icon->get_content()));
	}
	object ann_icon =_FILEPATH->path_to_object(
	    "/images/icon_annotation.png");
	ul(tar->tar_content(obj->get_identifier()+"/icons/icon_annotation.png",
			    ann_icon->get_content()));
	object gui = _FILEPATH->path_to_object("/gui_css");
	array inv = gui->get_inventory();
	foreach(inv, object o) {
	    ul(tar->tar_content(obj->get_identifier()+"/gui_css/"+
				o->get_identifier(), o->get_content()));
	}
	ul(0); // finished
    };
    if ( err != 0 )
	FATAL("ERROR in exporter thread:\n"+sprintf("%O", err));
    mExports[vars["exportID"]] = doc;
}

string generate_id()
{
    return this_user()->get_identifier()+"_" + time();
}

mixed execute(mapping vars)
{
    int    oid = (int)vars["object"];
    object obj = find_object(oid);

    string         xml;
    string result, tmp;
    object o;
    
    try_event(EVENT_EXECUTE, CALLER, this_object());
    OBJ("/stylesheets/export_public.xsl")->get_stylesheet("german");
    OBJ("/stylesheets/export_public.xsl")->get_stylesheet("english");
    OBJ("/stylesheets/export_public.xsl")->get_stylesheet("chinese");

    if ( !objectp(obj) && !stringp(vars["exportID"]) )
      return result_page("No object found to export (variable 'object') !",
			 "JavaScript:history.back();");


#if 1
    int i;
    if ( (i=search(vars["__internal"]["client"], "MSIE")) >= 0 ) {
	vars["client"] = "MSIE";
	vars["client-version"] = vars["__internal"]["client"][i+1];
    }
    else {
	string client, version;
	sscanf((string)vars["__internal"]["client"][0],"%s/%s",client,version);
	vars["client"] = client;
	vars["client-version"] = version;
    }
#endif
    if ( !stringp(vars["exportID"]) ) {
	vars["exportID"] = generate_id();
	if ( stringp(vars["export_mimetypes"]) )
	  vars["export_mimetypes"] = ({ vars["export_mimetypes"] });
	
	object factory = _Server->get_factory(CLASS_DOCUMENT);
	object o = factory->execute( ([ "name": vars["exportID"]+".tar", ]) );
	o->move(this_user());
	start_thread(exporter, obj, o, vars, vars["exportID"], this_user());
    }
    if ( objectp(mExports[vars["exportID"]]) ) {
	//mixed res = ({ copy_value(mExports[vars["exportID"]]), 
	//	   "application/x-gtar" });
	get_module("temp_objects")->add_temp_object(mExports[vars->exportID], 
						    time() + 60*60*5);
	mixed res = result_page("Export finished, now downloading...",
				"", "Downloading export",
				"<meta http-equiv=\"refresh\" content=\"2;"+
				"URL="+_FILEPATH->object_to_filename(
				    mExports[vars["exportID"]])+"\">");
	m_delete(mExports, vars["exportID"]);
	return res;
    }
    else {
	return 
	  result_page("Export in progress, please wait ...<BR/>",
		      "", "Export",
		      "<meta http-equiv=\"refresh\" content=\"10;URL="+
		      get_identifier()+"?exportID="+vars["exportID"]+"\">");
    }
}
