inherit "/classes/Script";
inherit "/base/serialize";

#include <macros.h>
#include <database.h>
#include <classes.h>
#include <attributes.h>
#include <events.h>
#include <config.h>
#include <web.h>
#include <exception.h>
#include <types.h>


string make_error(string err, mapping vars)
{
    object                  libxslt;
    object                   errors;
    
    libxslt = _Server->get_module("libxslt");
    errors  = _FILEPATH->path_to_object("/stylesheets/errors.xsl");
    return libxslt->run("<error><message>"+err+"</message></error>\n",
			errors, vars);
}

/**
 * execute - execute the script
 *  
 * @param vars - variables
 * @return the html representation for an object
 * @author Thomas Bopp (astra@upb.de) 
 */
string|int
execute(mapping vars)
{
    int   oid = (int)vars["object"];
    object                  factory;   
    

    array(string) types=({"unknown", "int", "float", "string", "object",
			   "array", "mapping", "map-entry", "program",
			   "time", "function" });

    factory = _Database->find_object(oid);
    if ( !objectp(factory) || !(factory->get_object_class()& CLASS_FACTORY) ) 
	return make_error("Unable to find factory object !", vars);

    if ( !stringp(vars["key"]) || strlen(vars["key"]) == 0 )
	return make_error("No value for key given !", vars);
	
    int type = search(types, vars["type"]);
    MESSAGE("TYPE: "+vars["type"] + ", found="+type);
    if ( type == -1 ) 
	return make_error("Invalid Type!", vars);
    mixed val = vars["default"];
    switch ( type ) {
    case CMD_TYPE_INT:
	    val = (int)val;
	    break;
	case CMD_TYPE_FLOAT:
	    val = (float)val;
	    break;
	case CMD_TYPE_STRING:
	    val = (string)val;
	    break;
	case CMD_TYPE_OBJECT:
	    val = find_object((int)val);
	    break;
	case CMD_TYPE_ARRAY:
        case CMD_TYPE_MAPPING:
	    val = unserialize(val);
	    break;
	    
    }

    try_event(EVENT_EXECUTE, CALLER, this_object());

    object user = this_user();
    factory->register_attribute(vars["key"], type, vars["description"],
				0, EVENT_ATTRIBUTES_CHANGE, 
				(int)vars["acquire"], CONTROL_ATTR_USER,
				val);
    run_event(EVENT_EXECUTE, CALLER, this_object());
    return redirect("navigate.pike?object="+vars["object"]+"&room="+
		    vars["room"]);
}

