inherit "/classes/Script";

private static mapping mCache = ([ ]);

import Graphic;

#include <macros.h>
#include <database.h>
#include <attributes.h>
#include <classes.h>

mixed service_manager = get_module("ServiceManager");
mixed xslt_processor = get_module("libxslt");

int get_oid(string str)
{
  int oid;
  
  if ( !stringp(str) )
    return 0;
  else if ( sscanf(str, "%d.%*s", oid) >= 1 )
    return oid;
  else
    return (int)str;
}

static void set_thumbnail_data ( object obj, string thumb_data, int x, int y ) {
    object img = get_factory( CLASS_DOCUMENT )->execute( ([
        "name":sprintf( "thumbnail_%dx%d.jpg", x, y ),
         "mimetype":"image/jpeg" ]) );
    if ( !objectp(img) )
        return;
    img->set_acquire( obj );
    mapping thumbnails = obj->query_attribute( "DOC_THUMBNAILS" );
    if ( !mappingp(thumbnails) ) thumbnails = ([ ]);
    mapping thumbnail = ([ ]);
    img->set_content( thumb_data );
    thumbnail->image = img;
    thumbnail->timestamp = time();
    thumbnail->x = x;
    thumbnail->y = y;
    thumbnails[sprintf("%dx%d", x, y)] = thumbnail;
    array depending_objects = obj->query_attribute( "OBJ_DEPENDING_OBJECTS" );
    if ( !arrayp(depending_objects) ) depending_objects = ({ });
    depending_objects += ({ img });
    array depending_on = img->query_attribute( "OBJ_DEPENDING_ON" );
    if ( !arrayp(depending_on) ) depending_on = ({ });
    depending_on += ({ obj });
    
    // set data:
    seteuid( USER("root") );
    mixed err = catch {
        obj->set_attribute( "DOC_THUMBNAILS", thumbnails );
        obj->set_attribute( "OBJ_DEPENDING_OBJECTS", depending_objects );
        img->set_attribute( "OBJ_DEPENDING_ON", depending_on );
    };
    seteuid( 0 );
}

mixed handle_thumbnail ( mixed result, mixed userData ) {
    if ( !mappingp(userData) )
        return result;

    object obj = userData->obj;
    int x = userData->x;
    int y = userData->y;
    if ( !stringp(result) || !objectp(obj) || !intp(x) || !intp(y) )
        return result;
    set_thumbnail_data( obj, result, x, y );
    return result;
}


object|array(string)|int execute(mapping vars)
{
  int oid = get_oid(vars["object"]);
  string  path = vars["objectpath"];
  string                       html;
  object                        obj;
  object                         ss;

  int                 version_major;
  int                 version_minor;
  int                 version_micro;

  LOG("Getting object="+oid);

  sscanf( _Server->get_version(), "%d.%d.%d", version_major, version_minor, version_micro );
  
  if ( stringp(path) && strlen(path) > 0 ) {
    if ( !objectp(obj = mCache[path]) ) {
      obj = _FILEPATH->path_to_object(path);
      mCache[path] = obj;
    }
  }
  else {	
    if ( oid == 0 ) 
	return ({"<html><body><h1>No object found to get!</h1></body></html>",
		   "text/html" });
    obj = find_object(oid);
  }
  if ( !objectp(obj) ) 
      THROW("Failed to find object.", E_ERROR);
  
  if ( obj->get_object_class() & CLASS_USER )
      obj = obj->query_attribute(OBJ_ICON);
  if ( !objectp(obj) )
      return -2;
     
  if ( arrayp(obj->describe_attribute("xsl:document")) &&
       objectp(ss=obj->query_attribute("xsl:document")) )
  {
    if ( vars["source"] == "true" ) {
      string xml = obj->get_content();
      
      xml = (((xml/"<")*"&lt;")/">")*"&gt;";
      return ({ "<html><body><pre>\n"+xml + "\n</pre></body></html>",
		  "text/html" });
    }
    LOG("Running XSL Processor...");
    html = xslt_processor->run(obj->get_content(), 
						 ss, vars);
    return ({ "<html>"+html+"</html>\n", "text/html" });
    
  }
      int thumb_x = (int)vars["xsize"];
      int thumb_y = (int)vars["ysize"];
      if ( thumb_x < 1 && thumb_y < 1 ) {
          return ({ obj->get_content(), obj->query_attribute(DOC_MIME_TYPE),
              obj->query_attribute(DOC_LAST_MODIFIED) });
      }
      
      int x, y;
      mixed image_obj = obj->get_image();
      if ( arrayp(image_obj) && sizeof(image_obj) > 1 ) {
          image_obj = image_obj[1];
	  if ( objectp(image_obj) ) {
              x = image_obj->xsize();
              y = image_obj->ysize();
          }
      }
      float aspect_ratio = 1.0;
      if ( y != 0 ) aspect_ratio = (float)x / (float)y;
      if ( aspect_ratio == 0.0 ) aspect_ratio = 1.0;
      
      if ( thumb_y < 1 ) thumb_y = (int)((float)thumb_x / aspect_ratio);
      
      // check whether image is already smaller or equal than the thumbnail
      if ( x > 0 && x <= thumb_x && y > 0 && y <= thumb_y ) {
          return ({ obj->get_content(), obj->query_attribute(DOC_MIME_TYPE),
              obj->query_attribute(DOC_LAST_MODIFIED) });
      }
      
      mixed thumbnails = obj->query_attribute( "DOC_THUMBNAILS" );
      mapping thumbnail;
      if ( mappingp(thumbnails) ) {
	  thumbnail = thumbnails[ sprintf("%dx%d", thumb_x, thumb_y) ];
	  //TODO: try thumbnail sizes of thumb_y +/- 1 (might differ when rounding)
	  if ( mappingp(thumbnail) && objectp(thumbnail->image) &&
	       thumbnail->x == thumb_x && thumbnail->y == thumb_y &&
	       thumbnail->timestamp >= obj->query_attribute(DOC_LAST_MODIFIED) ) {
	      return thumbnail->image->get_content();
	  }
      }
      if ( !mappingp(thumbnail) ) {
	  thumbnail = ([ ]);
	  thumbnails += ([ sprintf("%dx%d", thumb_x, thumb_y) : thumbnail ]);
      }
      
      if ( thumb_x < 1 ) {
          return ({ obj->get_image_data(),
	      obj->query_attribute(DOC_MIME_TYPE),
	      obj->query_attribute(DOC_LAST_MODIFIED),
	  });
      }
      
      // try to call a thumbnail service asynchronously:
      string service;
      mixed params;
      object serviceManager = get_module("ServiceManager");
      if ( !objectp(serviceManager) ||
           version_major < 1 || (version_major == 1 && version_minor < 7) ) {
          // do nothing (steam didn't support async services until v1.7)
      }
      /*
      else if ( serviceManager->is_service("jgraphic") ) {
          service = "jgraphic";
          params = ([ "image":obj->get_content(),
              "old-width":x, "old-height":y,
              "width":thumb_x, "height":thumb_y, "maintainAspect":1 ]);
      }
      */
      else if ( serviceManager->is_service("graphic") ) {
          service = "graphic";
          params = ({ obj, thumb_x, thumb_y, true });
      }
      
      if ( stringp(service) && !zero_type(params) ) {
          object async_res = service_manager->call_service_async( service, params );
          async_res->mimetype="image/jpeg";
          async_res->userData = ([ "obj":obj, "x":thumb_x, "y":thumb_y ]);
          async_res->processFunc = handle_thumbnail;
          return async_res;
      }

      // if no fitting services are available, generate the thumbnail directly:
      string thumb_data = obj->query_thumbnail( thumb_x, thumb_y, true );
      if ( stringp(thumb_data) )
          set_thumbnail_data( obj, thumb_data, thumb_x, thumb_y );
      return ({
          thumb_data,
          obj->query_attribute(DOC_MIME_TYPE),
          obj->query_attribute(DOC_LAST_MODIFIED),
      });
}

