inherit "/classes/Script";

#include <macros.h>
#include <events.h>
#include <exception.h>
#include <database.h>
#include <attributes.h>
#include <classes.h>
#include <access.h>
#include <roles.h>

#define ROOMSTR(o)\
"The monitored object: <a href=\"/scripts/navigate.pike?object="+o->get_object_id()+"\">"+o->get_identifier()+"</a><BR />"
#define ROOMSTRDE(o)\
"Das ueberwachte Objekt: <a href=\"/scripts/navigate.pike?object="+o->get_object_id()+"\">"+o->get_identifier()+"</a><BR />"

#define RES(e,g) ([ "german":g, "english":e, "chinese":e, ])
#define RES_MSG(e,g,l) RES(e,g)[l]

// time intervall of sender thread to send cumulated event emails
// 1800 = 30 minutes
#define SENDER_TIME 1800

static array(object) subscribers = ({ });
static mapping mDownloads = ([ ]);
static array aSaid = ({ });
static int iEventCount;
static object oMonitor;
static Thread.Queue tMessages = Thread.Queue();

class Message {
    string  msg;
    object user;
    int classification;
    object classify_object;
};

mapping notify_header(object obj, int event)
{
    mapping eventstr = ([ ]);

    object factory = _Server->get_factory(obj->get_object_class());

    switch ( event ) {
    case EVENT_ANNOTATE:
    case EVENT_ANNOTATE|EVENTS_MONITORED:
	if ( obj->is_message_board() )
	    eventstr = RES("Event: New Message on "+obj->get_identifier(), "Ereignis: neue Nachricht in "+obj->get_identifier());
	else
	    eventstr = RES("Event: New Annotation on "+obj->get_identifier(), "Ereignis: Neuer Kommentar an "+obj->get_identifier());
	break;
    case EVENT_ENTER_INVENTORY:
	eventstr = RES("Event: New Object: " + obj->get_identifier(), "Ereignis: Neues Objekt "+obj->get_identifier());
	break;
    case EVENT_LEAVE_INVENTORY:;
	eventstr = RES("Event: Object(s) leaving inventory", "Ereignis: Objekt(e) wurden entfernt");
	break;
    }
    return eventstr;
}

void create_script()
{
    subscribers = ({ });
    iEventCount = 0;
    mDownloads  = ([ ]);
}

int get_event_count()
{
    return iEventCount;
}

array(object) get_subscribers()
{
    subscribers -= ({ 0 });
    return copy_value(subscribers);
}

/**
 * Initialization of add_data_storage has to be done before loading,
 * thus call it from init()
 * @see init() in /classes/Script.pike
 */
void init_script()
{
    add_data_storage("monitor", retrieve_notifications, restore_notifications);
}

final mapping
retrieve_notifications()
{
    Thread.Queue q = Thread.Queue();
    array a = ({ });
    object msg;
    while ( tMessages->size() > 0 && (msg=tMessages->read() ) ) {
	q->write(msg);
	if ( objectp(msg) )
	    a += ({ mkmapping(indices(msg), values(msg)) });
    }
    while ( q->size() >0 && (msg=q->read()) ) tMessages->write(msg);
    subscribers -= ({ 0 });
    return ([ "subscribers": subscribers,
	      "EventCount":iEventCount,
	      "downloads": mDownloads,
	      "messages": a,
	      "said": aSaid,
	      "monitor": oMonitor, ]);
}

final void
restore_notifications(mapping data)
{
    if ( CALLER != _Database )
	THROW("invalid call to restore_notifications()", E_ACCESS);
    if ( mappingp(data) ) {
	subscribers = data["subscribers"];
	iEventCount = data["EventCount"];
	mDownloads  = data["downloads"];
	oMonitor    = data["monitor"];
	aSaid = data["said"];
	if ( arrayp(data->messages) && sizeof(data->messages) > 0 )
	{
	    foreach(data->messages, mapping msg) {
		object message = Message();
		message->msg = msg->msg;
		message->user = msg->user;
		message->classification = msg->classification;
		message->classify_object = msg->classify_object;
		tMessages->write(message); // hope this works
	    }
	}
	if ( !mappingp(mDownloads) )
	    mDownloads = ([ ]);
    }
}


object get_monitoring()
{
    return oMonitor;
}

void set_monitoring(object dest)
{
    // initially listen to events inside the room
    if ( objectp(dest) ) {
	add_event(dest, EVENT_ENTER_INVENTORY, PHASE_NOTIFY, monitor);
	add_event(dest, EVENT_LEAVE_INVENTORY, PHASE_NOTIFY, monitor);
	add_event(dest, EVENT_DELETE, PHASE_NOTIFY, monitor);
	add_event(dest, EVENT_ANNOTATE, PHASE_NOTIFY, monitor);
	add_event(dest, EVENT_SAY, PHASE_NOTIFY, monitor);
//	add_event(dest, EVENT_DOWNLOAD, PHASE_NOTIFY, monitor);
//	add_event(dest, EVENT_UPLOAD, PHASE_NOTIFY, monitor);
	add_event(dest, EVENTS_MONITORED|EVENT_ENTER_INVENTORY,
		  PHASE_NOTIFY, monitor);
	add_event(dest, EVENTS_MONITORED|EVENT_LEAVE_INVENTORY,
		  PHASE_NOTIFY, monitor);
	add_event(dest, EVENTS_MONITORED|EVENT_DELETE,
		  PHASE_NOTIFY, monitor);
	add_event(dest, EVENTS_MONITORED|EVENT_ANNOTATE,
		  PHASE_NOTIFY, monitor);
	add_event(dest, EVENTS_MONITORED|EVENT_DOWNLOAD,
		  PHASE_NOTIFY, monitor);
	add_event(dest, EVENTS_MONITORED|EVENT_UPLOAD,
		  PHASE_NOTIFY, monitor);
	oMonitor = dest;
	require_save("monitor");
    }
}

void load_object()
{
    set_attribute("xsl:content",
		  ([ _STEAMUSER:
		     _FILEPATH->path_to_object("/stylesheets/monitor.xsl"), ]));
    call(message_sender, SENDER_TIME);
}

string execute(mapping vars)
{

  try_event(EVENT_EXECUTE, CALLER, this_object());

  string action = vars["_action"];
  switch(action) {
  case "unsubscribe":
      subscribers -= ({ this_user() });
      require_save("monitor");
      return result_page("You unsubscribed this monitor.",
			 "JavaScript:history.back();");
      break;
  case "subscribe":
  default:
    //case "subscribe":
    // subscribe users to this rooms events
      if ( search(subscribers, this_user()) == -1 )
	  subscribers += ({ this_user() });
      require_save("monitor");
      return result_page("You subscribed to events.",
			 "JavaScript:history.back();");
      break;
  }
  run_event(EVENT_EXECUTE, CALLER, this_object());
}

array(object) get_downloaded()
{
    return indices(mDownloads);
}

mapping get_downloads()
{
    return mDownloads;
}

static void
send_subscribers_collect(mapping msg, int classify, object classify_object)
{
    // do not mail directly, but get some events together (classify)
    foreach(subscribers, object s) {
	if ( !objectp(s) )
	    continue;
	object m = Message();
	m->msg = msg[(s->query_attribute(USER_LANGUAGE)!=0?s->query_attribute(USER_LANGUAGE):"english")];
	m->user = s;
	m->classification = classify;
	m->classify_object = classify_object;
	tMessages->write(m);
    }
    require_save("monitor");
}

static void
send_subscribers(object obj, mapping msg, mapping header, string|void mtype)
{
    object parent=obj->get_annotating();
    array args=({
	    0,   //header set later, depends on user language
	    0,   //msg set later, depends on user language
	    this_user()->get_steam_email(),
	    obj->get_object_id()+"@"+_Server->get_server_name(),
	    mtype,
             sprintf("%s %s <%s>",
                    this_user()->query_attribute(USER_FIRSTNAME),
                    this_user()->query_attribute(USER_FULLNAME),
                    this_user()->get_steam_email()),
	    MOD("message")->get_time(obj->query_attribute(OBJ_LAST_CHANGED)),
	    sprintf("<sTeam%d-%d@%s>",
                    obj->query_attribute(DOC_LAST_MODIFIED)||
                    obj->query_attribute(OBJ_CREATION_TIME),
                    obj->get_object_id(),
                    _Server->get_server_name()),
	    sprintf("<sTeam%d-%d@%s>",
                    parent->query_attribute(DOC_LAST_MODIFIED)||
                    parent->query_attribute(OBJ_CREATION_TIME),
		    parent->get_object_id(),
		    _Server->get_server_name())
		 });
    string lang;
    foreach(subscribers, object s)
    {
	if ( !objectp(s) || s == this_user() )
	    continue;
         lang = (stringp(s->query_attribute(USER_LANGUAGE))?s->query_attribute(USER_LANGUAGE):"english");
         args[0] = header[lang];
         args[1] = msg[lang];
	_Server->get_module("smtp")->send_mail(
	                     s->query_attribute(USER_EMAIL),
	                     @args);
    }
}

void add_subscriber(object u)
{
    _SECURITY->check_access(this(), this_user(),
			    SANCTION_READ,ROLE_READ_ALL, false);
    if ( u->get_object_class() & CLASS_GROUP ) {
	foreach(u->get_members(CLASS_USER), object user) {
	    if ( objectp(user) && search(subscribers, user) == -1 )
		    subscribers += ({ user });
	}
    }
    else if ( search(subscribers, u) == -1 )
	subscribers += ({ u });
    require_save("monitor");
}

void remove_subscriber(object u)
{
    _SECURITY->check_access(this(), this_user(),
			    SANCTION_READ,ROLE_READ_ALL, false);
    subscribers -= ({ u });
    require_save("monitor");
}

static void message_sender()
{
    mapping collects = ([ ]);
    object msg;
    while ( tMessages->size() > 0 && (msg = tMessages->read()) )
    {
	if ( !mappingp(collects[msg->user]) )
	    collects[msg->user] = ([ ]);
	if ( !arrayp(collects[msg->user][msg->classification]) )
	    collects[msg->user][msg->classification] = ({ });
	collects[msg->user][msg->classification] += ({ msg });
    }
    foreach(indices(collects), object user) {
	foreach ( indices(collects[user]), int event ) {
	    string message = "";
	    mapping header;
	    object sender;
	    foreach(collects[user][event], msg) {
		message += msg->msg[(user->query_attribute(USER_LANGUAGE)!=0?user->query_attribute(USER_LANGUAGE):"english")] + "\n<br/>---\n<br/>";
		    header = notify_header(msg->classify_object,event);
		    sender = msg->classify_object;
	    }
	    user->mail(message, header[(user->query_attribute(USER_LANGUAGE)!=0?user->query_attribute(USER_LANGUAGE):"english")], (objectp(sender)?
					 sender->get_object_id() + "@" +
					 _Server->get_server_name():""));
	}
    }
    call(message_sender, SENDER_TIME); // repeat sending
}

void monitor(int event, mixed ... params)
{
    mapping msg;
    mapping header;
    object   event_obj;

    if ( event & EVENT_ATTRIBUTES_CHANGE )
	return;

    if ( !objectp(oMonitor) || !objectp(this_user()) )
      return;

    iEventCount++;


    string eventStr = "";
    if ( event & EVENTS_MONITORED )
	eventStr = "monitored: ";
    switch ( event & 0x0fffffff ) {
    case EVENT_ENTER_INVENTORY:
	eventStr += " enter-inventory";
	break;
    case EVENT_LEAVE_INVENTORY:
	eventStr += "leave-inventory";
	break;
    case EVENT_MOVE:
	eventStr += "moved";
	break;
    case EVENT_ANNOTATE:
	eventStr += "annotate";
	break;
    case EVENT_DOWNLOAD:
	eventStr += "download";
	break;
    case EVENT_DELETE:
	eventStr += "deleted";
	break;
    default:
	eventStr += "("+event +")";
    }

    mapping templ = ([ "{USERNAME}":
                              this_user()->query_attribute(USER_FIRSTNAME)+" "+
                              this_user()->query_attribute(USER_FULLNAME),
		       "{USEREMAIL}": this_user()->get_steam_email(),
		       "{ENVNAME}": oMonitor->get_identifier(),
                       "{ENVURL}":"",
		       "{EVENT}": eventStr,
		       ]);

    int cnt = 1;
    foreach ( params, mixed param ) {
	templ["{PARAM"+cnt+"}"] = (objectp(param) ? param->get_identifier() + "["+param->get_object_id()+"]": (arrayp(param) ? "[ARRAY]": param));
	cnt++;
    }
#if 0
    msg = replace(CONTENTOF("/documents/events/debug.mail"),
		  indices(templ), values(templ));
    _ROOT->mail(msg, "event debug");
#endif
    switch(event) {
    case EVENT_ENTER_INVENTORY:
	if ( !(params[1]->get_object_class() & CLASS_DOCUMENT) )
	    return;
	header = notify_header(params[1], event);
	event_obj = params[1];
	templ["{OBJNAME}"] = params[1]->get_identifier();
	msg = RES(CONTENTOF("/documents/events/entering_object.mail"), CONTENTOF("/documents/events/entering_object_de.mail"));
	break;
    case EVENT_DOWNLOAD|EVENTS_MONITORED:
	if ( !arrayp(mDownloads[params[2]]) )
	    mDownloads[params[2]] = ({ });
	mDownloads[params[2]] += ({ this_user() });
	require_save("monitor");
	return; // no message
	break;
    case EVENT_LEAVE_INVENTORY:
	if ( !(params[1]->get_object_class() & CLASS_DOCUMENT) )
	    return;
	event_obj = params[1];
	templ["{OBJNAME}"] = params[1]->get_identifier();

	msg = RES(CONTENTOF("/documents/events/leaving_object.mail"), CONTENTOF("/documents/events/leaving_object_de.mail"));
	break;
    case EVENT_ANNOTATE:
         header = notify_header(get_monitoring(), EVENT_ANNOTATE);
	event_obj = get_environment();
	msg = RES(ROOMSTR(get_environment())+"New Comment on Room.", ROOMSTRDE(get_environment())+"New Comment on Room.");
	send_subscribers(event_obj, msg, header);
	return;
	break;
    case EVENT_ANNOTATE|EVENTS_MONITORED:
	event_obj = params[2];
	// find root of thread...
	object parent = event_obj->get_annotating();
	while ( objectp(parent) ) {
	    event_obj = parent;
	    parent = parent->get_annotating();
	}
	if ( event_obj->get_object_class() & CLASS_USER )
	    return;

	msg = RES(CONTENTOF("/documents/events/annotation.mail"), CONTENTOF("/documents/events/annotation_de.mail") );
	header = RES("("+event_obj->get_identifier()+"): " + params[3]->get_identifier(), "("+event_obj->get_identifier()+"): " + params[3]->get_identifier());

	string mtype = params[3]->query_attribute(DOC_MIME_TYPE);
	if ( !stringp(mtype) ) mtype = "text/html";
	if ( search(mtype, "text") == -1 ) {
	    send_subscribers(params[3], params[3]->get_content(), header,
			     params[3]->query_attribute(DOC_MIME_TYPE));
	}
	templ["{OBJNAME}"] = params[2]->get_identifier();
	templ["{OBJURL}"]  = "";
	templ["{SUBJECT}"] = params[3]->get_identifier();
	templ["{MESSAGE}"] = params[3]->get_content();
	templ["{PARENT_MESSAGE}"]= (params[2]->get_object_class() & CLASS_DOCUMENT ? params[2]->get_content() : "");
	templ["{PARENT_USER}"]=
                params[2]->get_creator()->query_attribute(USER_FIRSTNAME)+" "+
                params[2]->get_creator()->query_attribute(USER_FULLNAME);
         foreach(indices(msg), object key) {
           if (stringp(msg[key]))
	     msg[key] = replace(msg[key], indices(templ), values(templ));
         }

	 send_subscribers(params[3], msg, header);
	 return;
	 break;
    default:
	return;
    }
    foreach(indices(msg), object key) {
      if (stringp(msg[key])) msg[key] = replace(msg[key], indices(templ), values(templ));
    }
    send_subscribers_collect(msg, event, event_obj);
    require_save("monitor");
}

array get_said()
{
    return aSaid;
}

// for irc...
string status(int event)
{

}
