
inherit "/classes/Script";

import httplib;
 
#include <macros.h>
#include <database.h>
#include <classes.h>
#include <attributes.h>
#include <events.h>
#include <config.h>
#include <client.h>
#include <web.h>
#include <exception.h>
#include <types.h>
#include <access.h>
#include <roles.h>

#define _XML _Server->get_module("Converter:XML")
#define _SABLOTRON _Server->get_module("libxslt")


array(string) automatic_tasks = ({ "create_group_exit", "remove_group_exit" });

/**
 * execute - execute the script
 *  
 * @param vars - variables
 * @return the html representation for an object
 * @author Thomas Bopp (astra@upb.de) 
 */
string|int
execute(mapping vars)
{
    int   oid = (int)vars["object"];
    object                      obj;
    string                      xml;
    string                     html;
    mixed                  err, xsl;
    mapping                  xslMap;
    object                     user;
    array(object)            groups;
    array                     tasks;


    try_event(EVENT_EXECUTE, CALLER, this_object());

    user = this_user();

    obj = find_object(oid);
    if ( !objectp(obj) ) {
	obj = user->query_attribute(USER_WORKROOM);
	vars["object"] = (string)obj->get_object_id();
    }

    if ( !objectp(obj) ) 
	return -2;

    if ( obj == _ROOTROOM ) {
	if ( !_ADMIN->is_member(user) )
	    return redirect("/home/"+user->get_identifier()+"/", 0);
    }

    if ( obj->get_object_class() & CLASS_CONTAINER ) {
      err = catch {
	_SECURITY->check_access(obj, user, SANCTION_READ,ROLE_READ_ALL, false);
      };
      if ( err != 0 ) {
	  return result_page("No access to view "+ obj->get_identifier(),
			     "JavaScript:history.back();");
      }
    }

    if ( obj->get_object_class() & CLASS_ROOM ) {
	if ( !(user->get_status() & CLIENT_FEATURES_MOVE) && user->get_environment() != obj)
	    err = catch { user->move(obj); };
    }
    else {
	// get the room in the environment
	object r_env = obj->get_environment();
	while ( objectp(r_env) && (r_env->get_object_class() & CLASS_ROOM) )
	    r_env = r_env->get_environment();
	if ( objectp(r_env) )
	    if ( !(user->get_status() & CLIENT_FEATURES_MOVE) &&
		 user->get_environment() == r_env )
		err = catch {
		    user->move(r_env);
		};
    }

    mapping client_map = identify_browser(
	vars["__internal"]["client"],
	vars["__internal"]["request_headers"]);
    string client_prefix = "";
    string l = this_user()->query_attribute(USER_LANGUAGE);
    if ( stringp(l) )
	client_map->language = l;
    
    object tmod = _Server->get_module("tasks");
    if ( objectp(tmod) ) {
	tasks = tmod->get_tasks(user);
	int todo = 0;
	if ( arrayp(tasks) && sizeof(tasks) > 0 ) {
	    if ( !stringp(vars["type"]) )
		vars["type"] = "content";
	    html = "<form action='/scripts/browser.pike'>"+
		"<input type='hidden' name='_action' value='tasks'/>"+
		"<input type='hidden' name='object' value='"+vars["object"]+"'/>"+
		"<input type='hidden' name='type' value='"+vars["type"]+"'/>"+
		"<input type='hidden' name='id' value='"+vars["object"]+"'/>"+
		"<input type='hidden' name='room' value='"+vars["room"]+"'/>"+
		"<input type='hidden' name='mode' value='"+vars["mode"]+"'/>"+
		"<h3>Tasks:</h3><br/><br/>";

	    foreach(tasks, object t) {
    if (!objectp(t))
      continue;
    int auto = 1;
    mixed err = catch{ 
      auto = search(automatic_tasks, t->func);
    };
    if ( auto == -1  ) {
      html += "<input type='checkbox' name='tasks' value='"+
			t->tid+"' checked='true'/><SPAN CLASS='text0sc'> "+
			t->descriptions[client_map["language"]] + 
			"</SPAN><br/>\n";
		    todo++;
		}
		else if ( t->obj == 
			  _FILEPATH->path_to_object("/scripts/browser.pike") ) 
		{
		   tmod->run_task(t->tid); // risky ?
		}
	    }
	    html += "<br/><br/><input type='submit' value='ok'/></form>";
	    if ( todo > 0 )
		return result_page(html, "no");
	}
	tmod->tasks_done(user);
    }


    if ( !stringp(vars->type) || vars->type == "" ) {
	if ( obj->get_object_class() & CLASS_MESSAGEBOARD )
	    vars->type = "annotations";
	else
	    vars->type = "content";
    }
    
    // PDA detection - use different stylesheet (xsl:PDA:content, etc)
    if ( client_map["xres"] == "240" ) 
	vars->type = "PDA:" + vars->type;


    xsl = httplib.get_xsl_stylesheet(user, obj, vars);
    if ( !objectp(xsl) )
	steam_error("XSL Stylesheet not found for type "+vars->type+" !\n");
    
    LOG("XSL:"+xsl->get_identifier());
    
    err = catch {
	    xml = _XML->get_xml(obj, xsl, vars, (int)vars["active"]);
    };
    if ( err != 0 ) {
	LOG("XMLCONV:\n"+PRINT_BT(err));
	throw(err);
    }

    if ( vars["source"] == "true" ) {
	xml = (((xml/"<")*"&lt;")/">")*"&gt;";
  return xml;
	return "<html><body><pre>\n"+xml + "\n</pre></body></html>";
    }
    
    LOG("OBJ="+master()->describe_object(obj));

    vars["host"] = _Server->query_config(CFG_WEBSERVER);
    vars["port_ftp"] = (string)_Server->query_config(CFG_WEBPORT_FTP);
    vars["port_http"] = (string)_Server->query_config(CFG_WEBPORT_HTTP);
    vars["port_irc"] = (string)_Server->query_config("irc_port");
    vars["user_id"] = (string)(objectp(user) ? user->get_object_id(): "0");
    vars["selection"] = (string)(objectp(user)? (
	arrayp(user->query_attribute("USER_SELECTION"))?
	sizeof(user->query_attribute("USER_SELECTION")):"0") : "0");

    vars += client_map;

    html = _SABLOTRON->run(xml, xsl, vars);

    run_event(EVENT_EXECUTE, CALLER, this_object());
	
    return html;
}


/**
 * Add a visited room or container to the trail of objects.
 *  
 * @param object visit - the visited place
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>) 
 */
void add_trail(object user, object visit, int max_size)
{
    array aTrail = user->query_attribute("trail");
    if ( !arrayp(aTrail) ) 
	aTrail = ({ visit });
    else {
	if ( visit == aTrail[-1] )
	    return;
	aTrail += ({ visit });
	if ( sizeof(aTrail) > max_size )
	    aTrail = aTrail[sizeof(aTrail)-max_size..];
    }
    user->set_attribute("trail", aTrail);
}

/**
 * Get the complete trail.
 *  
 * @return the trail.
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>) 
 */
array(object) get_trail(object user)
{
    return user->query_attribute("trail");
}

object get_last_trail(object user)
{
    array rooms =  user->query_attribute("trail");
    if ( arrayp(rooms) )
	return rooms[-1];
    return 0;
}


/**
 * Pop the first element from the trail.
 *  
 * @return the popped element.
 * @author <a href="mailto:astra@upb.de">Thomas Bopp</a>) 
 */
object pop_trail(object user)
{
    array(object) aTrail = user->query_attribute("trail");
    if ( sizeof(aTrail) > 0 ) {
	object obj = aTrail[0];
	aTrail = aTrail[1..];
	user->set_attribute("trail", aTrail);
	return obj;
    }
    return 0;
}
