inherit "/classes/Script";

#include <macros.h>
#include <database.h>
#include <classes.h>
#include <attributes.h>
#include <events.h>
#include <config.h>
#include <web.h>
#include <exception.h>
#include <types.h>

#define _XMLCONVERTER _Server->get_module("Converter:XML")
#define _SABLOTRON _Server->get_module("libxslt")

void init()
{
    ::init();
}

/**
 * execute - execute the script
 *  
 * @param vars - variables
 * @return the html representation for an object
 * @author Thomas Bopp (astra@upb.de) 
 */
string|int
execute(mapping vars)
{
    int   oid = (int)vars["object"];
    object                  env,obj;
    object            previous,next;
    int                 objposition;
    string                      xml;
    string                     html;
    mixed                  err, xsl;
    object                     user;
    array(object)            groups;
    array(string)       xmlfieldtmp;
    array(string)          xmlfield;
    string      type = vars["type"];
    string         mimetypeprevious;
    string             mimetypenext;
    
    try_event(EVENT_EXECUTE, CALLER, this_object());

    user = this_user();
    
    if ( oid == 0 ) {
	obj = user->query_attribute(USER_WORKROOM);
	if ( !objectp(obj) ) obj = _ROOTROOM;
	
	vars["object"] = obj->get_object_id();
    }
    else {
	obj = find_object(oid);
    }
    if ( !objectp(obj) ) 
	return -1;

    if ( !stringp(type) || type == "" ) 
	type = "content";

    mapping client_map = identify_browser(
	vars["__internal"]["client"],
	vars["__internal"]["request_headers"]);
    string client_prefix = "";

    string l = this_user()->query_attribute(USER_LANGUAGE);

    if ( stringp(l) )
        client_map->language = l; 

    vars|=client_map;

    mapping xslMap = obj->query_attribute("xsl:"+type);
    if ( !mappingp(xslMap) ) xslMap = ([ ]);
    object aGroup = user->get_active_group();
    xsl = find_object((int)vars["style"]);
    if ( !objectp(xsl) ) {
	if ( objectp(xslMap[aGroup]) )
	    xsl = xslMap[aGroup];
	else {
	    groups = user->get_groups();
	    foreach( groups, object grp ) {
		if ( objectp(xslMap[grp]) )
		    xsl = xslMap[grp];
	    }
	}
    }

    if ( !objectp(xsl) )
	xsl = _FILEPATH->path_to_object("/stylesheets/"+type+".xsl");
    
  
    LOG("XSL:"+xsl->get_identifier());
    
    /* get XML data for requested object */
    float        t;
    err = catch {
	xml = _XMLCONVERTER->get_xml(obj, xsl, vars, (int)vars["active"]);
    };
    if ( err != 0 ) {
	LOG("XMLCONV:\n"+PRINT_BT(err));
	throw(err);
    }

    /* get environment */
    env = obj->get_environment();
    array(object) inv =env->get_inventory_by_class(CLASS_DOCUMENT|CLASS_IMAGE);

    int prev = 1;
    previous = 0;
    next = 0;
    foreach(inv, object o) {
	if ( o == obj ) {
	    prev = 0; // now look for next
	}
	else if ( search(o->query_attribute(DOC_MIME_TYPE), "image/") == 0 )
	{
	    if ( prev )
		previous = o;
	    else {
		next = o;
		break;
	    }
	}
    }
    
    //strip xml code from last </Object> tag
    xmlfieldtmp = xml / "</Object>";
    xmlfield = xmlfieldtmp[0..(sizeof(xmlfieldtmp)-2)];
    xml = xmlfield * "</Object>";

    
    //Add Previous / Next Information
    if ( objectp(previous) )
    {
	xml = xml + "<previous><id>" + ((string) (previous->get_object_id())) + "</id><name>" + previous->get_identifier() +"</name></previous>";
    }
    if ( objectp(next) )
    {
      xml = xml + "<next><id>" + ((string) (next->get_object_id())) + "</id><name>" + next->get_identifier() + "</name></next>";
    }

    //Add last </Object> Tag
    xml = xml + "</Object>";



    if ( vars["source"] == "true" ) {
	xml = (((xml/"<")*"&lt;")/">")*"&gt;";
	return "<html><body><pre>\n"+xml + "\n</pre></body></html>";
    }
    
    LOG("OBJ="+master()->describe_object(obj));


    vars["host"] = _Server->query_config(CFG_WEBSERVER);
    vars["port_ftp"] = (string)_Server->query_config(CFG_WEBPORT_FTP);
    vars["port_http"] = (string)_Server->query_config(CFG_WEBPORT_HTTP);
    vars["user_name"] = (string)(objectp(user)?user->get_identifier():"none");
    vars["user_id"] = (string)(objectp(user) ? user->get_object_id(): "0");
    vars["selection"] = (string)(objectp(user)? (
	arrayp(user->query_attribute("USER_SELECTION"))?
	sizeof(user->query_attribute("USER_SELECTION")):"0") : "0");

    xslMap = obj->query_attribute("xsl:"+type);
    if ( !mappingp(xslMap) ) xslMap = ([ ]);
    groups = user->get_groups();
    foreach( groups, object grp ) {
	if ( objectp(xslMap[grp]) )
	    xsl = xslMap[grp];
    }

    if ( !objectp(xsl) )
	xsl = _FILEPATH->path_to_object("/stylesheets/"+type+".xsl");

    vars |= client_map;
    html = _SABLOTRON->run(xml, xsl, vars);
    run_event(EVENT_EXECUTE, CALLER, this_object());
    
    return html;
}
