inherit "/classes/Script";

#include <macros.h>
#include <attributes.h>
#include <database.h>
#include <classes.h>
#include <events.h>


string rss_extern(object o, string mode, object fp)
{
  string rss = "";
  string t = get_rss_time(o);

  rss = "<item>\n"+
    " <title>"+o->get_identifier() + "</title>\n"+
    " <description>"+o->query_attribute(OBJ_DESC)+"</description>\n"+
    " <link>"+o->query_attribute(DOC_EXTERN_URL)+"</link>\n";
  
  if ( mode == "2.0" )
    rss +=
      " <category>steam</category>\n"+
      " <category>documents</category>\n"+
      " <pubDate>"+t+"</pubDate>\n";
  rss += "</item>\n";
  return rss;
}

string uml_to_entity(string str)
{
    if ( !stringp(str) ) return "";
    return 
        replace(str, ({ "ä","ö","ü", "Ä", "Ö", "Ü", "&" }),
                ({ "&auml;", "&ouml;", "&uuml;", "&Auml;", "&Ouml;",
                   "&Uuml;", "&amp;" }));
}

string get_rss_author(object o)
{
  object user;
  if ( o->get_object_class() & CLASS_DOCUMENT )
    user = o->query_attribute(DOC_USER_MODIFIED);

  if ( !objectp(user) )
    user = o->get_creator();
  

  return user->get_steam_email()+ " ("+ user->get_name() + ")";
}

string get_rss_time(object o, void|string attr)
{
  if ( !stringp(attr) )
    attr = OBJ_CREATION_TIME;
  string t = ctime(o->query_attribute(attr));
  string day, clock, date, year, month, mday;

  day = t[..2];
  date = t[4..9];
  clock = t[11..18];
  year = t[20..23];
  sscanf(date, "%s %s", month, mday);

  int d;
  sscanf(mday, "%d", d);
  if ( d < 10 )
    mday = "0" + d;
  else
    mday = (string)d;

  string res = day + ", " + mday + " " + month + " " + year + " " + clock + " CET";
  return res;
}

string rss_document(object o, string mode, object fp, string server)
{
  string rss = "";
  string t = get_rss_time(o, DOC_LAST_MODIFIED);
  
  object ann = o->get_annotating();
  string link = "";

  object forum;
  if ( objectp(ann) ) {
    while ( objectp(ann) ) {
      forum = ann;
      ann = ann->get_annotating();
    }
  }
  else {
    link = fp->object_to_filename(o);
  }
  link = httplib.replace_uml(link);
  if ( objectp(forum) )
    link = fp->object_to_filename(forum)+"?active="+o->get_object_id();
  
  
  link = server + link;
  object creator = o->get_creator();
  
  rss = "<item>\n"+
    " <title>"+o->get_identifier() + "</title>\n"+
    " <description>"+o->query_attribute(OBJ_DESC)+"</description>\n"+
    " <link>"+link+"</link>\n";
  if ( mode == "2.0" ) {
    rss +=
      " <category>steam</category>\n"+
      " <category>documents</category>\n"+
      " <pubDate>"+t+"</pubDate>\n"+
      " <lastBuildDate>"+get_rss_time(o, DOC_LAST_MODIFIED)+"</lastBuildDate>\n";

    rss += "<author>"+get_rss_author(o)+"</author>\n";

    if ( o->get_content_size() > 0 ) {
      string str = o->get_content();
      if ( search(o->query_attribute(DOC_MIME_TYPE), "text/html") == 0 ) {
        str = htmllib.parse_rxml(str, ([ ]), ([ ]));
        rss += " <content:encoded><![CDATA[\n"+str + "\n]]></content:encoded>\n";
      }
      else if ( search(o->query_attribute(DOC_MIME_TYPE), "text/wiki") == 0 ) {
        str = get_module("wiki")->wiki_to_html_plain(o);
        str = "<html><body>"+str + "</body></html>";
        
        rss += " <content:encoded><![CDATA[\n"+str + "\n]]></content:encoded>\n";
      }
      else if ( search(o->query_attribute(DOC_MIME_TYPE), "text/") == 0 ) {
        str = uml_to_entity(str);
        rss += " <content:encoded><![CDATA[\n"+str + "\n]]></content:encoded>\n";
      }
    }
  }
  
  rss += "</item>\n";
  return rss;
}

static string rss_doc_versions(object o, object fp, string server)
{
#ifdef DOC_VERSIONS
  mapping versions = o->query_attribute(DOC_VERSIONS);
  if ( !mappingp(versions) )
    return "";
  int i = 1;
  string rss = "";
  while ( objectp(versions[i]) ) {
    object v = versions[i];
    string link = "/scripts/get.pike?object="+v->get_object_id();
    link = server + link;
    object creator = v->get_creator();
    string t = 
    rss += "<item>\n"+
      " <title>Version: "+ v->query_attribute(DOC_VERSION)+ "</title>\n"+
      " <description>"+v->query_attribute(OBJ_DESC)+"</description>\n"+
      " <link>"+link+"</link>\n"+
      " <category>steam</category>\n"+
      " <category>wiki</category>\n"+
      " <category>versions</category>\n"+
      " <pubDate>"+get_rss_time(v)+"</pubDate>\n"+
      " <lastBuildDate>"+get_rss_time(v, DOC_LAST_MODIFIED)+"</lastBuildDate>";
    
    rss += "<author>" + get_rss_author(v) + "</author>\n";
    
    rss +=" </item>\n";
    i++;
  }
  return rss;
#endif
  return "";
}


mixed execute(mapping vars)
{
  int           feed;
  object         obj;
  string mode = "2.0";
  string tag = "rss";
  string mimetype = "application/rss+xml";
  

  object fp = vars->fp;
  if ( !objectp(fp) ) {
    fp = _FILEPATH;
  }

  string server = _Server->get_server_name();
  if ( fp == _FILEPATH )
    server = "https://"+server;
  else
    server = "http://"+server;

  feed = (int)vars->feed;

  if ( stringp(vars->v) )
    mode = vars->v;
  
  if ( feed == 0 ) {
    if ( stringp(vars->feed) )
      obj = fp->path_to_object(vars->feed);
  }
  else
    obj = find_object(feed);

  if ( !objectp(obj) )
    //return -1;
    return ({ "Feed not found: invalid object " + vars->feed, "text/html" });
  
  string rss = "<?xml version='1.0' encoding='utf-8'?>\n";
  switch ( mode ) {
    case "0.9" :
      rss += "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns=\"http://my.netscape.com/rdf/simple/0.9/\">";
      tag = "rdf";
      mimetype = "text/xml";
      break;
    default :
    rss += "<rss version='"+mode+"'>\n";
    tag = "rss";
    mimetype = "application/rss+xml";
  }

  // try to use rss module first:
  object rssmod = get_module("rss");
  if ( objectp(rssmod) ) 
      return ({ rssmod->rss(obj, fp, mode), mimetype });

  // otherwise create the feed ourselves:
  rss += "<channel>\n"+
    "<title>"+obj->query_attribute(OBJ_NAME)+"</title>\n"+
    "<description>"+obj->query_attribute(OBJ_DESC)+"</description>\n"+
    "<link>"+server+fp->object_to_filename(obj)+"</link>\n";
  
  rss += "<generator>http://www.open-steam.org/scripts/rss.pike?v=0.1</generator>\n";
  
  array inv;
  
  if ( obj->get_object_class() & CLASS_CONTAINER )
    inv = obj->get_inventory();
  else if ( obj->get_object_class() & CLASS_MESSAGEBOARD )
    inv = obj->get_annotations();
  else if ( obj->get_object_class() & CLASS_DOCUMENT ) {
    // first versioning for wiki
    if ( obj->query_attribute(DOC_MIME_TYPE) == "text/wiki" )
      rss += rss_doc_versions(obj, fp, server);
    rss += rss_document(obj, mode, fp, server);
    inv = ({ });
  }
  
  foreach(inv, object o) {
    if ( !objectp(o) )
      continue;
    if ( o->get_object_class() & CLASS_DOCUMENT ) {
      rss += rss_document(o, mode, fp, server);
    }
    else if ( o->get_object_class() & CLASS_DOCEXTERN ) {
      rss += rss_extern(o, mode, fp);
    }
  }
  
  rss += "</channel>\n";
  rss += "</" + tag + ">\n";
  return ({ rss, mimetype });
}

#define REFRESH_TIME 60*15 // 15 minutes

static mapping import_cache = ([ ]);
static mapping import_time = ([ ]);

string rss_import(string url)
{
  string data;
  if ( import_time[url] > 0 && time() - import_time[url] < REFRESH_TIME )
    data = import_cache[url];
  else {
    import_cache[url] = _rss_import(url);
    import_time[url] = time();
    data = import_cache[url];
  }
  return data;
}

static string _rss_import(string url)
{
  // this needs to be cached
  string data = Protocols.HTTP.get_url_data(url);
  if ( !stringp(data) ) {
    werror("Unable to fetch url " + url);
  }
  array lines = data / "\n";
  data = "";
  foreach(lines, string l) {
    if ( search(l, "<?") == -1 && search(l, "<!") == -1 )
      data += l + "\n";
  }
  return data;
}
