inherit "/classes/Script";

#include <macros.h>
#include <attributes.h>
#include <classes.h>
#include <database.h>

static object services = get_module("ServiceManager");

string object_to_dc(object obj)
{
  string rdf = "<rdf:RDF\n"+
  "xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n"+
  "xmlns:dc=\"http://purl.org/dc/elements/1.1/\">\n";

  object creator = obj->get_creator() || USER("root");

  rdf += sprintf("<rdf:Description rdf:about=\"%s\">\n"+
                 "<dc:creator>%s</dc:creator>\n"+
                 "<dc:title>%s</dc:title>\n"+
                 "<dc:description>%s</dc:description>\n"+
                 "<dc:subject>%s</dc:subject>\n"+
                 "<dc:source>%s</dc:source>\n"+
                 "<dc:date>%s</dc:date>\n"+
                 "<dc:type>%s</dc:type>\n"+
                 "<dc:identifier>%d</dc:identifier>\n"+
                 "</rdf:Description>\n",
                 get_module("filepath:tree")->object_to_filename(obj),
                 creator->get_name(),
                 obj->get_identifier() || "",
                 obj->query_attribute(OBJ_DESC) || "",
                 "",
                 "",
                 (string)obj->query_attribute(OBJ_CREATION_TIME),
                 obj->query_attribute(DOC_MIME_TYPE) || "",
                 obj->get_object_id()
                 );
  rdf += "</rdf:RDF>"; 
  return rdf;

}


class ReturnHandler {
    inherit Async.HtmlHandler;
    mapping finished = ([ ]);
    array returnObjects;
    
    void create() {
        // set head and foot
        head = "<html><body>";
    }
    
    void set_result_objects(array ret) {
        returnObjects = ret;
        foreach ( returnObjects, object obj) {
            if ( !objectp(obj) )
                continue;
            obj->resultFunc = asyncResult;
        }
    }
    
    void asyncResult(mixed id, mixed result) {
        if ( arrayp(result) ) {
            for ( int i = 0; i < sizeof(result); i++ ) {
                if ( objectp(result[i]) )
                    result[i] = object_to_dc(result[i]);
                else if ( intp(result[i]) )
                    result[i] = object_to_dc(find_object(result[i]));
            }
            result = "<search>\n"+
                (result * "\n")+"</search>";
  
        } else if ( !stringp(result) ) {
            werror("0-Result from %O\n", id);
            return;
        }
        // xslt
        object xsl = OBJ("/stylesheets/searchResult.xsl");
        string xml = "<?xml version='1.0' encoding='utf-8'?>\n" + result;
        _fd->write(get_module("libxslt")->run(xml, xsl, ([ ])));
    }
}

mixed execute(mapping vars)
{
    if ( !stringp(vars->_action) ) {
        return result_page("<form action='/sources/search.pike' method='POST'>"+
            "<input type='hidden' name='_action' value='search' />"+
            "<input type='hidden' name='type' value='execute' />"+
            "Suche: <input type='text' name='keyword' /> &#160; <input type='submit' value='Suchen' />"+
            "</form>", "");
    }
    object steamRes =  search_steam(vars);
    object milessRes; 
    mixed err = catch(milessRes = search_miless(vars));
    if ( err ) 
        werror("%O\n%O\n", err);
    
    
    array results = ({ steamRes, milessRes });
    object result = ReturnHandler();
    result->set_result_objects(results);
    return result;
}

object search_miless(mapping vars)
{
  mapping params = ([ ]);
  params->command = "search";
  params->hostType = "miless";
  params->hostUrl = "http://ub076.uni-paderborn.de";
  params->formAction = _FILEPATH->object_to_filename(this_object());
  params->vars = vars;

  Async.Return res = services->call_service_async("search-miless", params);
  return res;
}

array oeq(string|int value)
{
   return ({ "=", (string)value });
}
array olt(string|int value)
{
    return ({ "<", (string)value });
}
array ogt(string|int value)
{
    return ({ ">", (string)value });
}


object search_steam(mapping vars) 
{
  array(string) asFilterClasses =
    ([
      "document" : ({"\"/classes/Document\"", "\"/classes/DocGraphics\"", "\"/classes/DocText\"",
		     "\"/classes/DocMP3\""}),
      "container" : ({"\"/classes/Container\"", "\"/classes/Room\"" }),
      "gate" : ({ "\"/classes/Exit\"" }),
      "room" : ({ "\"/classes/Room\"" }),
      "messageboard" : ({"\"/classes/Messageboard\""}),
      "galery" : ({"\"/classes/Container\"", "\"/classes/Room\"" }),
      "link" : ({ "\"/classes/Link\"", "\"/classes/DocExtern\"" }),
      "user" : ({ "\"/classes/User\"" }),
      "group" : ({ "\"/classes/Group\"" }),
      "user+group":  ({ "\"/classes/Group\"", "\"/classes/User\"" }), ])[vars["advsearch_objtype"]];
  
  array(object) oSearchResults = ({ });
  
  string sQuery;
  sQuery=vars["keywords"];
  sQuery = "%"+sQuery+"%";
  
  if ( !arrayp(asFilterClasses) )
    asFilterClasses = ({ });
  vars->user = this_user()->get_object_id();
  object oQueryExecuter = get_module("searching")->searchQuery(0, vars, asFilterClasses);
  oQueryExecuter->first_query(STORE_ATTRIB, OBJ_NAME,
			      oQueryExecuter->like(sQuery),
			      asFilterClasses);
  
  oQueryExecuter->extend(STORE_ATTRIB, OBJ_KEYWORDS,
			 oQueryExecuter->like(sQuery));
  oQueryExecuter->extend(STORE_ATTRIB, OBJ_DESC,
			 oQueryExecuter->like(sQuery));
  
  if (vars["advsearch_objtype"]=="user" || vars["advsearch_objtype"]=="user+group")
  {
    oQueryExecuter->extend(STORE_ATTRIB, USER_FIRSTNAME,
			   oQueryExecuter->like(sQuery));
    oQueryExecuter->extend(STORE_ATTRIB, USER_FULLNAME,
			   oQueryExecuter->like(sQuery));
    oQueryExecuter->extend(STORE_ATTRIB, USER_EMAIL,
			   oQueryExecuter->like(sQuery));
  }
  if (stringp(vars->advsearch_fulltext) && vars["advsearch_fulltext"]!="no")
    oQueryExecuter->extend_ft(vars["keywords"]);

 oQueryExecuter->limit(STORE_ATTRIB, OBJ_VERSIONOF, oQueryExecuter->eq(0));
 
 if ( vars->advsearch_objtype=="document" &&
      vars["advsearch_mimetype"]!="all")
   oQueryExecuter->limit(STORE_ATTRIB, DOC_MIME_TYPE,
			 oQueryExecuter->eq(vars["advsearch_mimetype"]));

 if (stringp(vars["advsearch_creator"]) && strlen(vars->advsearch_creator)>0 )
 {
   object oCreator = USER(vars["advsearch_creator"]);
   if (objectp(oCreator)) {
     string sCreator="%"+oCreator->get_object_id();
     oQueryExecuter->limit(STORE_ACCESS, 
			   "Creator", oQueryExecuter->eq(sCreator));
   }
 }
 if ( stringp(vars["advsearch_modifiedby"]) && strlen(vars->advsearch_modifiedby) > 0 ) {
   object oEditor = USER(vars["advsearch_modifiedby"]);
   if (objectp(oEditor)) {
     string sEditor="%"+oEditor->get_object_id();
     oQueryExecuter->limit(STORE_ATTRIB, DOC_USER_MODIFIED, oQueryExecuter->eq(sEditor));
   }
 }
 
 if ( stringp(vars["advsearch_maxsize"]) && strlen(vars->advsearch_maxsize) > 0 ) {
   int maxsize = (int) vars["advsearch_maxsize"];
   oQueryExecuter->limit(STORE_CONTENT, "CONTENT_SIZE", olt(maxsize));
 }
 if (stringp(vars["advsearch_minsize"]) && strlen(vars->advsearch_minsize)>0 )
 {
   int minsize = (int) vars["advsearch_minsize"];
   oQueryExecuter->limit(STORE_CONTENT, "CONTENT_SIZE", ogt(minsize));
 }
 
 Calendar.Calendar calendar = Calendar.ISO->set_language("german");
 string sdate = vars["advsearch_datefrom"];
 if (sdate !="TT.MM.JJJ") {
   mixed err = catch{
     Calendar.Day date = calendar->dwim_day(sdate);
     oQueryExecuter->limit(STORE_ATTRIB, OBJ_LAST_CHANGED,
                                  ogt(date->unix_time()));
   };
 }
 sdate = vars["advsearch_dateto"];
 if (sdate !="TT.MM.JJJJ") {
   mixed err = catch{
     Calendar.Day date = calendar->dwim_day(sdate);
     oQueryExecuter->limit(STORE_ATTRIB, OBJ_LAST_CHANGED,
			   olt(date->unix_time()));
   };
 } 
 object result = oQueryExecuter->run_async();
 result->processFunc = getSearchObjects;
    
 return result;
}

mixed getSearchObjects(array results)
{
    array objects = ({ });
    
    foreach(results, mixed res) {
        objects += ({ find_object((int)res) });
    }
    return objects;
}

