inherit "/classes/Script";

#include <macros.h>
#include <database.h>
#include <classes.h>
#include <attributes.h>
#include <events.h>
#include <config.h>
#include <web.h>
#include <exception.h>
#include <types.h>
#include <access.h>
#include <roles.h>

#define _XML _Server->get_module("Converter:XML")
#define _SABLOTRON _Server->get_module("libxslt")

void init()
{
    ::init();
}

/**
 * execute - execute the script
 *  
 * @param vars - variables
 * @return the html representation for an object
 * @author Thomas Bopp (astra@upb.de) 
 */
string|int
execute(mapping vars)
{
    int   oid = (int)vars["object"];
    object                      obj;
    string                      xml;
    string                     html;
    mixed                  err, xsl;
    mapping                  xslMap;
    object                     user;
    array(object)            groups;
    string      type = vars["type"];

    try_event(EVENT_EXECUTE, CALLER, this_object());

    user = this_user();
    
    if ( oid == 0 ) {
	obj = user->query_attribute(USER_WORKROOM);
	if ( !objectp(obj) ) obj = _ROOTROOM;
	
	vars["object"] = obj->get_object_id();
    }
    else {
	obj = find_object(oid);
    }
    if ( !objectp(obj) ) 
	return -2;

    _SECURITY->check_access(obj, user, SANCTION_READ,ROLE_READ_ALL, false);
    if ( user == _GUEST && obj->get_object_class() & CLASS_MESSAGEBOARD )
	return -1;

    if ( obj->get_object_class() & CLASS_CONTAINER ) {
	string iname = obj->query_attribute("html:index");
	if ( !stringp(iname) )
	    iname = "index.html";
	object index = obj->get_object_byname(iname);
	if ( objectp(index) )
	    return index->get_content();
    }

    xsl = obj->query_attribute("xsl:public");
    if ( !objectp(xsl) )
	xsl = _FILEPATH->path_to_object("/stylesheets/public.xsl");
    
    err = catch {
	xml = _XML->get_xml(obj, xsl, vars, (int)vars["active"]);
    };
    if ( err != 0 ) {
	LOG("XMLCONV:\n"+PRINT_BT(err));
	throw(err);
    }

    if ( vars["source"] == "true" ) {
	xml = (((xml/"<")*"&lt;")/">")*"&gt;";
	return "<html><body><pre>\n"+xml + "\n</pre></body></html>";
    }
    
    LOG("OBJ="+master()->describe_object(obj));

    vars["host"] = _Server->query_config(CFG_WEBSERVER);
    vars["port_ftp"] = (string)_Server->query_config(CFG_WEBPORT_FTP);
    vars["port_http"] = (string)_Server->query_config(CFG_WEBPORT_HTTP);
    vars["port_irc"] = (string)_Server->query_config("irc_port");
    vars["user_name"] = (string)(objectp(user)?user->get_identifier():"none");
    vars["user_id"] = (string)(objectp(user) ? user->get_object_id(): "0");
    vars["selection"] = (string)(objectp(user)? (
	arrayp(user->query_attribute("USER_SELECTION"))?
	sizeof(user->query_attribute("USER_SELECTION")):"0") : "0");
    int i;
    mapping client_map = identify_browser(
	vars["__internal"]["client"],
	vars["__internal"]["request_headers"]);

    if ( client_map->language != "english" &&
	 client_map->language != "german" )
	client_map["language"] = "english";

    vars |= client_map;

    float ti = gauge {
	html = _SABLOTRON->run(xml, xsl, vars);
    };
    LOG("XSL: took really="+ti+"seconds...");
    run_event(EVENT_EXECUTE, CALLER, this_object());
    //html += "<!--\n"+xml+"\n-->\n";
    return html;
}


