<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="xml" indent="yes" doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd" doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN" encoding="utf-8"/>

<xsl:param name="max_age">2592000</xsl:param>

<xsl:template match="thread">
<xsl:param name="script"/>
<xsl:variable name="myid"><xsl:value-of select="id" /></xsl:variable> 
<xsl:variable name="active-thread"><xsl:choose ><xsl:when test="count(/Object/active-thread/array/object[id=$myid])>0">1</xsl:when><xsl:otherwise>0</xsl:otherwise></xsl:choose></xsl:variable>
<xsl:if test="((depth=0 and $active-thread=0) or $active-thread=1)">
  <li>
    <a name="aid{id}"></a>
    <xsl:choose>
      <xsl:when test="depth=0"> 
        <xsl:if test="$active-thread=1 and depth=0">
          <xsl:choose>
          <xsl:when test="count(thread)>0">
            <img class="treenode" src="/images/treenode_collapse.gif" alt="collapse tree" />
          </xsl:when>
          <xsl:otherwise>
            <img class="treenode" src="/images/treenode_empty.gif" alt="empty tree" />
          </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
        <xsl:if test="$active-thread=0 and depth=0">
          <xsl:choose>
            <xsl:when test="count(thread)>0">
              <a class="tree" href="{$script}?object={$object}&amp;type=annotations&amp;active={id}&amp;room={$room}&amp;from_obj={$from_obj}&amp;to_obj={$to_obj}"><img class="treenode" src="/images/treenode_expand.gif" alt="expand tree" /></a>
            </xsl:when>
            <xsl:otherwise>
              <img class="treenode" src="/images/treenode_empty.gif" alt="empty tree" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <!-- for all parent, grandparent, etc within this thread -->
        <xsl:for-each select="ancestor::thread">
          <xsl:variable name="mmid"><xsl:value-of select="id"/></xsl:variable>
          <xsl:choose>
            <xsl:when test="count(following-sibling::thread/subject) = 0">
             <img class="treenode" src="/images/treenode_spaceholder.gif" VSPACE="1" BORDER="0" ALIGN="left"/>
            </xsl:when>
            <xsl:otherwise>
              <img class="treenode" src="/images/treenode_vline.gif" VSPACE="1" BORDER="0" ALIGN="left"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
        <xsl:choose>
          <xsl:when test="position() = last()">
            <img class="treenode" src="/images/treenode_qline.gif" VSPACE="1" BORDER="0" ALIGN="left"/>
          </xsl:when>
          <xsl:otherwise>
            <img class="treenode" src="/images/treenode_tline.gif" VSPACE="1" BORDER="0" ALIGN="left"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:variable name="mysubject">
      <xsl:choose><xsl:when test="string(subject)=''">{ANNOTATION_NOSUBJECT}</xsl:when><xsl:otherwise><xsl:value-of select="subject"/></xsl:otherwise></xsl:choose>
    </xsl:variable>
    <span>
     <xsl:attribute name="class"><xsl:choose><xsl:when test="id=$active">current</xsl:when><xsl:otherwise>not_current</xsl:otherwise></xsl:choose></xsl:attribute>
     <xsl:choose>
       <xsl:when test="id=$active">
         <xsl:value-of select="$mysubject"/> 
       </xsl:when>
       <xsl:when test="read=0 or (depth=0 and count(descendant::thread[read=0 and id != $active])>0 and $active-thread=0)">
         <strong>
           <a href="{$script}?object={$object}&amp;type=annotations&amp;active={id}&amp;room={$room}&amp;from_obj={$from_obj}&amp;to_obj={$to_obj}"><xsl:value-of select="$mysubject"/></a>
         </strong>
       </xsl:when>
       <xsl:otherwise>
         <a href="{$script}?object={$object}&amp;type=annotations&amp;active={id}&amp;room={$room}&amp;from_obj={$from_obj}&amp;to_obj={$to_obj}"><xsl:value-of select="$mysubject"/></a>
       </xsl:otherwise>
     </xsl:choose>
     &#160;({ANNOTATION_BY}
     <a href="/scripts/navigate.pike?object={author/object/id}&amp;type=details&amp;room={$room}" title="{ANNOTATION_USER_OPENPROFILE}"><xsl:value-of select="author/object/name"/></a>,
     <xsl:text disable-output-escaping="yes">&lt;script type="text/javascript"&gt;//&lt;![CDATA[
       print_transformed_date("</xsl:text><xsl:value-of select="created/date"/><xsl:text disable-output-escaping="yes">");
     //]]&gt;&lt;/script&gt;</xsl:text>,&#160;
     <xsl:if test="modified/time!=created/time">
       {ANNOTATION_THREAD_MODIFIED_ON}
       <xsl:text disable-output-escaping="yes">&lt;script type="text/javascript"&gt;//&lt;![CDATA[
         print_transformed_date("</xsl:text><xsl:value-of select="modified/date"/><xsl:text disable-output-escaping="yes">");
       //]]&gt;&lt;/script&gt;</xsl:text><xsl:if test="/Object/access/annotateable/@user != 'false'">,&#160;</xsl:if>
     </xsl:if>
     <xsl:if test="/Object/access/annotateable/@user != 'false'">
       <a href="javascript:open_edit('object={id}&amp;type=annotreply&amp;active={id}&amp;room={$room}&amp;objtype={/Object/@type}')" title="{ANNOTATION_THREAD_STARTREPLY}{NEW_WINDOW}">{ANNOTATION_THREAD_STARTREPLY}</a>
     </xsl:if>)
    </span>
  </li>

  <xsl:apply-templates select="thread"/>
</xsl:if>
</xsl:template>


<xsl:template name="formatHTMLnewlines">
  <xsl:param name="orgtext"/>
  <xsl:variable name="lPart" select="substring-before($orgtext,'&#10;')"/>
  <xsl:variable name="rPart" select="substring-after($orgtext,'&#10;')"/>
   <xsl:choose>
     <xsl:when test="contains($orgtext,'&#10;')">
      <xsl:value-of select="$lPart"/><BR/>
     </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$orgtext"/>
    </xsl:otherwise>
  </xsl:choose>

  <xsl:choose>
    <xsl:when test="contains($rPart,'&#10;')">
      <xsl:call-template name="formatHTMLnewlines"><xsl:with-param name="orgtext"><xsl:value-of select="$rPart"/></xsl:with-param>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$rPart"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template match="annotations" name="annotations">
<xsl:param name="script"/>
<xsl:param name="ispublic">false</xsl:param>
  <xsl:choose>
    <xsl:when test="/Object/annotated!=0">
      <xsl:for-each select="Object">
        <xsl:if test="$active=id">
          <div class="userimage">
            <xsl:choose>
              <xsl:when test="@type='Container'">
                <a href="/scripts/navigate.pike?object={modified-by}&amp;type=details&amp;room={$room}" title="{ANNOTATION_USER_OPENPROFILE}"><img class="usericon_annotation" src="/scripts/get.pike?object={modified-by}" alt="{modified-by/object/name}" /></a>
              </xsl:when>
              <xsl:otherwise>
                <a href="/scripts/navigate.pike?object={modified-by/object/id}&amp;type=details&amp;room={$room}" title="{ANNOTATION_USER_OPENPROFILE}"><img class="usericon_annotation" src="/scripts/get.pike?object={modified-by/object/id}" alt="{modified-by/object/name}" /></a>
              </xsl:otherwise>
            </xsl:choose>
          </div>

          <h3 class="annotext"><xsl:value-of select="name"/></h3>

          <p class="annotext">{ANNOTATION_CREATED_BY} 
          <a href="/scripts/navigate.pike?object={owner/object/id}&amp;type=details&amp;room={$room}" title="{ANNOTATION_USER_OPENPROFILE}">
          <xsl:value-of select="owner/object/name" disable-output-escaping="yes"/></a> {ANNOTATION_ON} <xsl:text disable-output-escaping="yes">&lt;script type="text/javascript"&gt;//&lt;![CDATA[
              print_transformed_date("</xsl:text><xsl:value-of select="created/date"/><xsl:text disable-output-escaping="yes">")
            //]]&gt;&lt;/script&gt;</xsl:text>

            <xsl:if test="last-modified/time!=created/time">
              <br />
              {ANNOTATION_MODIFIED_BY} 
              <a href="/scripts/navigate.pike?object={modified-by/object/id}&amp;type=details&amp;room={$room}" title="{ANNOTATION_USER_OPENPROFILE}">
              <xsl:value-of select="modified-by/object/name" disable-output-escaping="yes"/></a> {ANNOTATION_ON} <xsl:text disable-output-escaping="yes">&lt;script type="text/javascript"&gt;//&lt;![CDATA[
                print_transformed_date("</xsl:text><xsl:value-of select="last-modified/date"/><xsl:text disable-output-escaping="yes">")
              //]]&gt;&lt;/script&gt;</xsl:text>
            </xsl:if>
          </p>

          <p class="annotext">
            <xsl:choose>
              <xsl:when test="content/mime-type='text/html' or content/mime-type='text/plain' or content/mime-type='text/wiki'">
                <xsl:value-of select="/Object/active-content" disable-output-escaping="yes"/>
              </xsl:when>
              <xsl:otherwise>
                Attachement-MIME-Type: <xsl:value-of select="content/mime-type"/> - <a href="/scripts/get.pike?object={id}">Download</a> (<xsl:value-of select="content/size"/> bytes)
              </xsl:otherwise>
            </xsl:choose>
          </p>

          <xsl:if test="owner/object/id=/Object/user/id or /Object/user/admin = 'true'">
            <p class="right">
              <xsl:if test="content/mime-type='text/html' or content/mime-type='text/plain' or content/mime-type='text/wiki'">
                <xsl:if test="/Object/@type='Messageboard'">
                  <a href="javascript:open_edit('object={id}&amp;type=annotation&amp;active={id}&amp;room={$room}')" title="{ANNOTATION_EDIT_MESSAGEBOARD}{NEW_WINDOW}">
                    {ANNOTATION_EDIT_MESSAGEBOARD}
                  </a>
                </xsl:if>
                <xsl:if test="/Object/@type!='Messageboard'">
                  <a href="javascript:open_edit('object={id}&amp;type=annotation&amp;active={id}&amp;room={$room}')" title="{ANNOTATION_EDIT_OTHER}{NEW_WINDOW}">
                    {ANNOTATION_EDIT_OTHER}
                  </a>
                </xsl:if>
                |
              </xsl:if>
              <a href="javascript:check_delannot(document.browser,{id})">
                <xsl:choose>
                  <xsl:when test="/Object/@type='Messageboard'">{ANNOTATION_DELETE_MESSAGEBOARD}</xsl:when>
                  <xsl:otherwise>{ANNOTATION_DELETE_OTHER}</xsl:otherwise>
                </xsl:choose>
              </a>
            </p>
          </xsl:if>
        </xsl:if>
      </xsl:for-each>

      <p>
        <xsl:if test="/Object/@type='Messageboard'">
          <xsl:if test="/Object/access/annotateable/@user != 'false'">  
            <input type="button" name="newthread" value="{ANNOTATION_ADD_MESSAGEBOARD}" title="{ANNOTATION_ADD_MESSAGEBOARD}{NEW_WINDOW}" onclick="open_edit('object={$object}&amp;type=annotcreate&amp;active={$active}&amp;room={$room}&amp;objtype={/Object/@type}')" />
            <input type="button" name="reply" value="{ANNOTATION_REPLY_MESSAGEBOARD}" title="{ANNOTATION_REPLY_MESSAGEBOARD}{NEW_WINDOW}" onclick="open_edit('object={$active}&amp;type=annotreply&amp;active={$active}&amp;room={$room}&amp;objtype={/Object/@type}')" />
          </xsl:if>
          <xsl:if test="$ispublic='false'">
            <input type="button" name="setallread" value="{ANNOTATION_SETALLREAD_MESSAGEBOARD}" onclick="document.location.href='/scripts/browser.pike?id={$object}&amp;type=annotations&amp;active={$active}&amp;room={$room}&amp;_action=set_read'" />
          </xsl:if>
        </xsl:if>
        <xsl:if test="/Object/@type!='Messageboard'">
          <xsl:if test="/Object/access/annotateable/@user != 'false'">
            <input type="button" name="newthread" value="{ANNOTATION_ADD_OTHER}" title="{ANNOTATION_ADD_OTHER}{NEW_WINDOW}" onclick="open_edit('object={$object}&amp;type=annotcreate&amp;active={$active}&amp;room={$room}&amp;objtype={/Object/@type}')" />
            <input type="button" name="reply" value="{ANNOTATION_REPLY_OTHER}" title="{ANNOTATION_REPLY_OTHER}{NEW_WINDOW}" onclick="open_edit('object={$active}&amp;type=annotreply&amp;active={$active}&amp;room={$room}&amp;objtype={/Object/@type}')" />
          </xsl:if>
          <xsl:if test="$ispublic='false'">
            <input type="button" name="setallread" value="{ANNOTATION_SETALLREAD_OTHER}" onclick="document.location.href='/scripts/browser.pike?id={$object}&amp;type=annotations&amp;active={$active}&amp;room={$room}&amp;_action=set_read'" />
          </xsl:if>
        </xsl:if>
      </p>

      <ul>
        <xsl:apply-templates select="/Object/annotations/thread">
          <xsl:with-param name="script"><xsl:value-of select="$script"/></xsl:with-param>
        </xsl:apply-templates>
      </ul>
    </xsl:when>

    <xsl:when test="@size=0 and parent::Object/access/annotateable/@user='false'">
      <p class="center hint">{ANNO_NO_PERMISSION}</p>
    </xsl:when>

    <xsl:otherwise>
      <p class="center hint">{NO_ANNOS}</p>
      <xsl:if test="/Object/access/annotateable/@user != 'false'">
        <p class="center">
          <xsl:if test="/Object/@type='Messageboard'"><input type="button" name="newthread" value="{ANNOTATION_ADD_MESSAGEBOARD}" onclick="open_edit('object={$object}&amp;type=annotcreate&amp;active={$active}&amp;room={$room}&amp;objtype={/Object/@type}')" title="{ANNOTATION_ADD_MESSAGEBOARD}{NEW_WINDOW}" /></xsl:if>
          <xsl:if test="/Object/@type!='Messageboard'"><input type="button" name="newthread" value="{ANNOTATION_ADD_OTHER}" onclick="open_edit('object={$object}&amp;type=annotcreate&amp;active={$active}&amp;room={$room}&amp;objtype={/Object/@type}')" title="{ANNOTATION_ADD_OTHER}{NEW_WINDOW}" /></xsl:if>
        </p>
      </xsl:if>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="text() | @*">
  <xsl:copy><xsl:apply-templates /></xsl:copy>
</xsl:template>

<xsl:template match="TABLE | TR | TD | SELECT | OPTION | MAP | AREA | A | a | BR">
   <xsl:copy><xsl:apply-templates select="@* | * | text()"/></xsl:copy>
</xsl:template>

</xsl:stylesheet>