<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- wrapper template to bundle calling of Javascript:print_shortername -->
  <xsl:template name="print_shortername">
    <xsl:param name="text"/>
    <xsl:variable name="wbr"><xsl:text>&lt;wbr /&gt;</xsl:text></xsl:variable>
    <xsl:if test="$text!=''">
      <xsl:value-of select="translate($text, '.', $wbr)" disable-output-escaping="yes"/>
    </xsl:if>
  </xsl:template>

<!-- prints the short version of a help text and offers an option to show the long version of the help text -->
  <xsl:template name="print_helptext">
    <xsl:param name="short">none</xsl:param>
    <xsl:param name="long">none</xsl:param>
    <xsl:choose>
      <xsl:when test="$long = 'none'">
        <xsl:value-of select="$short"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$long" disable-output-escaping="yes"/>
<!--        <A TITLE="{INCLUDEMETHODS_MOREHELP_TOOLTIP}" HREF="JavaScript:document.write(<xsl:text><![CDATA[']]></xsl:text><xsl:value-of select='$long'/><xsl:text><![CDATA[']]></xsl:text>);">{INCLUDEMETHODS_MOREHELP}</A>  -->
<!--        <SCRIPT LANGUAGE="Javascript" TYPE="text/javascript">document.write(<xsl:text><![CDATA["]]></xsl:text><xsl:value-of select='$long'/><xsl:text><![CDATA["]]></xsl:text>);</SCRIPT> -->
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

<!-- prints out document size. automatic output in MByte, KByte od Bytes... -->
  <xsl:template name="show_size">
    <xsl:param name="size">0</xsl:param>
    <xsl:choose>
      <xsl:when test="round(($size div 1024) div 1024) > 0">
        <xsl:value-of select="round(($size div 1024) div 1024)"/>&#160;{INCLUDEMETHODS_MBYTE}
      </xsl:when>
      <xsl:when test="round($size div 1024) > 0">
        <xsl:value-of select="round($size div 1024)"/>&#160;{INCLUDEMETHODS_KBYTE}
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$size"/>&#160;{INCLUDEMETHODS_BYTE}
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>  
  
<!-- used by details.xsl and version.xsl -->
  <xsl:template name="show_versions">
    <xsl:param name="this_link"/>  
    <table width="100%" style="auto">
      <tr>
      <th>
        {INCLUDEMETHODS_VERSIONS_NUMBER}
      </th>
      <th>
        {INCLUDEMETHODS_VERSIONS_DESCRIPTION}
      </th>
      <th>
        {INCLUDEMETHODS_VERSIONS_SIZE}
      </th>  
      <th>
        {INCLUDEMETHODS_VERSIONS_MODIFIED}
      </th>
      <th>
        {INCLUDEMETHODS_VERSIONS_MODIFIER}
      </th>
      <th>
        {INCLUDEMETHODS_VERSIONS_ACTION}
      </th>
      </tr>
      <tr>
        <td align="right" valign="top">
          <a href="/scripts/download.pike/{/Object/id}/{/Object/name}"><xsl:value-of select="/Object/version"/></a>
        </td>
        <td valign="top">
          <a href="/scripts/download.pike/{/Object/id}/{/Object/name}"><xsl:value-of select="/Object/name"/></a>
        </td>
        <td valign="top">
          <xsl:call-template name="show_size"><xsl:with-param name="size"><xsl:value-of select="/Object/size"/></xsl:with-param></xsl:call-template>
        </td>      
        <td valign="top">
          <xsl:text disable-output-escaping="yes">&lt;script type="text/javascript"&gt;//&lt;![CDATA[
          print_transformed_date("</xsl:text><xsl:value-of select="/Object/last-modified/date"/><xsl:text disable-output-escaping="yes">");
          //]]&gt;&lt;/script&gt;</xsl:text>
        </td>
        <td valign="top">
          <a href="/scripts/navigate.pike?object={/Object/modified-by/object/id}&amp;type=details&amp;room={$room}"><xsl:value-of select="/Object/modified-by/object/name"/></a>
          <!--<a href="Javascript:open_userdetails_byname('{/Object/modified-by/object/name}');"><xsl:value-of select="/Object/modified-by/object/name"/></a>-->
        </td>
        <td valign="top">
          {INCLUDEMETHODS_VERSIONS_ACTION_ACTUAL}
          <xsl:if test="starts-with(mime-type, 'text') or mime-type = 'source/pike' or mime-type = 'application/x-javascript'">
            <br />
            <a href="{/Object/path/path}?jump=versions&amp;selecteddiff={id}{$this_link}">{INCLUDEMETHODS_VERSIONS_ACTION_SELECT_FOR_DIFF}</a>
            <xsl:if test="count(/Object/versions/Object) > 0">
              <xsl:variable name="lowerversion"><xsl:value-of select="/Object/versions/Object[1]/version"/></xsl:variable>        
              <br />
              <a href="/scripts/navigate.pike?object={/Object/id}&amp;type=diff&amp;newversion={id}&amp;oldversion={/Object/versions/Object[1]/id}&amp;jump=versions">{INCLUDEMETHODS_VERSIONS_ACTION_DIFF_PREVIOUS} <xsl:value-of select="$lowerversion"/></a>
            </xsl:if>
            <xsl:if test="$selecteddiff > 0">
              <br />
              <a href="/scripts/navigate.pike?object={/Object/id}&amp;type=diff&amp;newversion={id}&amp;oldversion={$selecteddiff}&amp;jump=versions">{INCLUDEMETHODS_VERSIONS_ACTION_DIFF_SELECTED} <xsl:value-of select="/Object/versions/Object[id=$selecteddiff]/version"/></a>
            </xsl:if>
          </xsl:if>
        </td>
      </tr>
      <xsl:for-each select="versions/Object">
        <tr>
          <td align="right" valign="top">
            <a href="/scripts/download.pike/{id}/{name}"><xsl:value-of select="version"/></a>
          </td>
          <td valign="top">
            <a href="/scripts/download.pike/{id}/{name}"><xsl:value-of select="name"/></a>
          </td>
          <td valign="top">
            <xsl:call-template name="show_size"><xsl:with-param name="size"><xsl:value-of select="size"/></xsl:with-param></xsl:call-template>
          </td>      
          <td valign="top">
            <xsl:text disable-output-escaping="yes">&lt;script type="text/javascript"&gt;//&lt;![CDATA[
            print_transformed_date("</xsl:text><xsl:value-of select="last-modified/date"/><xsl:text disable-output-escaping="yes">");
            //]]&gt;&lt;/script&gt;</xsl:text>
          </td>
          <td valign="top">
            <a href="/scripts/navigate.pike?object={modified-by/object/id}&amp;type=details&amp;room={$room}"><xsl:value-of select="modified-by/object/name"/></a>
          </td>
          <td valign="top">
            <xsl:if test="/Object/access/writeable/@user != 'false'">
              <a href="/scripts/browser.pike?id={/Object/id}&amp;_action=restore_version&amp;restore={id}&amp;jump=versions{$this_link}">{INCLUDEMETHODS_VERSIONS_ACTION_RESTORE}</a><br />
            </xsl:if>
            <xsl:call-template name="show_diff_actions">
              <xsl:with-param name="this_link"><xsl:value-of select="$this_link"/></xsl:with-param>
            </xsl:call-template>
          </td>
        </tr>
      </xsl:for-each>
    </table>
  </xsl:template>
  
  <xsl:template name="show_diff_actions">
    <xsl:param name="this_link"/>
    <xsl:if test="starts-with(mime-type, 'text') or mime-type = 'source/pike' or mime-type = 'application/x-javascript'">
      <a href="{/Object/path/path}?jump=versions&amp;selecteddiff={id}{$this_link}">{INCLUDEMETHODS_VERSIONS_ACTION_SELECT_FOR_DIFF}</a>
      <xsl:if test="count(following-sibling::Object[1]/version) > 0">
        <xsl:variable name="lowerversion"><xsl:value-of select="following-sibling::Object[1]/version"/></xsl:variable>        
        <br />
        <a href="/scripts/navigate.pike?object={/Object/id}&amp;type=diff&amp;newversion={id}&amp;oldversion={/Object/versions/Object[version=$lowerversion]/id}&amp;jump=versions">{INCLUDEMETHODS_VERSIONS_ACTION_DIFF_PREVIOUS} <xsl:value-of select="$lowerversion"/></a>
      </xsl:if>
      <xsl:if test="$selecteddiff > 0">
        <br />
        <a href="/scripts/navigate.pike?object={/Object/id}&amp;type=diff&amp;newversion={id}&amp;oldversion={$selecteddiff}&amp;jump=versions">{INCLUDEMETHODS_VERSIONS_ACTION_DIFF_SELECTED} <xsl:value-of select="/Object/versions/Object[id=$selecteddiff]/version"/></a>
      </xsl:if>
    </xsl:if>
  </xsl:template>  
  
  
  <xsl:template name="show_diff_embedded">
    <div id="diff" style="padding-bottom: 1em;">
      <p>
        <xsl:for-each select="/Object/diffs/diff/content/line">
          <xsl:variable name="pos"><xsl:value-of select="position()"/></xsl:variable>
          <xsl:choose>
            <xsl:when test="count(/Object/diffs/diff/change[$pos = number(@line1)]) > 0">
              <div style="font-family: courier;">  
                <xsl:if test="count(/Object/diffs/diff/change[$pos = number(@line1)]/deleted/line) > 0">
                  <div style="background-color: #FFDEDE">
                    <xsl:for-each select="/Object/diffs/diff/change[$pos = number(@line1)]/deleted/line">
                      -&#160;&#160;&#160;<xsl:value-of select="translate(., ' ', '&#160;')" disable-output-escaping="no" /><br />
                    </xsl:for-each>
                  </div>
                </xsl:if>
                <xsl:if test="count(/Object/diffs/diff/change[$pos = number(@line1)]/added/line) > 0">
                  <div style="background-color: #DEFFDE">
                    <xsl:for-each select="/Object/diffs/diff/change[$pos = number(@line1)]/added/line">            
                      +&#160;&#160;&#160;<xsl:value-of select="translate(., ' ', '&#160;')" disable-output-escaping="no" /><br />
                    </xsl:for-each>
                  </div>
                </xsl:if>
              </div>
            </xsl:when>
            <xsl:otherwise>
              <xsl:if test="count(/Object/diffs/diff/change[$pos > number(@line1) and count(added/line) + number(@line1) > $pos ]) = 0">
                <div style="font-family: courier;">
                  &#160;&#160;&#160;&#160;<xsl:value-of select="translate(., ' ', '&#160;')" disable-output-escaping="no" /><br />
                </div>
              </xsl:if>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
      </p>
    </div>
  </xsl:template>
  
  <xsl:template name="show_diff">
    <div id="diff" style="padding-bottom: 1em;">
      <p>
        <xsl:for-each select="/Object/diffs/diff/content/line">
          <xsl:variable name="pos"><xsl:value-of select="position()"/></xsl:variable>
          <xsl:choose>
            <xsl:when test="count(/Object/diffs/diff/change[$pos = number(@line1)]) > 0">
              <xsl:if test="count(/Object/diffs/diff/change[$pos > number(@line1) and number(@line1) > $pos - 3]) = 0">
                <div style="padding-top: 1em; padding-left:0.5em; font-family: courier; font-weight: bold;">
                  {DIFF_CHANGE_LOCATION} <xsl:value-of select="/Object/diffs/diff/change[$pos = number(@line1)]/@line1" /> - <xsl:value-of select="/Object/diffs/diff/change[$pos = number(@line1)]/@line2"/>
                </div>
              </xsl:if>
              <xsl:variable name="padding"><xsl:choose><xsl:when test="count(/Object/diffs/diff/change[$pos > number(@line1) and number(@line1) > $pos - 3]) = 0">padding-top:0.5em;</xsl:when><xsl:otherwise></xsl:otherwise></xsl:choose></xsl:variable>
              <div style="{$padding}font-family: courier;">  
                <xsl:if test="count(/Object/diffs/diff/change[$pos = number(@line1)]/deleted/line) > 0">
                  <div style="background-color: #FFDEDE">
                    <xsl:for-each select="/Object/diffs/diff/change[$pos = number(@line1)]/deleted/line">
                      -&#160;&#160;&#160;<xsl:value-of select="translate(., ' ', '&#160;')" disable-output-escaping="no" /><br />
                    </xsl:for-each>            
                  </div>
                </xsl:if>
                <xsl:if test="count(/Object/diffs/diff/change[$pos = number(@line1)]/added/line) > 0">
                  <div style="background-color: #DEFFDE">
                    <xsl:for-each select="/Object/diffs/diff/change[$pos = number(@line1)]/added/line">            
                      +&#160;&#160;&#160;<xsl:value-of select="translate(., ' ', '&#160;')" disable-output-escaping="no" /><br />
                    </xsl:for-each>
                  </div>
                </xsl:if>
              </div>
            </xsl:when>
            <xsl:otherwise>
              <xsl:if test="( ( 1=2 and  count(/Object/diffs/diff/change[number(@line1) > $pos and $pos + 4 > number(@line1)])) or count(/Object/diffs/diff/change[$pos > number(@line1) and count(added/line) + number(@line1) > $pos - 3 ]) ) and count(/Object/diffs/diff/change[$pos > number(@line1) and count(added/line) + number(@line1) > $pos ]) = 0">
                <div style="font-family: courier;">
                  &#160;&#160;&#160;&#160;<xsl:value-of select="translate(., ' ', '&#160;')" disable-output-escaping="no" /><br />
                </div>
              </xsl:if>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
      </p>
    </div>
  </xsl:template>
  
  <xsl:template name="show_results">
    <xsl:if test="$message!='' and $message!='ok'">
      <p>
        <img class="arrowright_groups" src="/images/arrow_right_small.gif" alt=""/><strong>{GROUP_HINT}</strong>: <xsl:value-of select="$message"/>
        <!--&gt;&gt; <strong>{GROUP_HINT}</strong>: <xsl:value-of select="$message"/>-->
        <xsl:if test="normalize-space($error) != ''">
          &#8211; {GROUP_REASON}: <xsl:value-of select="$error"/>
        </xsl:if>
      </p>
    </xsl:if>
  </xsl:template>
  
</xsl:stylesheet>